#include <fstream>
#include <iomanip>
#include <string>
#include <string.h>
#include <stdio.h>

#include "NavierStokes.h"
#include "Thompson2D.h"
#include "kinCond1D.h"
#include "streamFunction2D.h"

#include "Array.h"
#include "ArrayOperators.h"
#include "GlobLoc.h"
#include "TensorCoordSys.h"
#include "Domain.h"
#include "Field.h"
#include "Filter.h"
#include "Integ.h"
#include "Constr.h"
#include "Mesh.h"
#include "param.h"
#include "paramList.h"
#include "Plot.h"
#include "Result.h"
#include "Solver.h"

using namespace std ;
using namespace keyFE2 ;
using namespace keyFlow ;



int main(void) {
  
  
  char buf[256];
  
  Double visc1=1.0;
  Double visc2=1.0;
  Double capillarity=8.4;
  Double strainRate=1.0;
  Double bubbleRadius=1.0;
  
  // Reading and renumbering of the mesh:
  
  Mesh MESH;
  MESH.read("brol.mesh","emc2Mesh");
  string sortType="frontal";
  if (sortType=="position")
  {
    Domain Dom=MESH.getDefDomain();
    Field coord=MESH.getCoordinateField(Dom);
    coord.setName("coordinates");
    Array ACoord((TYPE="arrayField",ORDER=1,DIM=2,NAME="Champ_de_coordonnes",
		  DERIV=false,COMPONENT(0)=coord.component(0),
		  COMPONENT(1)=coord.component(1)));
    Array ABrol((TYPE="array",ORDER=1,DIM=2,NAME="orient",
                 COMPONENT(0)=Double(1.0),COMPONENT(1)=Double(1.0e-4)));
    Array ADist=accumulation(ACoord,ABrol,1);
    MESH=renumberDefDomainArray(MESH,ADist);
  }
  else if (sortType=="frontal") {
    Domain Bnd=MESH.getDomain(6);
    MESH=renumberDefDomainFrontal(MESH,Bnd);
  }
  else if (sortType=="none") {
    // Nothing to do!
  }
  else {
    cerr << "Error in main :" << endl 
      << "   unknown sorting type :  " << sortType << "." << endl ;
    exit(0);
  }

  // Domains:

  Domain Dom,Dom1,Dom2,Bnd1,Bnd2,Bnd3,Bnd4,Bnd5,Bnd6,Bnd7;
  Dom=MESH.getDefDomain();
  Dom.setName("main domain");
  Dom1=MESH.getDomain(0);
  Dom1.setName("inside the bubble");
  Dom2=MESH.getDomain(1);
  Dom2.setName("outside the bubble");
  Bnd1=MESH.getDomain(2);
  Bnd1.setName("boundary 1");
  Bnd2=MESH.getDomain(3);
  Bnd2.setName("boundary 2");
  Bnd3=MESH.getDomain(4);
  Bnd3.setName("boundary 3");
  Bnd4=MESH.getDomain(5);
  Bnd4.setName("boundary 4");
  Bnd5=MESH.getDomain(6);
  Bnd5.setName("boundary 5");
  Bnd6=MESH.getDomain(7);
  Bnd6.setName("boundary 6");
  Bnd7=MESH.getDomain(8);
  Bnd7.setName("interface");
  
  Domain Corners1,Corners2;
  Corners1=Bnd3.getBoundary();
  Corners2=Bnd4.getBoundary();
  Domain cornerUR=Corners1*Corners2;
  cornerUR.setName("upper right corner");
  Corners1=Bnd2.getBoundary();
  Corners2=Bnd7.getBoundary();
  Domain attachPoint=Corners1*Corners2;
  attachPoint.setName("attachment point of free surface");
  Domain leftBnd=Bnd5+Bnd6;
  leftBnd.setName("left boundary");
  Domain lowerBnd=Bnd1+Bnd2;
  lowerBnd.setName("lower boundary");
  Domain extBnd=Bnd3+Bnd4;
  extBnd.setName("external boundary");
  Domain intBnd=leftBnd+lowerBnd;
  intBnd.setName("internal boundary");
  
  // Coordinate systems:
  string coordInterp="quadratic";
  Field coord; 
  Array SPHERE;
  {
    Field tmpCoord=MESH.getCoordinateField(Dom);
    coord=Field((DOM=Dom,INTERPOLATION=coordInterp,NBRCOMPONENTS=2,
                 COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
                 NAME="coordinates")); 
    coord.interpolate(tmpCoord,Dom,tmpCoord);
    
    // Initial sphere:
    
    Array ACoord((TYPE="arrayField",ORDER=1,DIM=2,DERIV=false,
                  COMPONENT(0)=tmpCoord.component(0),
                  COMPONENT(1)=tmpCoord.component(1),
                  NAME="ACoord"));
    Array AX((TYPE="arrayField",ORDER=0,COMPONENT()=tmpCoord.component(0),
              DERIV=false,NAME="AX"));
    Array AY((TYPE="arrayField",ORDER=0,COMPONENT()=tmpCoord.component(1),
              DERIV=false,NAME="AY"));
    Array AR((TYPE="array",ORDER=0,COMPONENT()=bubbleRadius,
              NAME="bubbleRadius"));
    SPHERE=sqrt(AR*AR*pow(AX*AX+AY*AY,-1.0))*ACoord;
  }
  coord.setName("coordinates");

  Field coord2((DOM=Dom,INTERPOLATION=coordInterp,NBRCOMPONENTS=2,
                COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
                NAME="coord2")); 
  Array ACoord2((TYPE="arrayField",ORDER=1,DIM=2,DERIV=false,
                 COMPONENT(0)=coord2.component(0),
                 COMPONENT(1)=coord2.component(1),
                 NAME="ACoord2"));
  
  // Other Fields:
  
  Field pressure1((DOM=Dom1,INTERPOLATION="linear",NBRCOMPONENTS=1,
		   COMPONENTNAME(0)="p1",NAME="pressure1"));
  Field pressure2((DOM=Dom2,INTERPOLATION="linear",NBRCOMPONENTS=1,
		   COMPONENTNAME(0)="p2",NAME="pressure2"));
  Field velocities((DOM=Dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		    COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		    NAME="velocities"));
  Field psi((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=1,
	     COMPONENTNAME(0)="psi",NAME="psi"));
  Field tangent((DOM=Bnd7,INTERPOLATION="quadratic",NBRCOMPONENTS=1,
	         COMPONENTNAME(0)="Vs",NAME="tangentVelo"));
      
  // TimeDep:
  
  TimeDep TD((NAME="transient",SCHEME="Galerkin",MAXSTEPSNBR=500,
	      DTMIN=0.000001,DTMAX=10.0,DTINIT=0.01,TINIT=0.0,EPSILON=0.001,
	      TFINAL=100.0,
	      ACTIVE=velocities,INFLU=coord,ACTIVE=pressure1,ACTIVE=pressure2));

  // Remeshing:
  
  Thompson2D Prob1((param("type","2D2D - coordinates - "),
		    param("domain",Dom),
		    param("coord",coord),
		    param("integRule","order 5"),
		    param("name","2D Thompson equations")));
  Prob1.setLayer(-100);
  
  kinCond1D Prob3((param("type","3D1D - Coord V - "),
		   param("domain",Bnd7),
		   param("coord",coord),
		   param("velo",velocities),
		   param("axisym",true),
		   param("surfaceTension",capillarity),
		   param("integRule","order 5"),
		   param("timeDep",TD),
		   param("name","All for free surface") ));

  // Various quantities from Prob1 and Prob3:
    
  Array X=Prob1.X;
  Array Y=Prob1.Y;
  Array testX=Prob1.testX;
  Array testY=Prob1.testY;
  Array refX=Prob1.refX;
  Array refY=Prob1.refY;
  
  Array ACoordXY=Prob1.coordinates;
  Array ATestCoordXY=Prob1.testCoordinates;
  TensorCoordSys TCSXY=Prob1.tcs;
  
  GlobLoc GL0D((COORD=ACoordXY,GLTYPE="0D"));
  GlobLoc GL1D=Prob3.gl;
  GlobLoc GL2D=Prob1.gl;
  TensorCoordSys TCS=Prob3.tcs;
  
  // Boundary conditions for Thompson remeshing:
  
  Constr Prob1BC1((NAME="Thompson lower boundary",TYPE="usual",DOM=lowerBnd,
		   TEST=testY,EQUATION=Y-refY,GLOBLOC=GL1D,LAYER=100));
  Constr Prob1BC2((NAME="Thompson right boundary",TYPE="usual",DOM=Bnd3,
		   TEST=testX,EQUATION=X-refX,GLOBLOC=GL1D,LAYER=100));
  Constr Prob1BC3((NAME="Thompson upper boundary",TYPE="usual",DOM=Bnd4,
		   TEST=testY,EQUATION=Y-refY,GLOBLOC=GL1D,LAYER=100));
  Constr Prob1BC4((NAME="Thompson left boundary",TYPE="usual",DOM=leftBnd,
		   TEST=testX,EQUATION=X-refX,GLOBLOC=GL1D,LAYER=100));
		   
  Constr Prob1BC5((NAME="Spherical free surface",TYPE="usual",DOM=Bnd7,
		   TEST=ATestCoordXY,EQUATION=ACoordXY-SPHERE,GLOBLOC=GL1D));
  Constr Prob1BC6((NAME="Fixing coordinates",TYPE="usual",DOM=Dom,
		    TEST=ATestCoordXY,EQUATION=ACoordXY-ACoord2,GLOBLOC=GL2D));

  // Navier-Stokes equations:
  
  NavierStokes Prob2((param("type","3D2D - V P - F GPn"),
		        param("domain",Dom1),
		        param("coord",coord),
		        param("coordMobil",true),
		        param("velo",velocities),
		        param("pressure",pressure1),
		        param("axisym",true),
		        param("integRule","order 5"),
		        param("name","2D Stokes equations Dom1"),
		        param("rho",0.0),
		        param("timeDep",TD),
		        param("viscosity",visc1) ));
  
  NavierStokes Prob5((param("type","3D2D - V P - F GPn"),
		        param("domain",Dom2),
		        param("coord",coord),
		        param("coordMobil",true),
		        param("velo",velocities),
		        param("pressure",pressure2),
		        param("axisym",true),
		        param("integRule","order 5"),
		        param("name","2D Stokes equations Dom2"),
		        param("rho",0.0),
		        param("timeDep",TD),
		        param("viscosity",visc2) ));
  
  TensorCoordSys TCS2=Prob2.tcs;
  
  // Boundary conditions for NS equations:
    
  Array Avelo=Prob2.velocities;
  Array testMvt((TYPE="arrayTest",TEST=Avelo,NAME="Equations de mouvement"));  
  
  Array U((TYPE="arrayComp",ORDER=0,NAME="u",COMPONENT()=Avelo.component(0)));
  Array V((TYPE="arrayComp",ORDER=0,NAME="v",COMPONENT()=Avelo.component(1)));
  Array testU((TYPE="arrayComp",ORDER=0,NAME="u",COMPONENT()=testMvt.component(0)));
  Array testV((TYPE="arrayComp",ORDER=0,NAME="v",COMPONENT()=testMvt.component(1)));

  Constr Prob2BC1((NAME="U=0 on left boundary",TYPE="usual",DOM=leftBnd,
		   TEST=testU,EQUATION=U,GLOBLOC=GL1D));
  Constr Prob2BC2((NAME="V=0 on lower boundary",TYPE="usual",DOM=lowerBnd,
		   TEST=testV,EQUATION=V,GLOBLOC=GL1D));
  Constr Prob2BC3((NAME="U extensional Flow",TYPE="usual",DOM=extBnd,
		   TEST=testU,EQUATION=U+(0.5*strainRate)*X,GLOBLOC=GL1D));
  Constr Prob2BC4((NAME="V extensional Flow",TYPE="usual",DOM=extBnd,
		   TEST=testV,EQUATION=V-strainRate*Y,GLOBLOC=GL1D));
  
  Array Apressure1=Prob2.pressure;
  Array Apressure2=Prob5.pressure;
  Array testDiv2((TYPE="arrayTest",TEST=Apressure2,
		  NAME="test for continuity equations"));
  Constr Prob5BC1((TYPE="usual",NAME="P=0 on a corner",DOM=cornerUR,
		   TEST=testDiv2,EQUATION=Apressure2,GLOBLOC=GL0D,
		   TENSORCOORDSYS=TCS));
  
  // Equations for stream function:
  
  streamFunction2D Prob4((param("type","3D2D - psi - V"),
			  param("axisym",true),
		          param("psi",psi),
		          param("coord",coord),
		          param("velo",velocities),
		          param("domain",Dom),
		          param("integRule","order 5"),
		          param("name","stream function equations") ));
  
  // Boundary conditions for stream function:
  
  Array Astream=Prob4.psi;
  Array testStream=test(Astream);
  Constr Prob4BC1((NAME="Psi=0 on boundary 1",TYPE="usual",DOM=intBnd,
		   TEST=testStream,EQUATION=Astream,GLOBLOC=GL1D,
		   TENSORCOORDSYS=TCSXY));
  		      
  // Solvers:		   
		   
  Solver initMesh((PROBLEM=Prob1,
                   CONSTR=Prob1BC1,CONSTR=Prob1BC2,CONSTR=Prob1BC3,
		   CONSTR=Prob1BC4,CONSTR=Prob1BC5,
		   CONVERGENCE=1.0e-4,DIVERGENCE=1.0e4,NITMAX=10));
  initMesh.NewtonRaphson();
  coord2.interpolate(ACoordXY,Dom,ACoordXY); 
		   
  Solver initFlow((CONSTR=Prob1BC6,
		   PROBLEM=Prob2,
		   CONSTR=Prob2BC1,CONSTR=Prob2BC2,CONSTR=Prob2BC3,
		   CONSTR=Prob2BC4,
		   PROBLEM=Prob5,
		   CONSTR=Prob5BC1,
		   CONVERGENCE=1.0e-4,DIVERGENCE=1.0e4,NITMAX=10));
  initFlow.NewtonRaphson();
		   
  Solver flow((PROBLEM=Prob1,
	       CONSTR=Prob1BC1,CONSTR=Prob1BC2,CONSTR=Prob1BC3,
	       CONSTR=Prob1BC4,
	       PROBLEM=Prob2,
	       CONSTR=Prob2BC1,CONSTR=Prob2BC2,CONSTR=Prob2BC3,
	       CONSTR=Prob2BC4,
	       PROBLEM=Prob3,
	       PROBLEM=Prob5,
	       CONSTR=Prob5BC1,
	       CONVERGENCE=1.0e-4,DIVERGENCE=1.0e7,NITMAX=1));
  Solver STREAM((PROBLEM=Prob4,CONSTR=Prob4BC1,
		 CONVERGENCE=1.0e-4,DIVERGENCE=1.0e4,NITMAX=1));
  
  Plot output1((DOM=Dom1,COORD=ACoordXY,RESULT=Avelo,RESULT=Apressure1,
	        RESULT=Astream));
  Plot output2((DOM=Dom2,COORD=ACoordXY,RESULT=Avelo,RESULT=Apressure2,
	        RESULT=Astream));
		
  Result Rvelo((NAME="velocities",ARRAY=Avelo,DOM=Dom,
                INTERPOLATION="linear"));  
  Result Rpressure1((NAME="pressure1",ARRAY=Apressure1,DOM=Dom1,
                     INTERPOLATION="linear"));       
  Result Rpressure2((NAME="pressure2",ARRAY=Apressure2,DOM=Dom2,
                     INTERPOLATION="linear"));       
  Result Rstream((NAME="stream",ARRAY=Astream,DOM=Dom,
                  INTERPOLATION="linear")); 
  Plot outputDX((DOM=Dom,COORD=ACoordXY,TYPE="openDX",
                 RESULT=Rvelo,RESULT=Rpressure1,
	         RESULT=Rpressure2,RESULT=Rstream));
  
//  output1.exec("bubble_dom1");
//  output2.exec("bubble_dom2");
  outputDX.exec("bubble");

  // Time-dependent solution:
  
  string filename;
  {
    TD.addInflu(flow);  
    while(TD.isToBeContinued()) {
//flow.setZero(1.0e-100);
      flow.NewtonRaphson();
      if (TD.getCurrentStepIndex()<=1||TD.getCurrentStepIndex()%5==0) {
        STREAM.NewtonRaphson();
	
//        sprintf(buf,"%d",TD.getCurrentStepIndex());
//        filename="bubble_dom1_";
//        filename+=buf;
//        output1.exec(filename);
    
//        sprintf(buf,"%d",TD.getCurrentStepIndex());
//        filename="bubble_dom2_";
//        filename+=buf;
//        output2.exec(filename);
	
        sprintf(buf,"%d",TD.getCurrentStepIndex());
        filename="bubble_";
     	filename+=buf;
     	outputDX.exec(filename);
      
      }
    }
  }

}
  
  
  
  
  
  
  
  
