#include <fstream>
#include <iomanip>
#include <string>
#include <string.h>
#include <stdio.h>

//#include "NavierStokes1D.h"
//#include "NavierStokes.h"
#include "NavierStokes.h"
#include "streamFunction2D.h"

#include "Array.h"
#include "ArrayOperators.h"
#include "GlobLoc.h"
#include "TensorCoordSys.h"
#include "Domain.h"
#include "Field.h"
#include "Integ.h"
#include "Constr.h"
#include "Mesh.h"
#include "param.h"
#include "paramList.h"
#include "Plot.h"
#include "Solver.h"

using namespace std ;
using namespace keyFE2 ;
using namespace keyFlow ;



int main(void) {
  char buf[256];
  int i;
  
  // Reading and renumbering of the mesh:
  
  Mesh MESH;
  MESH.read("squareStep.msh","rsMesh");
  string sortType="position";
  if (sortType=="position")
    {
      Domain Dom=MESH.getDomain(0);
      Field coord=MESH.getCoordinateField(Dom);
      coord.setName("coordinates");
      Array ACoord((TYPE="arrayField",ORDER=1,DIM=2,NAME="Champ_de_coordonnes",
		    DERIV=false,COMPONENT(0)=coord.component(0),
		    COMPONENT(1)=coord.component(1)));
      Array ABrol((TYPE="array",ORDER=1,DIM=2,NAME="orient",COMPONENT(0)=1.0,
		   COMPONENT(1)=1.0e-4));
      Array ADist=accumulation(ACoord,ABrol,1);
      MESH=renumberDefDomainArray(MESH,ADist);
    }
  else if (sortType=="frontal") {
    Domain Bnd1=MESH.getDomain(1);
    MESH=renumberDefDomainFrontal(MESH,-Bnd1);
  }
  else if (sortType=="none") {
    // Nothing to do!
  }
  else {
    cerr << "Error in main :" << endl 
      << "   unknown sorting type :  " << sortType << "." << endl ;
    exit(0);
  }
  MESH.write("squareStep.msh2","rsMesh");
  
  // Domains:
  
  Domain Dom,Bnd1,Bnd2,Bnd3,Bnd4;
  Dom=MESH.getDomain(0);
  Dom.setName("main domain");
  Bnd1=MESH.getDomain(1);
  Bnd1.setName("left boundary");
  Bnd2=MESH.getDomain(2);
  Bnd2.setName("lower boundary");
  Bnd3=MESH.getDomain(3);
  Bnd3.setName("right boundary");
  Bnd4=MESH.getDomain(4);
  Bnd4.setName("upper boundary");
  
  // Fields:
  
  Field coord=MESH.getCoordinateField(Dom);
  coord.setName("coordinates Field");
  Field pressure((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=1,
		  COMPONENTNAME(0)="p",NAME="pressure"));
  Field velocities((DOM=Dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		    COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		    NAME="velocities"));
  
  // Navier-Stokes equations:
  
  NavierStokes Prob1((param("type","2D2D - V P - F GPn"),
	              param("domain",Dom),
	              param("coord",coord),
	              param("velo",velocities),
	              param("pressure",pressure),
	              param("integRule","order 5"),
	              param("name","2D Navier-Stokes equations"),
	              param("viscosity",1.0),
	              param("rho",0.0) ));

  Field gpn((DOM=Bnd1,INTERPOLATION="constantOnDomain",NBRCOMPONENTS=1,
             COMPONENTNAME(0)="gpn",NAME="normal gradient field"));
  Array Q((TYPE="array",ORDER=0,COMPONENT()=-1.0,NAME="Q"));
  NavierStokes Prob3((param("type","2D1D - V P GPn - F Q"),
	              param("domain",Bnd1),
	              param("coord",coord),
	              param("velo",velocities),
	              param("pressure",pressure),
	              param("gradPn",gpn),
	              param("flowRate",Q),
	              param("integRule","order 5"),
	              param("name","2D Navier-Stokes equations"),
	              param("viscosity",1.0),
	              param("rho",0.0) ));
  Prob3.setLayer(100);
  
  Array ACoord=Prob1.coordinates;
  TensorCoordSys TCS=Prob1.tcs;
  GlobLoc GL=Prob1.gl;
  
  GlobLoc GL2((COORD=ACoord,GLTYPE="2D1D"));
  Integ INTG((GLOBLOC=GL,TENSORCOORDSYS=TCS,RULE=3));
  Integ INTG2((GLOBLOC=GL2,TENSORCOORDSYS=TCS,RULE=3));  
  
  GlobLoc::setDefaultGL(GL);
  
  // Everything for Navier-Stokes equations:
  
  Array Avelo=Prob1.velocities;
  Array Apressure=Prob1.pressure;
  
  Array testMvt((TYPE="arrayTest",TEST=Avelo,
	         NAME="test for momentum equations")); 
    
  // Other boundary conditions :
  
  Array AveloZero((TYPE="array",ORDER=1,DIM=2,NAME="vitessesZero",
		   TENSORCOORDSYS=TCS));
  Constr Prob1BC1((NAME="V=0 lower boundary",TYPE="usual",DOM=Bnd2,
		   TEST=testMvt,EQUATION=Avelo-AveloZero,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
  Constr Prob1BC2((NAME="V=0 upper boundary",TYPE="usual",DOM=Bnd4,
		   TEST=testMvt,EQUATION=Avelo-AveloZero,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
  
  // Equations for stream function:
  
  Field psi((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=1,
	     COMPONENTNAME(0)="psi",NAME="psi"));
  streamFunction2D Prob2((param("type","2D2D - psi - V"),
		          param("psi",psi),
		          param("coord",coord),
		          param("velo",velocities),
		          param("domain",Dom),
		          param("integRule","order 5"),
		          param("name","stream function equations") ));
  
  Array Astream=Prob2.psi;
  Array testStream=test(Astream);
  Field streamBC((DOM=Bnd4,INTERPOLATION="constantOnDomain",
		  NBRCOMPONENTS=1,COMPONENTNAME(0)="psi",
		  NAME="value of stream function for BC"));
  Array AstreamBC((TYPE="arrayField",ORDER=0,NAME="streamBC",
		   TENSORCOORDSYS=TCS,COMPONENT()=streamBC.component(0)));
  
  Constr Prob2BC1((NAME="Psi=0 lower boundary",TYPE="usual",DOM=Bnd2,
		   TEST=testStream,EQUATION=Astream,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
  Constr Prob2BC2((NAME="Psi upper boundary condition",TYPE="usual",
		   DOM=Bnd4,TEST=testStream,EQUATION=Astream-AstreamBC,
		   GLOBLOC=GL2,TENSORCOORDSYS=TCS));
  
  // Resolution:
  
  Solver NS((PROBLEM=Prob1,CONSTR=Prob1BC1,CONSTR=Prob1BC2,
             PROBLEM=Prob3,
	     CONVERGENCE=1.0e-4,DIVERGENCE=1.0e4,NITMAX=6));
  Solver STREAM((PROBLEM=Prob2,CONSTR=Prob2BC1,CONSTR=Prob2BC2,
		 CONVERGENCE=1.0e-4,DIVERGENCE=1.0e4,NITMAX=1));
  
  Plot output((DOM=Dom,COORD=ACoord,RESULT=Avelo,RESULT=Apressure,
	       RESULT=Astream));
  
  for (i=0;i<=10;i++) {
    Prob1.rho.component()=20.0*i;
    Prob3.rho.component()=20.0*i;
    
    cerr << "New continuation step with rho = " << 20.0*i << endl ;
    
    NS.NewtonRaphson();
    
    STREAM.NewtonRaphson();
    streamBC.writeValues(cerr);
    
    sprintf(buf,"%d",i);
    string filename="squareStep";
    filename+=buf;
    output.exec(filename);
  }
}


