#include <fstream>
#include <iomanip>
#include <string>
#include <string.h>
#include <stdio.h>

#include "NavierStokes.h"
#include "BirdCarreau.h"
#include "streamFunction2D.h"

#include "Array.h"
#include "ArrayOperators.h"
#include "GlobLoc.h"
#include "TensorCoordSys.h"
#include "Domain.h"
#include "Field.h"
#include "Integ.h"
#include "Constr.h"
#include "Mesh.h"
#include "param.h"
#include "paramList.h"
#include "Plot.h"
#include "Solver.h"

using namespace keyFE2 ;
using namespace keyFlow ;
using namespace std ;



int main(void) {
  char buf[256];
  Int i;
  
  // Reading and renumbering of the mesh:
  
  Mesh MESH;
  MESH.read("suspR.mesh","emc2Mesh");
  string sortType="frontal";
  if (sortType=="position")
    {
      Domain Dom=MESH.getDomain(0);
      Field coord=MESH.getCoordinateField(Dom);
      coord.setName("coordinates");
      Array ACoord((TYPE="arrayField",ORDER=1,DIM=2,NAME="Champ_de_coordonnes",
		    DERIV=false,COMPONENT(0)=coord.component(0),
		    COMPONENT(1)=coord.component(1)));
      Array ABrol((TYPE="array",ORDER=1,DIM=2,NAME="orient",
		   COMPONENT(0)=Double(1.0),COMPONENT(1)=Double(1.0e-4)));
      Array ADist=accumulation(ACoord,ABrol,1);
      MESH=renumberDefDomainArray(MESH,ADist);
    }
  else if (sortType=="frontal") {
    Domain Bnd1=MESH.getDomain(1);
    MESH=renumberDefDomainFrontal(MESH,Bnd1);
  }
  else if (sortType=="none") {
    // Nothing to do!
  }
  else {
    cerr << "Error in main :" << endl 
      << "   unknown sorting type :  " << sortType << "." << endl ;
    exit(0);
  }
  
  // Domains:
  
  Domain Dom,Bnd1,Bnd2,Bnd3,Bnd4,Bnd5,Bnd6,Bnd7,Corner,BndTop;
  Dom=MESH.getDomain(0);
  Dom.setName("main domain");
  Bnd1=MESH.getDomain(1);
  Bnd1.setName("lower right boundary");
  Bnd2=MESH.getDomain(2);
  Bnd2.setName("right boundary");
  Bnd3=MESH.getDomain(3);
  Bnd3.setName("upper right boundary");
  Bnd4=MESH.getDomain(4);
  Bnd4.setName("upper left boundary");
  Bnd5=MESH.getDomain(5);
  Bnd5.setName("left boundary");
  Bnd6=MESH.getDomain(6);
  Bnd6.setName("lower left boundary");
  Bnd7=MESH.getDomain(7);
  Bnd7.setName("circle boundary");
  {
    Domain Corners1,Corners2;
    Corners1=Bnd1.getBoundary();
    Corners2=Bnd2.getBoundary();
    Corner=Corners1*Corners2;
  }
  BndTop=Bnd3+Bnd4;
  BndTop.setName("upper boundary");
  
  // Fields:
  
  Field coord=MESH.getCoordinateField(Dom);
  coord.setName("coordinates Field");
  Field pressure((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=1,
		  COMPONENTNAME(0)="p",NAME="pressure"));
  Field velocities((DOM=Dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		    COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		    NAME="velocities"));
  
  // Navier-Stokes equations:
  
  NavierStokes Prob1((param("type","2D2D - V P - F GPn"),
		        param("domain",Dom),
		        param("coord",coord),
		        param("velo",velocities),
		        param("pressure",pressure),
		        param("integRule","order 5"),
		        param("name","2D Navier-Stokes equations"),
		        param("viscosity",10.0),
		        param("rho",0.0) ));
  
  Array ACoord=Prob1.coordinates;
  TensorCoordSys TCS=Prob1.tcs;
  GlobLoc GL=Prob1.gl;
  
  GlobLoc GL0((COORD=ACoord,GLTYPE="0D"));
  GlobLoc GL2((COORD=ACoord,GLTYPE="2D1D"));
  Integ INTG((GLOBLOC=GL,TENSORCOORDSYS=TCS,RULE=3));
  Integ INTG2((GLOBLOC=GL2,TENSORCOORDSYS=TCS,RULE=3));  
  
  GlobLoc::setDefaultGL(GL);
  
  // Everything for Navier-Stokes equations:
  
  Array Avelo=Prob1.velocities;
  Array Apressure=Prob1.pressure;
  
  Array testMvt((TYPE="arrayTest",TEST=Avelo,
	         NAME="test for momentum equations")); 
  Array testDiv((TYPE="arrayTest",TEST=Apressure,
	         NAME="test for continuity equations")); 
  Array rho=Prob1.rho;
  
  // Periodicity and other boundary conditions:
  
  Field angularVelo((DOM=Bnd7,INTERPOLATION="constantOnDomain",
		     NBRCOMPONENTS=1,COMPONENTNAME(0)="omega",
		     NAME="angular velocity for BC"));
  Array AangularVelo((TYPE="arrayField",ORDER=0,NAME="AangularVelo",
		      TENSORCOORDSYS=TCS,
		      COMPONENT()=angularVelo.component(0)));
  Array X((TYPE="arrayComp",ORDER=0,DIM=1,NAME="coord X",
	   COMPONENT()=ACoord.component(0),TENSORCOORDSYS=TCS));
  Array Y((TYPE="arrayComp",ORDER=0,DIM=1,NAME="coord Y",
	   COMPONENT()=ACoord.component(1),TENSORCOORDSYS=TCS));
  Array mX=-1.0*X;
  Array Atangent((TYPE="arrayComp",ORDER=1,DIM=2,NAME="vecteur tangent au cercle",
		  COMPONENT(0)=Y.component(),COMPONENT(1)=mX.component(),
		  TENSORCOORDSYS=TCS));
  Constr Prob1BC1((NAME="V=0 on circle",TYPE="usual",DOM=Bnd7,TEST=testMvt,
		   EQUATION=Avelo-Atangent*AangularVelo,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
  
  Array AveloOne((TYPE="array",ORDER=1,DIM=2,NAME="vitessesUnit",
		  TENSORCOORDSYS=TCS,COMPONENT(0)=Double(1.0)));
  Constr Prob1BC2((NAME="periodicity 3-4 on V",TYPE="periodic",LEFTDOM=Bnd4,
		   LEFTTEST=testMvt,LEFTEQUATION=Avelo-AveloOne,RIGHTDOM=-Bnd3,
		   RIGHTTEST=testMvt,RIGHTEQUATION=Avelo-AveloOne,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
  
  Constr Prob1BC3((NAME="periodicity 2-5 on V",TYPE="periodic",LEFTDOM=Bnd5,
		   LEFTTEST=testMvt,LEFTEQUATION=Avelo,RIGHTDOM=-Bnd2,
		   RIGHTTEST=testMvt,RIGHTEQUATION=-Avelo,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
  
  Constr Prob1BC4((NAME="periodicity 1-6 on V",TYPE="periodic",LEFTDOM=Bnd6,
		   LEFTTEST=testMvt,LEFTEQUATION=Avelo,RIGHTDOM=-Bnd1,
		   RIGHTTEST=testMvt,RIGHTEQUATION=Avelo,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
  
  Constr Prob1BC5((NAME="periodicity 3-4 on P",TYPE="periodic",LEFTDOM=Bnd4,
		   LEFTTEST=testDiv,LEFTEQUATION=Apressure,RIGHTDOM=-Bnd3,
		   RIGHTTEST=testDiv,RIGHTEQUATION=-Apressure,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
  
  Constr Prob1BC6((NAME="periodicity 2-5 on P",TYPE="periodic",LEFTDOM=Bnd5,
		   LEFTTEST=testDiv,LEFTEQUATION=Apressure,RIGHTDOM=-Bnd2,
		   RIGHTTEST=testDiv,RIGHTEQUATION=-Apressure,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
  
  Constr Prob1BC7((NAME="periodicity 1-6 on P",TYPE="periodic",LEFTDOM=Bnd6,
		   LEFTTEST=testDiv,LEFTEQUATION=Apressure,RIGHTDOM=-Bnd1,
		   RIGHTTEST=testDiv,RIGHTEQUATION=-Apressure,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
  
  Array ApressureZero((TYPE="array",ORDER=0,NAME="pressureZero",
		       TENSORCOORDSYS=TCS,COMPONENT()=Double(0.0)));
  Constr Prob1BC8((NAME="P=0 on a corner",TYPE="usual",DOM=Corner,
		   TEST=testDiv,EQUATION=Apressure-ApressureZero,GLOBLOC=GL0,
		   TENSORCOORDSYS=TCS));
  
  
  
  // Equations for stream function:
  
  Field psi((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=1,
	     COMPONENTNAME(0)="psi",NAME="psi"));
  streamFunction2D Prob2((param("type","2D2D - psi - V"),
		          param("psi",psi),
		          param("coord",coord),
		          param("velo",velocities),
		          param("domain",Dom),
		          param("integRule","order 5"),
		          param("name","stream function equations") ));
  
  Array Astream=Prob2.psi;
  Array testStream=test(Astream);
  Field streamBC((DOM=Bnd4,INTERPOLATION="constantOnDomain",
		  NBRCOMPONENTS=1,COMPONENTNAME(0)="psi",
		  NAME="value of stream function for BC"));
  Array AstreamBC((TYPE="arrayField",ORDER=0,NAME="streamBC",
		   TENSORCOORDSYS=TCS,COMPONENT()=streamBC.component(0)));
  
  Array APsiZero((TYPE="array",ORDER=0,NAME="psi = 0",TENSORCOORDSYS=TCS));
  Constr Prob2BC1((NAME="Psi=0 lower boundary",TYPE="usual",DOM=Bnd7,
		   TEST=testStream,EQUATION=Astream,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
  
  // Resolution:
  
  Solver NS((PROBLEM=Prob1,
	     CONSTR=Prob1BC1,CONSTR=Prob1BC2,CONSTR=Prob1BC3,CONSTR=Prob1BC4,
	     CONSTR=Prob1BC5,CONSTR=Prob1BC6,CONSTR=Prob1BC7,CONSTR=Prob1BC8,
	     CONVERGENCE=Double(1.0e-4),DIVERGENCE=Double(1.0e4),NITMAX=6));
  Solver STREAM((PROBLEM=Prob2,CONSTR=Prob2BC1,
		 CONVERGENCE=Double(1.0e-4),DIVERGENCE=Double(1.0e4),
		 NITMAX=1));
  
  Plot output((DOM=Dom,COORD=ACoord,RESULT=Avelo,RESULT=Apressure,
	       RESULT=Astream));
  
  for (i=0;i<=5;i++) {
    rho.component()=200.0*i;
    
    cerr << "New continuation step with rho = " << 200.0*i << endl ;
    
    NS.NewtonRaphson();
    STREAM.NewtonRaphson();
    
    sprintf(buf,"%d",i);
    string filename="susp";
    filename+=buf;
    output.exec(filename);
  }
}


