#ifndef BEAMSECTION_H
#define BEAMSECTION_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include "Array.h"
#include "GlobLoc.h"
#include "Domain.h"
#include "Mesh.h"
#include "Integ.h"
#include "TensorCoordSys.h"

using namespace keyFE2 ;

class beamSection {
  
  private:
    
    Double E ;
    Double G ;
  
  private:
    
    Mesh MESH;
    Domain Dom,Bnd,Point ; // Domains.
  
    Array ACoord1 ; // Basic coordinates.
    Array CoG ; // Geometric Center of the section (in basic coordinates).
    Array A ; // Area of the section.
  
    Array ACoord2 ; // Centered coordinates.
    Double Ipyy,Ipzz,Ipyz ;
  
    Array ACoord3 ; // Principal coordinates.
    Array Iy,Iz ; // Inertias in principal coordinates.
  
    Array APhi ; // Torsional skewing function.
    Array ToC ; // Center for torsion.
    Array J ; // Inertia for torsion.
  
    Array Af ; // y shear skewing function.
    Array ky ; // Reducted shear factor in direction y.
    Array Ag ; // z shear skewing function.
    Array kz ; // Reducted shear factor in direction z.
  
    Double dYc,dZc;
    Double Iyp,Izp,Iyzp;
    Double alfa ;
    Double dA ;
    Double ToCy,ToCz;
    Double dIy,dIz,dIyz,dJ,dCw,dCwPat;
    Double dky,dkz;
  
  public :
    
    Array Nx,Ty,Tz ; // Forces.
    Array Mt,Mfy,Mfz ; // Moments.
  
    Array Sxx,Sxy,Sxz,Sshear,SVM; // Stresses.
  
  // Constructor:
  
    beamSection(paramList PL) ;
    ~beamSection(void) ;
  
    void plotResults(std::string filename,
		     Double Fx,Double Fy,Double Fz,
		     Double Mx,Double My,Double Mz);
  
} ;

#endif
