
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <fstream>
#include <cmath>
#include <iomanip>
#include <string>
#include <string.h>
#include <stdio.h>

#include "Array.h"
#include "ArrayOperators.h"
#include "GlobLoc.h"
#include "TensorCoordSys.h"
#include "Domain.h"
#include "Field.h"
#include "Integ.h"
#include "Constr.h"
#include "Mesh.h"
#include "param.h"
#include "paramList.h"
#include "Plot.h"
#include "Solver.h"
#include "TimeDep.h"



using namespace std ;
using namespace keyFE2 ;

int main(void) {
  char buf[256];

  Mesh MESH;
  
  // Lecture et renumrotation du maillage :
  
  MESH.read("planeJet.msh","rsMesh");
  string sortType="frontal";
  if (sortType=="position")
  {
    Domain Dom=MESH.getDomain(0);
    Field coord=MESH.getCoordinateField(Dom);
    coord.setName("coordinates");
    Array ACoord((TYPE="arrayField",ORDER=1,DIM=2,NAME="Champ de coordonnes",
		  DERIV=false,COMPONENT(0)=coord.component(0),
		  COMPONENT(1)=coord.component(1)));
    Array ABrol((TYPE="array",ORDER=1,DIM=2,NAME="orient",COMPONENT(0)=1.0,
		 COMPONENT(1)=1.0e-4));
    Array ADist=accumulation(ACoord,ABrol,1);
    MESH=renumberDefDomainArray(MESH,ADist);
  }
  else if (sortType=="frontal") {
    Domain Bnd4=MESH.getDomain(4);
    Domain Bnd5=MESH.getDomain(5);
    Domain leftBnd=Bnd4+Bnd5;
    MESH=renumberDefDomainFrontal(MESH,leftBnd);
  }
  else if (sortType=="none") {
    // Rien  faire !
  }
  else {
    cerr << "Error in main :" << endl 
      << "   unknown sorting type :  " << sortType << "." << endl ;
    exit(0);
  }
  MESH.write("planeJet.msh2","rsMesh");
  
  // Diffrents domaines :
  
  Domain Dom,Bnd1,Bnd2,Bnd3,Bnd4,Bnd5;
  Dom=MESH.getDomain(0);
  Dom.setName("main domain");
  Bnd1=MESH.getDomain(1);
  Bnd1.setName("lower boundary");
  Bnd2=MESH.getDomain(2);
  Bnd2.setName("right boundary");
  Bnd3=MESH.getDomain(3);
  Bnd3.setName("upper boundary");
  Bnd4=MESH.getDomain(4);
  Bnd4.setName("left wall boundary");
  Bnd5=MESH.getDomain(5);
  Bnd5.setName("inflow boundary");
  
  // Tout ce qui concerne les coordonnes et intgrations :
  
  Field coord=MESH.getCoordinateField(Dom);
  coord.setName("coordinates");
  Array ACoord((TYPE="arrayField",ORDER=1,DIM=2,NAME="Champ de coordonnes",
		DERIV=false,COMPONENT(0)=coord.component(0),
		COMPONENT(1)=coord.component(1)));

  TensorCoordSys TCS((TCSTYPE="Cartesian2D",COORD=ACoord));

  GlobLoc GL((COORD=ACoord,GLTYPE="2D2D"));
  GlobLoc GL2((COORD=ACoord,GLTYPE="2D1D"));

  Integ INTG((GLOBLOC=GL,TENSORCOORDSYS=TCS,RULE=3));
  Integ INTG2((GLOBLOC=GL2,TENSORCOORDSYS=TCS,RULE=3));  
  
  GlobLoc::setDefaultGL(GL);

  // Tout ce qui concerne les quations de Navier-Stokes :

  Field pressure((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=1,
		  COMPONENTNAME(0)="p",NAME="pressure"));
  Field velocities((DOM=Dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		    COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		    NAME="velocities"));
  Array Avelo((TYPE="arrayField",ORDER=1,DIM=2,NAME="vitesses",
	       TENSORCOORDSYS=TCS,COMPONENT(0)=velocities.component(0),
	       COMPONENT(1)=velocities.component(1)));
  Array Apressure((TYPE="arrayField",ORDER=0,NAME="pression",
		   TENSORCOORDSYS=TCS,COMPONENT()=pressure.component(0)));
  
  Array testMvt((TYPE="arrayTest",TEST=Avelo,NAME="Equations de mouvement"));  
  Array testDiv((TYPE="arrayTest",TEST=Apressure,
		 NAME="Equations de continuit"));
  
  Array Unit((TYPE="array",ORDER=2,DIM=2,COMPONENT(0,0)=1.0,
	      COMPONENT(1,1)=1.0,NAME="Unit tensor",TENSORCOORDSYS=TCS));
  Array rho((TYPE="array",ORDER=0,NAME="density",TENSORCOORDSYS=TCS,
	     COMPONENT()=500.0));
  
  // TimeDep :
  
  TimeDep TD((NAME="transient",SCHEME="Galerkin",MAXSTEPSNBR=700,
	      DTMIN=0.000001,DTMAX=1.0,DTINIT=0.01,TINIT=0.0,EPSILON=0.001,
	      TFINAL=20.0,INFLU=velocities,ACTIVE=pressure));
  
  // Navier-Stokes equations :

  Array gradVelo=grad(Avelo);
  gradVelo.setName("Champ gradients de vitesses");
  Array Convect=rho*(timeDeriv(Avelo,TD)+
		     contractedMultiplication(Avelo,0,gradVelo,1));
  Convect.setName("Termes convectifs");
  Array gradMvt=grad(testMvt);
  gradMvt.setName("gradMvt");
  
  Array divV=contract(gradVelo);
  divV.setName("divergence des vitesses");
  
  Array D=0.5*(gradVelo+transpose(gradVelo));
  D.setName("Taux de dformation");
  Array T=D-Apressure*Unit;
  T.setName("Tenseur des tensions de Cauchy");
  Array NavierStokes=accumulation(gradMvt,T)+accumulation(testMvt,Convect)
    +testDiv*divV;
  NavierStokes.setName("Equations de Navier-Stokes");
  
  Array Prob1=integrate(NavierStokes,Dom,INTG);
  
  // L'inflow est calcul explicitement :
  
  Array AY((TYPE="arrayComp",ORDER=0,NAME="Coord Y",
	    COMPONENT()=ACoord.component(1)));
  Array A0((TYPE="array",ORDER=0,NAME="0.03125",COMPONENT()=0.03125));
  Array A1((TYPE="array",ORDER=0,NAME="1.0",COMPONENT()=1.0));
  Array AVmax((TYPE="array",ORDER=0,NAME="AVmax",COMPONENT()=1.0));
  Array uIn;
  uIn=AVmax*(A1-(1.0/0.03125/0.03125)*pow(AY,2.0));
//  uIn=(A1-(1.0/0.03125/0.03125)*pow(AY,2.0));
  Array AveloIn((TYPE="arrayComp",ORDER=1,DIM=2,NAME="vitesses_inflow",
		 TENSORCOORDSYS=TCS,COMPONENT(0)=uIn.component()));
  Constr Prob1CF1((NAME="inflow",TYPE="usual",DOM=Bnd5,TEST=testMvt,
		   EQUATION=Avelo-AveloIn,GLOBLOC=GL2,TENSORCOORDSYS=TCS));
  
  // Autres conditions aux frontires :
  
  Array AveloY((TYPE="arrayField",ORDER=0,NAME="Vy",TENSORCOORDSYS=TCS,
	        COMPONENT(0)=velocities.component(1)));
  Array AveloY0((TYPE="array",ORDER=0,NAME="Vy=0",TENSORCOORDSYS=TCS,
		COMPONENT(0)=0.0));
  Array testMvtY((TYPE="arrayTest",TEST=AveloY,
		  NAME="Equations de mouvement Y"));  
  Constr Prob1CF2((NAME="Vy=0 lower boundary",TYPE="usual",DOM=Bnd1,
		   TEST=testMvtY,EQUATION=AveloY-AveloY0,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
  Array AveloZero((TYPE="array",ORDER=1,DIM=2,NAME="vitessesZero",
		   TENSORCOORDSYS=TCS));  
  Constr Prob1CF3((NAME="V=0 left boundary",TYPE="usual",DOM=Bnd4,
		   TEST=testMvt,EQUATION=Avelo-AveloZero,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));

  // Calcul de la fonction de Courant :
  
  Field courant((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=1,
		 COMPONENTNAME(0)="psi",NAME="courant"));
  Field courantBC((DOM=Bnd4,INTERPOLATION="constantOnDomain",
		   NBRCOMPONENTS=1,COMPONENTNAME(0)="psi",
		   NAME="courant for BC"));
  Array Acourant((TYPE="arrayField",ORDER=0,NAME="courant",TENSORCOORDSYS=TCS,
		  COMPONENT()=courant.component(0)));
  Array AcourantBC((TYPE="arrayField",ORDER=0,NAME="courantBC",
		    TENSORCOORDSYS=TCS,COMPONENT()=courantBC.component(0)));
  Array Avelo2((TYPE="arrayField",ORDER=1,DIM=2,NAME="vitesses",DERIV=false,
	        TENSORCOORDSYS=TCS,COMPONENT(0)=velocities.component(0),
	        COMPONENT(1)=velocities.component(1)));
     // (meme chose que Avelo, mais on ne drive pas par rapport aux valeurs
     //  nodales.)
  Array U((TYPE="arrayComp",ORDER=0,NAME="u",
	   COMPONENT()=Avelo2.component(0)));
  Array V((TYPE="arrayComp",ORDER=0,NAME="v",
	   COMPONENT()=Avelo2.component(1)));
  Array mV=-1.0*V;
  Array tmpVelo((TYPE="arrayComp",ORDER=1,DIM=2,NAME="special",
		 TENSORCOORDSYS=TCS,COMPONENT(0)=mV.component(),
		 COMPONENT(1)=U.component()));
  Array gradPsi=grad(Acourant);
  Array testCourant((TYPE="arrayTest",TEST=Acourant,
		     NAME="Equations de courant"));  
  Array gradCourant=grad(testCourant);
  Array eqCourant=gradPsi-tmpVelo;
  Array equationCourant=accumulation(gradCourant,eqCourant);
  Array Prob2=integrate(equationCourant,Dom,INTG);
  
  Constr Prob2CF1((NAME="Psi=0 lower boundary",TYPE="usual",DOM=Bnd1,
		   TEST=testCourant,EQUATION=Acourant,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
  Constr Prob2CF2((NAME="Psi upper boundary condition",TYPE="usual",
		   DOM=Bnd4,TEST=testCourant,EQUATION=Acourant-AcourantBC,
		   GLOBLOC=GL2,TENSORCOORDSYS=TCS));
  
  // Rsolution :
  Solver NS((PROBLEM=Prob1,CONSTR=Prob1CF1,CONSTR=Prob1CF2,
    CONSTR=Prob1CF3,CONVERGENCE=1.0e-4,DIVERGENCE=1.0e4,NITMAX=1));
  Solver STREAM((PROBLEM=Prob2,CONSTR=Prob2CF1,CONSTR=Prob2CF2,
		 CONVERGENCE=1.0e-4,DIVERGENCE=1.0e4,NITMAX=1));
  
  Plot output((DOM=Dom,COORD=ACoord,RESULT=Avelo,RESULT=Apressure,
	       RESULT=Acourant));
  
  TD.addInflu(NS);    
  while(TD.isToBeContinued()) {
    // Dbit :
    cerr << 1.0-1.0*(0.5+0.5*tanh((TD.getCurrentTime()-4.0)/0.25)) << endl ;
    AVmax.component()=1.0-1.0*(0.5+0.5*tanh((TD.getCurrentTime()-4.0)/0.25));
    NS.NewtonRaphson();
    if (TD.getCurrentStepIndex()==1
	||Int(TD.getCurrentTime()/0.5)!=Int(TD.getPreviousTime()/0.5)) {
      sprintf(buf,"%d",TD.getCurrentStepIndex());
      string filename="planeJet";
      filename+=buf;
      STREAM.NewtonRaphson();
      courantBC.writeValues(cerr);
      output.exec(filename);
    }
  }
}


