
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <fstream>
#include <iomanip>
#include <string>
#include <string.h>
#include <stdio.h>

#include "Array.h"
#include "ArrayOperators.h"
#include "GlobLoc.h"
#include "TensorCoordSys.h"
#include "Domain.h"
#include "Field.h"
#include "Integ.h"
#include "Constr.h"
#include "Mesh.h"
#include "param.h"
#include "paramList.h"
#include "Plot.h"
#include "Solver.h"

using namespace std ;
using namespace keyFE2 ;



int main(void) {
  char buf[256];
  int i;

  Mesh MESH;
  
  // Lecture et renumrotation du maillage :
  
  MESH.read("squareStep.msh","rsMesh");
  string sortType="position";
  if (sortType=="position")
  {
    Domain Dom=MESH.getDomain(0);
    Field coord=MESH.getCoordinateField(Dom);
    coord.setName("coordinates");
    Array ACoord((TYPE="arrayField",ORDER=1,DIM=2,NAME="Champ de coordonnes",
		  DERIV=false,COMPONENT(0)=coord.component(0),
		  COMPONENT(1)=coord.component(1)));
    Array ABrol((TYPE="array",ORDER=1,DIM=2,NAME="orient",COMPONENT(0)=1.0,
		 COMPONENT(1)=1.0e-4));
    Array ADist=accumulation(ACoord,ABrol,1);
    MESH=renumberDefDomainArray(MESH,ADist);
  }
  else if (sortType=="frontal") {
    Domain Bnd1=MESH.getDomain(1);
    MESH=renumberDefDomainFrontal(MESH,-Bnd1);
  }
  else if (sortType=="none") {
    // Rien  faire !
  }
  else {
    cerr << "Error in main :" << endl 
      << "   unknown sorting type :  " << sortType << "." << endl ;
    exit(0);
  }
  MESH.write("squareStep.msh2","rsMesh");
  
  // Diffrents domaines :
  
  Domain Dom,Bnd1,Bnd2,Bnd3,Bnd4,Corner;
  Dom=MESH.getDomain(0);
  Dom.setName("main domain");
  Bnd1=MESH.getDomain(1);
  Bnd1.setName("left boundary");
  Bnd2=MESH.getDomain(2);
  Bnd2.setName("lower boundary");
  Bnd3=MESH.getDomain(3);
  Bnd3.setName("right boundary");
  Bnd4=MESH.getDomain(4);
  Bnd4.setName("upper boundary");
  {
    Domain Corners1,Corners2;
    Corners1=Bnd1.getBoundary();
    Corners2=Bnd4.getBoundary();
    Corner=Corners1*Corners2;
  }
  
  // Tout ce qui concerne les coordonnes et intgrations :
  
  Field coord=MESH.getCoordinateField(Dom);
  coord.setName("coordinates");
  Array ACoord((TYPE="arrayField",ORDER=1,DIM=2,NAME="Champ de coordonnes",
		DERIV=false,COMPONENT(0)=coord.component(0),
		COMPONENT(1)=coord.component(1)));

  TensorCoordSys TCS((TCSTYPE="Cartesian2D",COORD=ACoord));

  GlobLoc GL((COORD=ACoord,GLTYPE="2D2D"));
  GlobLoc GL2((COORD=ACoord,GLTYPE="2D1D"));
  GlobLoc GL0((COORD=ACoord,GLTYPE="0D"));

  Integ INTG((GLOBLOC=GL,TENSORCOORDSYS=TCS,RULE=3));
  Integ INTG2((GLOBLOC=GL2,TENSORCOORDSYS=TCS,RULE=3));  
  
  GlobLoc::setDefaultGL(GL);

  // Tout ce qui concerne les quations de Navier-Stokes :

  Field pressure((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=1,
		  COMPONENTNAME(0)="p",NAME="pressure"));
  Field velocities((DOM=Dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		    COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		    NAME="velocities"));
  Array Avelo((TYPE="arrayField",ORDER=1,DIM=2,NAME="vitesses",
	       TENSORCOORDSYS=TCS,COMPONENT(0)=velocities.component(0),
	       COMPONENT(1)=velocities.component(1)));
  Array Apressure((TYPE="arrayField",ORDER=0,NAME="pression",
		   TENSORCOORDSYS=TCS,COMPONENT()=pressure.component(0)));
  
  Array testMvt((TYPE="arrayTest",TEST=Avelo,NAME="Equations de mouvement"));  
  Array testDiv((TYPE="arrayTest",TEST=Apressure,
		 NAME="Equations de continuit"));
  
  Array Unit((TYPE="array",ORDER=2,DIM=2,COMPONENT(0,0)=1.0,
	      COMPONENT(1,1)=1.0,NAME="Unit tensor",TENSORCOORDSYS=TCS));
  Array rho((TYPE="array",ORDER=0,NAME="density",TENSORCOORDSYS=TCS));
  
  // Navier-Stokes equations :

  Array gradVelo=grad(Avelo);
  gradVelo.setName("Champ gradients de vitesses");
  Array Convect=rho*contractedMultiplication(Avelo,0,gradVelo,1);
  Convect.setName("Termes convectifs");
  Array gradMvt=grad(testMvt);
  gradMvt.setName("gradMvt");
  
  Array divV=contract(gradVelo);
  divV.setName("divergence des vitesses");
  
  Array D=0.5*(gradVelo+transpose(gradVelo));
  D.setName("Taux de dformation");
  Array T=2.0*D-Apressure*Unit; // viscosity=2.0
  T.setName("Tenseur des tensions de Cauchy");
  Array NavierStokes=accumulation(gradMvt,T)+accumulation(testMvt,Convect)
    +testDiv*divV;
  NavierStokes.setName("Equations de Navier-Stokes");
  
  Array Prob1=integrate(NavierStokes,Dom,INTG);
  
  // Equation de dbit :
  
  Field fx((DOM=Dom,INTERPOLATION="constantOnDomain",NBRCOMPONENTS=1,
	    COMPONENTNAME(0)="fx",NAME="fx"));
  Array AFX((TYPE="arrayField",ORDER=0,NAME="AFX",TENSORCOORDSYS=TCS,
	     COMPONENT()=fx.component(0)));
  Array AveloX((TYPE="arrayField",ORDER=0,NAME="velo comp X",
		TENSORCOORDSYS=TCS,COMPONENT()=velocities.component(0)));
  Array AmvtX((TYPE="arrayComp",ORDER=0,NAME="mvt comp X",TENSORCOORDSYS=TCS,
	       COMPONENT()=testMvt.component(0)));
  Array testDebit((TYPE="arrayTest",TEST=AFX,NAME="Equation de dbit"));  
  Array ADebit((TYPE="array",ORDER=0,NAME="dbit",TENSORCOORDSYS=TCS,
		COMPONENT()=-1.0));
  Array Prob1CF1=integrate(testDebit*AveloX,Bnd1,INTG2)-testDebit*ADebit
    +integrate(AmvtX*AFX,Bnd1,INTG2)+integrate(0.0*AmvtX*AFX,Bnd3,INTG2);
  
  // Autres conditions aux frontires :
  
  Array AveloZero((TYPE="array",ORDER=1,DIM=2,NAME="vitessesZero",
		   TENSORCOORDSYS=TCS));
  Constr Prob1CF2((NAME="V=0 lower boundary",TYPE="usual",DOM=Bnd2,
		   TEST=testMvt,EQUATION=Avelo-AveloZero,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
  Constr Prob1CF3((NAME="V=0 upper boundary",TYPE="usual",DOM=Bnd4,
		   TEST=testMvt,EQUATION=Avelo-AveloZero,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
  Constr Prob1CF4((NAME="periodicity on V",TYPE="periodic",LEFTDOM=Bnd1,
		   LEFTTEST=testMvt,LEFTEQUATION=Avelo,RIGHTDOM=-Bnd3,
		   RIGHTTEST=testMvt,RIGHTEQUATION=-Avelo,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
  Array ApressureZero((TYPE="array",ORDER=0,NAME="pressureZero",
		       TENSORCOORDSYS=TCS));
  Constr Prob1CF5((NAME="P=0 on a corner",TYPE="usual",DOM=Corner,
		   TEST=testDiv,EQUATION=Apressure-ApressureZero,GLOBLOC=GL0,
		   TENSORCOORDSYS=TCS));
  Constr Prob1CF6((NAME="V=0 left boundary",TYPE="usual",DOM=Bnd1,
		   TEST=testMvt,EQUATION=Avelo-AveloZero,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
  Constr Prob1CF7((NAME="V=0 right boundary",TYPE="usual",DOM=Bnd3,
		   TEST=testMvt,EQUATION=Avelo-AveloZero,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));

  // Calcul de la fonction de Courant :
  
  Field courant((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=1,
		 COMPONENTNAME(0)="psi",NAME="courant"));
  Field courantBC((DOM=Bnd4,INTERPOLATION="constantOnDomain",
		   NBRCOMPONENTS=1,COMPONENTNAME(0)="psi",
		   NAME="courant for BC"));
  Array Acourant((TYPE="arrayField",ORDER=0,NAME="courant",TENSORCOORDSYS=TCS,
		  COMPONENT()=courant.component(0)));
  Array AcourantBC((TYPE="arrayField",ORDER=0,NAME="courantBC",
		    TENSORCOORDSYS=TCS,COMPONENT()=courantBC.component(0)));
  Array Avelo2((TYPE="arrayField",ORDER=1,DIM=2,NAME="vitesses",DERIV=false,
	        TENSORCOORDSYS=TCS,COMPONENT(0)=velocities.component(0),
	        COMPONENT(1)=velocities.component(1)));
     // (meme chose que Avelo, mais on ne drive pas par rapport aux valeurs
     //  nodales.)
  Array U((TYPE="arrayComp",ORDER=0,NAME="u",
	   COMPONENT()=Avelo2.component(0)));
  Array V((TYPE="arrayComp",ORDER=0,NAME="v",
	   COMPONENT()=Avelo2.component(1)));
  Array mV=-1.0*V;
  Array tmpVelo((TYPE="arrayComp",ORDER=1,DIM=2,NAME="special",
		 TENSORCOORDSYS=TCS,COMPONENT(0)=mV.component(),
		 COMPONENT(1)=U.component()));
  Array gradPsi=grad(Acourant);
  Array testCourant((TYPE="arrayTest",TEST=Acourant,
		     NAME="Equations de courant"));  
  Array gradCourant=grad(testCourant);
  Array eqCourant=gradPsi-tmpVelo;
  Array equationCourant=accumulation(gradCourant,eqCourant);
  Array Prob2=integrate(equationCourant,Dom,INTG);
  
  Constr Prob2CF1((NAME="Psi=0 lower boundary",TYPE="usual",DOM=Bnd2,
		   TEST=testCourant,EQUATION=Acourant,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
  Constr Prob2CF2((NAME="Psi upper boundary condition",TYPE="usual",
		   DOM=Bnd4,TEST=testCourant,EQUATION=Acourant-AcourantBC,
		   GLOBLOC=GL2,TENSORCOORDSYS=TCS));
  
  // Rsolution :
  
  for (i=0;i<=10;i++) {
    rho.component()=20.0*i;

    cerr << "New continuation step with rho = " << 20.0*i << endl ;
    
    Solver NS((PROBLEM=Prob1,PROBLEM=Prob1CF1,CONSTR=Prob1CF2,
      CONSTR=Prob1CF3,CONSTR=Prob1CF4,CONSTR=Prob1CF5,
      CONVERGENCE=1.0e-4,DIVERGENCE=1.0e4,NITMAX=6));
    NS.NewtonRaphson();

    Solver STREAM((PROBLEM=Prob2,CONSTR=Prob2CF1,CONSTR=Prob2CF2,
		   CONVERGENCE=1.0e-4,DIVERGENCE=1.0e4,NITMAX=1));
    STREAM.NewtonRaphson();
    courantBC.writeValues(cerr);
    
    Plot output((DOM=Dom,COORD=ACoord,RESULT=Avelo,RESULT=Apressure,
		 RESULT=Acourant));
    
    sprintf(buf,"%d",i);
    string filename="squareStep";
    filename+=buf;
    output.exec(filename);
  }
}


