
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <fstream>
#include <iomanip>
#include <string>
#include <string.h>
#include <stdio.h>

#include "Array.h"
#include "ArrayOperators.h"
#include "GlobLoc.h"
#include "TensorCoordSys.h"
#include "Domain.h"
#include "Field.h"
#include "Integ.h"
#include "Constr.h"
#include "Mesh.h"
#include "param.h"
#include "paramList.h"
#include "Plot.h"
#include "Solver.h"

using namespace std ;
using namespace keyFE2 ;



int main(void) {
  char buf[256];
  Int i;
  
  // Reading and renumbering of the mesh :
  
  Mesh MESH;
  MESH.read("suspR.mesh","emc2Mesh");
  string sortType="frontal";
  if (sortType=="position")
  {
    Domain Dom=MESH.getDomain(0);
    Field coord=MESH.getCoordinateField(Dom);
    coord.setName("coordinates");
    Array ACoord((TYPE="arrayField",ORDER=1,DIM=2,NAME="Champ_de_coordonnes",
		  DERIV=false,COMPONENT(0)=coord.component(0),
		  COMPONENT(1)=coord.component(1)));
    Array ABrol((TYPE="array",ORDER=1,DIM=2,NAME="orient",
                 COMPONENT(0)=Double(1.0),COMPONENT(1)=Double(1.0e-4)));
    Array ADist=accumulation(ACoord,ABrol,1);
    MESH=renumberDefDomainArray(MESH,ADist);
  }
  else if (sortType=="frontal") {
    Domain Bnd1=MESH.getDomain(1);
    MESH=renumberDefDomainFrontal(MESH,Bnd1);
  }
  else if (sortType=="none") {
    // Nothing to do !
  }
  else {
    cerr << "Error in main :" << endl 
      << "   unknown sorting type :  " << sortType << "." << endl ;
    exit(0);
  }
  
  // Domains :
  
  Domain Dom,Bnd1,Bnd2,Bnd3,Bnd4,Bnd5,Bnd6,Bnd7,Corner,BndTop;
  Dom=MESH.getDomain(0);
  Dom.setName("main domain");
  Bnd1=MESH.getDomain(1);
  Bnd1.setName("lower right boundary");
  Bnd2=MESH.getDomain(2);
  Bnd2.setName("right boundary");
  Bnd3=MESH.getDomain(3);
  Bnd3.setName("upper right boundary");
  Bnd4=MESH.getDomain(4);
  Bnd4.setName("upper left boundary");
  Bnd5=MESH.getDomain(5);
  Bnd5.setName("left boundary");
  Bnd6=MESH.getDomain(6);
  Bnd6.setName("lower left boundary");
  Bnd7=MESH.getDomain(7);
  Bnd7.setName("circle boundary");
  {
    Domain Corners1,Corners2;
    Corners1=Bnd1.getBoundary();
    Corners2=Bnd2.getBoundary();
    Corner=Corners1*Corners2;
  }
  BndTop=Bnd3+Bnd4;
  BndTop.setName("upper boundary");
  
  // Coordinates, Integration rules :
  
  Field coord=MESH.getCoordinateField(Dom);
  coord.setName("coordinates");
  Array ACoord((TYPE="arrayField",ORDER=1,DIM=2,NAME="Champ_de_coordonnes",
		DERIV=false,COMPONENT(0)=coord.component(0),
		COMPONENT(1)=coord.component(1)));

  TensorCoordSys TCS((TCSTYPE="Cartesian2D",COORD=ACoord));

  GlobLoc GL((COORD=ACoord,GLTYPE="2D2D"));
  GlobLoc GL2((COORD=ACoord,GLTYPE="2D1D"));
  GlobLoc GL0((COORD=ACoord,GLTYPE="0D"));

  Integ INTG((GLOBLOC=GL,TENSORCOORDSYS=TCS,RULE=3));
  Integ INTG2((GLOBLOC=GL2,TENSORCOORDSYS=TCS,RULE=3));  
  
  GlobLoc::setDefaultGL(GL);

  // Everything for Navier-Stokes equations :
  
  Field pressure((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=1,
		  COMPONENTNAME(0)="p",NAME="pressure"));
  Field velocities((DOM=Dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		    COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		    NAME="velocities"));
  Array Avelo((TYPE="arrayField",ORDER=1,DIM=2,NAME="vitesses",
	       TENSORCOORDSYS=TCS,COMPONENT(0)=velocities.component(0),
	       COMPONENT(1)=velocities.component(1)));
  Array Apressure((TYPE="arrayField",ORDER=0,NAME="pression",
		   TENSORCOORDSYS=TCS,COMPONENT()=pressure.component(0)));
  
  Array testMvt((TYPE="arrayTest",TEST=Avelo,NAME="Equations de mouvement"));  
  Array testDiv((TYPE="arrayTest",TEST=Apressure,NAME="Equations de continuit"));
  
  Array Unit((TYPE="array",ORDER=2,DIM=2,COMPONENT(0,0)=Double(1.0),
	      COMPONENT(1,1)=Double(1.0),NAME="Unit tensor",
              TENSORCOORDSYS=TCS));
  Array rho((TYPE="array",ORDER=0,NAME="density",TENSORCOORDSYS=TCS));
  
  // Navier-Stokes equations :

  Array gradVelo=grad(Avelo);
  gradVelo.setName("Champ gradients de vitesses");
  Array Convect=rho*contractedMultiplication(Avelo,0,gradVelo,1);
  Convect.setName("Termes convectifs");
  Array gradMvt=grad(testMvt);
  gradMvt.setName("gradMvt");
  
  Array divV=contract(gradVelo);
  divV.setName("divergence des vitesses");
  
  Array D=0.5*(gradVelo+transpose(gradVelo));
  D.setName("Taux de dformation");
  Array T=2.0*D-Apressure*Unit; // viscosity=1.0
  T.setName("Tenseur des tensions de Cauchy");
  Array NavierStokes=accumulation(gradMvt,T)+accumulation(testMvt,Convect)
    +testDiv*divV;
  NavierStokes.setName("Equations de Navier-Stokes");
  
  Array Prob1=integrate(NavierStokes,Dom,INTG);
  
  // Inflow is explicitly calculated :
  
  Field angularVelo((DOM=Bnd7,INTERPOLATION="constantOnDomain",
		     NBRCOMPONENTS=1,COMPONENTNAME(0)="omega",
		     NAME="angular velocity for BC"));
  Array AangularVelo((TYPE="arrayField",ORDER=0,NAME="AangularVelo",
		      TENSORCOORDSYS=TCS,
		      COMPONENT()=angularVelo.component(0)));
  Array X((TYPE="arrayComp",ORDER=0,DIM=1,NAME="coord X",
	   COMPONENT()=ACoord.component(0),TENSORCOORDSYS=TCS));
  Array Y((TYPE="arrayComp",ORDER=0,DIM=1,NAME="coord Y",
	   COMPONENT()=ACoord.component(1),TENSORCOORDSYS=TCS));
  Array mX=-1.0*X;
  Array Atangent((TYPE="arrayComp",ORDER=1,DIM=2,NAME="vecteur tangent au cercle",
		  COMPONENT(0)=Y.component(),COMPONENT(1)=mX.component(),
		  TENSORCOORDSYS=TCS));
  Constr Prob1CF1((NAME="V=0 on circle",TYPE="usual",DOM=Bnd7,TEST=testMvt,
		   EQUATION=Avelo-Atangent*AangularVelo,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
  
  Array AveloOne((TYPE="array",ORDER=1,DIM=2,NAME="vitessesUnit",
		  TENSORCOORDSYS=TCS,COMPONENT(0)=Double(1.0)));
  Constr Prob1CF2((NAME="periodicity 3-4 on V",TYPE="periodic",LEFTDOM=Bnd4,
		   LEFTTEST=testMvt,LEFTEQUATION=Avelo-AveloOne,RIGHTDOM=-Bnd3,
		   RIGHTTEST=testMvt,RIGHTEQUATION=Avelo-AveloOne,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
  
  Constr Prob1CF3((NAME="periodicity 2-5 on V",TYPE="periodic",LEFTDOM=Bnd5,
		   LEFTTEST=testMvt,LEFTEQUATION=Avelo,RIGHTDOM=-Bnd2,
		   RIGHTTEST=testMvt,RIGHTEQUATION=-Avelo,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));

  Constr Prob1CF4((NAME="periodicity 1-6 on V",TYPE="periodic",LEFTDOM=Bnd6,
		   LEFTTEST=testMvt,LEFTEQUATION=Avelo,RIGHTDOM=-Bnd1,
		   RIGHTTEST=testMvt,RIGHTEQUATION=Avelo,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
  
  Constr Prob1CF5((NAME="periodicity 3-4 on P",TYPE="periodic",LEFTDOM=Bnd4,
		   LEFTTEST=testDiv,LEFTEQUATION=Apressure,RIGHTDOM=-Bnd3,
		   RIGHTTEST=testDiv,RIGHTEQUATION=-Apressure,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
  
  Constr Prob1CF6((NAME="periodicity 2-5 on P",TYPE="periodic",LEFTDOM=Bnd5,
		   LEFTTEST=testDiv,LEFTEQUATION=Apressure,RIGHTDOM=-Bnd2,
		   RIGHTTEST=testDiv,RIGHTEQUATION=-Apressure,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
  
  Constr Prob1CF7((NAME="periodicity 1-6 on P",TYPE="periodic",LEFTDOM=Bnd6,
		   LEFTTEST=testDiv,LEFTEQUATION=Apressure,RIGHTDOM=-Bnd1,
		   RIGHTTEST=testDiv,RIGHTEQUATION=-Apressure,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
  
  Array ApressureZero((TYPE="array",ORDER=0,NAME="pressureZero",
		       TENSORCOORDSYS=TCS,COMPONENT()=Double(0.0)));
  Constr Prob1CF8((NAME="P=0 on a corner",TYPE="usual",DOM=Corner,
		   TEST=testDiv,EQUATION=Apressure-ApressureZero,GLOBLOC=GL0,
		   TENSORCOORDSYS=TCS));
  

  // Calculation of stream function :
  
  Field courant((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=1,
		 COMPONENTNAME(0)="psi",NAME="courant"));
  Array Acourant((TYPE="arrayField",ORDER=0,NAME="courant",TENSORCOORDSYS=TCS,
		  COMPONENT()=courant.component(0)));
  Array Avelo2((TYPE="arrayField",ORDER=1,DIM=2,NAME="vitesses",DERIV=false,
	        TENSORCOORDSYS=TCS,COMPONENT(0)=velocities.component(0),
	        COMPONENT(1)=velocities.component(1)));
     // (same as Avelo, but no derivation wrt nodal values)
  Array U((TYPE="arrayComp",ORDER=0,NAME="u",
	   COMPONENT()=Avelo2.component(0)));
  Array V((TYPE="arrayComp",ORDER=0,NAME="v",
	   COMPONENT()=Avelo2.component(1)));
  Array mV=-1.0*V;
  Array tmpVelo((TYPE="arrayComp",ORDER=1,DIM=2,NAME="special",
		 TENSORCOORDSYS=TCS,COMPONENT(0)=mV.component(),
		 COMPONENT(1)=U.component()));
  Array gradPsi=grad(Acourant);
  Array testCourant((TYPE="arrayTest",TEST=Acourant,
		     NAME="Equations de courant"));  
  Array gradCourant=grad(testCourant);
  Array eqCourant=gradPsi-tmpVelo;
  Array equationCourant=accumulation(gradCourant,eqCourant);
  Array Prob2=integrate(equationCourant,Dom,INTG);
  
  Constr Prob2CF1((NAME="Psi=0 lower boundary",TYPE="usual",DOM=Bnd7,
		   TEST=testCourant,EQUATION=Acourant,GLOBLOC=GL2,
		   TENSORCOORDSYS=TCS));
  
  // Rsolution :
  
  for (i=0;i<=5;i++) {
    rho.component()=200.0*i;

    cerr << "New continuation step with rho = " << 200.0*i << endl ;
    
    Solver NS((PROBLEM=Prob1,
      CONSTR=Prob1CF1,CONSTR=Prob1CF2,CONSTR=Prob1CF3,CONSTR=Prob1CF4,
      CONSTR=Prob1CF5,CONSTR=Prob1CF6,CONSTR=Prob1CF7,CONSTR=Prob1CF8,
      CONVERGENCE=Double(1.0e-4),DIVERGENCE=Double(1.0e4),NITMAX=6));
    NS.NewtonRaphson();

    Solver STREAM((PROBLEM=Prob2,CONSTR=Prob2CF1,
		   CONVERGENCE=Double(1.0e-4),DIVERGENCE=Double(1.0e4),
                   NITMAX=1));
    STREAM.NewtonRaphson();
    
    Plot output((DOM=Dom,COORD=ACoord,RESULT=Avelo,RESULT=Apressure,
		 RESULT=Acourant));
    
    sprintf(buf,"%d",i);
    string filename="susp";
    filename+=buf;
    output.exec(filename);
  }
}


