
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <fstream>
#include <iomanip>
#include <string>
#include <string.h>
#include <stdio.h>

#include "Array.h"
#include "ArrayOperators.h"
#include "GlobLoc.h"
#include "TensorCoordSys.h"
#include "Domain.h"
#include "Field.h"
#include "Filter.h"
#include "Integ.h"
#include "Constr.h"
#include "Mesh.h"
#include "param.h"
#include "paramList.h"
#include "Plot.h"
#include "Result.h"
#include "Solver.h"

using namespace std ;
using namespace keyFE2 ;



int main(void) {
  char buf[256];
  
  // Reading and renumbering of the mesh :
  
  Mesh MESH;
  MESH.read("threadR.mesh","emc2Mesh");
  string sortType="frontal";
  if (sortType=="position")
    {
      Domain Dom=MESH.getDomain(0);
      Field coord=MESH.getCoordinateField(Dom);
      coord.setName("coordinates");
      Array ACoord((TYPE="arrayField",ORDER=1,DIM=2,NAME="Champ_de_coordonnes",
                    DERIV=false,COMPONENT(0)=coord.component(0),
                    COMPONENT(1)=coord.component(1)));
      Array ABrol((TYPE="array",ORDER=1,DIM=2,NAME="orient",
                   COMPONENT(0)=Double(1.0),COMPONENT(1)=Double(1.0e-4)));
      Array ADist=accumulation(ACoord,ABrol,1);
      MESH=renumberDefDomainArray(MESH,ADist);
    }
  else if (sortType=="frontal") {
    Domain Bnd=MESH.getDomain(2);
    MESH=renumberDefDomainFrontal(MESH,Bnd);
  }
  else if (sortType=="none") {
    // Nothing to do !
  }
  else {
    cerr << "Error in main :" << endl 
      << "   unknown sorting type :  " << sortType << "." << endl ;
    exit(0);
  }
  
  // Domains :
  
  Domain Dom,Bnd1,Bnd2,Bnd3,Bnd4;
  Dom=MESH.getDomain(0);
  Dom.setName("main domain");
  Bnd1=MESH.getDomain(1);
  Bnd1.setName("lower boundary");
  Bnd2=MESH.getDomain(2);
  Bnd2.setName("right boundary");
  Bnd3=MESH.getDomain(3);
  Bnd3.setName("upper boundary");
  Bnd4=MESH.getDomain(4);
  Bnd4.setName("left boundary");
  
  // coordinate systems :
  
  Field coord; 
  {
    Field tmpCoord=MESH.getCoordinateField(Dom);
    coord=Field((DOM=Dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
                 COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
                 NAME="coordinates")); 
    coord.interpolate(tmpCoord,Dom,tmpCoord);
  }
  Field refCoord=MESH.getCoordinateField(Dom);
  refCoord.setName("reference coordinates");
  
  Array ACoordXY((TYPE="arrayField",ORDER=1,DIM=2,NAME="Champ de coordonnes",
                  COMPONENT(0)=coord.component(0),
                  COMPONENT(1)=coord.component(1),
                  DERIV=true)); 
  Array ARefCoordXY((TYPE="arrayField",ORDER=1,DIM=2,
                     NAME="Champ de coordonnes de rfrence",
                     COMPONENT(0)=refCoord.component(0),
                     COMPONENT(1)=refCoord.component(1),
                     DERIV=false));
  Array ATestCoordXY((TYPE="arrayTest",TEST=ACoordXY,
                      NAME="Equations de Thompson"));  
  
  TensorCoordSys TCSXY((TCSTYPE="Cartesian2D",COORD=ACoordXY));
  GlobLoc GL1((COORD=ACoordXY,GLTYPE="2D2D"));
  GlobLoc GL2((COORD=ACoordXY,GLTYPE="2D1D"));
  
  Integ INTG1((GLOBLOC=GL1,TENSORCOORDSYS=TCSXY,RULE=3));
  Integ INTG2((GLOBLOC=GL2,TENSORCOORDSYS=TCSXY,RULE=3));
  
  Array ACoordRZ((TYPE="arrayField",ORDER=1,DIM=3,NAME="Champ de coordonnes",
                  DERIV=true,COMPONENT(0)=coord.component(0),
                  COMPONENT(1)=coord.component(1)));  
  TensorCoordSys TCSRZ((TCSTYPE="CylindricalRZT",COORD=ACoordRZ));
  Array ACoordRZ2((TYPE="arrayField",ORDER=1,DIM=3,NAME="Champ de coordonnes",
                   DERIV=true,COMPONENT(0)=coord.component(0),
                   COMPONENT(1)=coord.component(1),TENSORCOORDSYS=TCSRZ));
  GlobLoc GL3((COORD=ACoordRZ,GLTYPE="3D2D"));
  GlobLoc GL4((COORD=ACoordRZ,GLTYPE="3D1D"));
  Integ INTG3((GLOBLOC=GL3,TENSORCOORDSYS=TCSRZ,RULE=3));
  Integ INTG4((GLOBLOC=GL4,TENSORCOORDSYS=TCSRZ,RULE=3));
  
  // Remaillage de Thompson :
  
  GlobLoc::setDefaultGL(GL1);
  
  Array X((TYPE="arrayComp",ORDER=0,NAME="X",
           COMPONENT()=ACoordXY.component(0),TENSORCOORDSYS=TCSXY));
  Array Y((TYPE="arrayComp",ORDER=0,NAME="Y",
           COMPONENT()=ACoordXY.component(1),TENSORCOORDSYS=TCSXY));
  Array testX((TYPE="arrayComp",ORDER=0,NAME="testX",
               COMPONENT()=ATestCoordXY.component(0),TENSORCOORDSYS=TCSXY));
  Array testY((TYPE="arrayComp",ORDER=0,NAME="testY",
               COMPONENT()=ATestCoordXY.component(1),TENSORCOORDSYS=TCSXY));
  Array refX((TYPE="arrayComp",ORDER=0,NAME="refX",
              COMPONENT()=ARefCoordXY.component(0),TENSORCOORDSYS=TCSXY));
  Array refY((TYPE="arrayComp",ORDER=0,NAME="refY",
              COMPONENT()=ARefCoordXY.component(1),TENSORCOORDSYS=TCSXY));
  
  Array thompson=accumulation(grad(testX,GL1),grad(refX,GL1))
    +accumulation(grad(testY,GL1),grad(refY,GL1));
  Array Prob1=integrate(thompson,Dom,INTG1);
  
  Filter thompsonFilter((NAME="Filter for free surface",DOM=Bnd3,ARRAY=ACoordXY));
  Array Prob1b((NAME="filtered thompson equations",TYPE="arrayFilter",
                EXPR=Prob1,FILTER=thompsonFilter));
  
  Array One((TYPE="array",ORDER=0,COMPONENT()=1.0,NAME="1.0",
             TENSORCOORDSYS=TCSXY));
  Array Ampli((TYPE="array",ORDER=0,NAME="amplitude",TENSORCOORDSYS=TCSXY));
  
  Constr Prob1CF1((NAME="Thompson lower boundary",TYPE="usual",DOM=Bnd1,
                   TEST=testX,EQUATION=X-refX,GLOBLOC=GL2));
  Constr Prob1CF2((NAME="Thompson right boundary",TYPE="usual",DOM=Bnd2,
                   TEST=testY,EQUATION=Y-refY,GLOBLOC=GL2));
  Constr Prob1CF3((NAME="Thompson upper boundary",TYPE="usual",DOM=Bnd3,
                   TEST=testX,
                   EQUATION=X-refX*(One+Ampli*(cos(refY*(3.1415926535/5.0)))),
                   GLOBLOC=GL2));
  Constr Prob1CF4((NAME="Thompson left boundary",TYPE="usual",DOM=Bnd4,
                   TEST=testY,EQUATION=Y-refY,GLOBLOC=GL2));
  
  // Tangential remeshing :
  
  GlobLoc::setDefaultGL(GL2);
  Array thompson1D=accumulation(grad(testY,GL2),grad(refY,GL2));
  Array Prob4=integrate(thompson1D,Bnd3,INTG2);
  Prob4.setName("to be printed");
  
  // Tout ce qui concerne les quations de Navier-Stokes :
  
  GlobLoc::setDefaultGL(GL3);
  
  Field pressure((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=1,
                  COMPONENTNAME(0)="p",NAME="pressure"));
  Field config((DOM=Dom,INTERPOLATION="linear",NBRCOMPONENTS=4,
                COMPONENTNAME(0)="A11",COMPONENTNAME(1)="A12",
                COMPONENTNAME(2)="A22",COMPONENTNAME(3)="A33",
                NAME="config"));
  {
    Array Ainit((TYPE="array",ORDER=1,DIM=4,COMPONENT(0)=1.0,COMPONENT(1)=0.0,
                 COMPONENT(2)=1.0,COMPONENT(3)=1.0));
    config.interpolate(Ainit,Dom,ACoordXY);
  }
  Array Aconfig((TYPE="arrayField",ORDER=2,DIM=3,NAME="configuration",
                 TENSORCOORDSYS=TCSRZ,
                 COMPONENT(0,0)=config.component(0),
                 COMPONENT(0,1)=config.component(1),
                 COMPONENT(1,0)=config.component(1),
                 COMPONENT(1,1)=config.component(2),
                 COMPONENT(2,2)=config.component(3)));
  Field velocities((DOM=Dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
                    COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
                    NAME="velocities"));
  Array Avelo((TYPE="arrayField",ORDER=1,DIM=3,NAME="vitesses",
               TENSORCOORDSYS=TCSRZ,COMPONENT(0)=velocities.component(0),
               COMPONENT(1)=velocities.component(1)));
  Array Avelo2D((TYPE="arrayField",ORDER=1,DIM=2,NAME="vitesses",
                 COMPONENT(0)=velocities.component(0),
                 COMPONENT(1)=velocities.component(1)));
  Array Apressure((TYPE="arrayField",ORDER=0,NAME="pression",
                   TENSORCOORDSYS=TCSRZ,COMPONENT()=pressure.component(0)));
  
  TimeDep TD((NAME="transient",SCHEME="Galerkin",MAXSTEPSNBR=700,
              DTMIN=0.000001,DTMAX=1.0,DTINIT=0.01,TINIT=0.0,EPSILON=0.001,
              TFINAL=2000.0,INFLU=velocities,INFLU=config,ACTIVE=pressure));
  
  Array testMvt((TYPE="arrayTest",TEST=Avelo,NAME="Equations de mouvement"));  
  Array testDiv((TYPE="arrayTest",TEST=Apressure,NAME="Equations de continuit"));
  
  Array Unit((TYPE="array",ORDER=2,DIM=3,COMPONENT(0,0)=1.0,COMPONENT(1,1)=1.0,
              COMPONENT(2,2)=1.0,NAME="Unit tensor",TENSORCOORDSYS=TCSRZ));
  
  Array gradVelo=grad(Avelo);
  gradVelo.setName("Champ gradients de vitesses");
  Array gradMvt=grad(testMvt);
  gradMvt.setName("gradMvt");
  
  Array divV=contract(gradVelo);
  divV.setName("divergence des vitesses");
  
  double lambda=5.0;
  double L=20.0;
  double etaVisc=2.0;
  
  Array Aconvect=timeDeriv(Aconfig,TD)
    +contractedMultiplication(Avelo-timeDeriv(ACoordRZ2,TD),0,grad(Aconfig),2)
      -contractedMultiplication(gradVelo,1,Aconfig,0)
        -contractedMultiplication(Aconfig,1,transpose(gradVelo),0);
  Array tmpFactor=1.0-(1.0/L/L)*contract(Aconfig,0,1);
  Array Aconstit=Aconfig+lambda*tmpFactor*Aconvect-(tmpFactor/(1.0-3.0/L/L))*Unit;
  Array extraStress=(etaVisc/lambda)*(tmpFactor*Aconfig-(1.0/(1.0-3.0/L/L))*Unit);
  Array configTest((TYPE="arrayTest",TEST=Aconfig,NAME="Equations de constitution"));
  Array viscoel=accumulation(configTest,Aconstit);
  
  Array D=0.5*(gradVelo+transpose(gradVelo));
  D.setName("Taux de dformation");
  Array T=D-Apressure*Unit+extraStress;
  T.setName("Tenseur des tensions de Cauchy");
  Array Stokes=accumulation(gradMvt,T)+testDiv*divV;
  Stokes.setName("Equations de Stokes");
  
  Array Prob2=integrate(Stokes+viscoel,Dom,INTG3);
  
  Array U((TYPE="arrayComp",ORDER=0,NAME="u",COMPONENT()=Avelo.component(0)));
  Array V((TYPE="arrayComp",ORDER=0,NAME="v",COMPONENT()=Avelo.component(1)));
  Array testU((TYPE="arrayComp",ORDER=0,NAME="u",COMPONENT()=testMvt.component(0)));
  Array testV((TYPE="arrayComp",ORDER=0,NAME="v",COMPONENT()=testMvt.component(1)));
  
  Constr Prob2CF1((NAME="U=0 on boundary 1",TYPE="usual",DOM=Bnd1,
                   TEST=testU,EQUATION=U,GLOBLOC=GL4));
  Constr Prob2CF2((NAME="V=0 on boundary 2",TYPE="usual",DOM=Bnd2,
                   TEST=testV,EQUATION=V,GLOBLOC=GL4));
  Constr Prob2CF3((NAME="V=0 on boundary 4",TYPE="usual",DOM=Bnd4,
                   TEST=testV,EQUATION=V,GLOBLOC=GL4));
  
  // Surface tension :
  
  GlobLoc::setDefaultGL(GL4);
  
  Array AST=1.0*accumulation(grad(testMvt),grad(ACoordRZ2));
  AST.setName("Surface tension");
  
  Array Prob3=integrate(AST,Bnd3,INTG4);
  
  // Kinematic condition :
  
  Array normale=GL4.getNormal(TCSRZ);
  Array kinematic=testX*(accumulation(normale,Avelo-timeDeriv(ACoordRZ2,TD)));
  Array Prob5=integrate(kinematic,Bnd3,INTG4);
  
  // Solution initiale :
  
  Plot output((DOM=Dom,COORD=ACoordXY,RESULT=Avelo2D,RESULT=Apressure,
               RESULT=Aconfig));
  Result Rconfig((NAME="config",ARRAY=Aconfig,DOM=Dom,
                  INTERPOLATION="constantPerElement"));       
  Plot output2((DOM=Dom,COORD=ACoordXY,TYPE="openDX",
                RESULT=Avelo2D,RESULT=Apressure,RESULT=Rconfig));
  Plot output3((DOM=Dom,COORD=ACoordXY,TYPE="gmsh",
                RESULT=Avelo2D,RESULT=Apressure,RESULT=Rconfig));

  {
    Ampli.component()=0.05;
    Solver Th((PROBLEM=Prob1b,CONSTR=Prob1CF1,CONSTR=Prob1CF2,CONSTR=Prob1CF3,
               CONSTR=Prob1CF4,
               PROBLEM=Prob2,CONSTR=Prob2CF1,CONSTR=Prob2CF2,CONSTR=Prob2CF3,
               PROBLEM=Prob3,PROBLEM=Prob4,
               CONVERGENCE=1.0e-4,DIVERGENCE=1.0e4,NITMAX=6));
    Th.NewtonRaphson();
    
    output.exec("thread");
    output2.exec("thread");
    output3.exec("thread");
  }
  
  // Time-dependent solution :
  
  {
    Solver Th2((PROBLEM=Prob1b,CONSTR=Prob1CF1,CONSTR=Prob1CF2,CONSTR=Prob1CF4,
                PROBLEM=Prob2,CONSTR=Prob2CF1,CONSTR=Prob2CF2,CONSTR=Prob2CF3,
                PROBLEM=Prob3,PROBLEM=Prob5,PROBLEM=Prob4,
                CONVERGENCE=1.0e-4,DIVERGENCE=1.0e4,NITMAX=1));
    TD.addInflu(Th2);  
        
    while(TD.isToBeContinued()) {
      Th2.NewtonRaphson();
      if (TD.getCurrentStepIndex()==1
          ||Int(TD.getCurrentTime()/0.5)!=Int(TD.getPreviousTime()/0.5)
          ||TD.getCurrentTime()>13.0) {
        sprintf(buf,"%d",TD.getCurrentStepIndex());
        string filename="thread_";
        filename+=buf;
        output.exec(filename);
        output2.exec(filename);
        output3.exec(filename);
      }
    }
  }
  
}








