#ifndef KEYFLOW_NS2DC_H
#define KEYFLOW_NS2DC_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include "Array.h"
#include "GlobLoc.h"
#include "Integ.h"
#include "TensorCoordSys.h"
#include "TimeDep.h"



namespace keyFlow {
  
  using namespace keyFE2 ;
  
  class NavierStokes2DConvect : public Array {
    
    public:
      
      // general data:
      
      TimeDep localTimeDep ;
      
      // basic ArrayFields:
      
      Array coordinates ;
      Array velocities ;
      Array pressure ;
      Array temperature ;
     
      Array force ;
      
      Array coordinatesTimeDeriv ;

      // Quantities related to coordinates systems:
      
      GlobLoc gl ;
      TensorCoordSys tcs ;
      Integ itg ;
      
      // Quantities mainly related to momentum and incompressibility equations:
      
      Array viscosity ;
      Array rho ;
      
      Array gradVeloTensor ;
      Array strainRateTensor ;
      Array strainRate ;
      Array CauchyStressTensor ;
      Array inertia ;
      
      // Quantities mainly related to energy equation:
      
      Array rhoCp ;
      Array conductivity ;
      
      Array energyFlux ;      
        
      // Constructor:
      
      NavierStokes2DConvect(paramList PL) ;
      
  } ;
    
}

#endif
