#ifndef KEYFLOW_KINCOND1D_H
#define KEYFLOW_KINCOND1D_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include "Array.h"
#include "Constr.h"
#include "Domain.h"
#include "Field.h"
#include "GlobLoc.h"
#include "Integ.h"
#include "Problem.h"
#include "Solver.h"
#include "TensorCoordSys.h"
#include "TimeDep.h"



namespace keyFlow {
  
  using namespace keyFE2 ;
  
  class kinCond1D : public Problem {
    
    private:
    
      Solver normalSolver;
    
    public:
      
      // general data:
      
      Domain dom ;
      TimeDep localTimeDep ;
      
      // basic ArrayFields:
      
      Array CartCoordinates ;
      Array coordinates ;
      Array velocities ;
    
      Array coordinatesTimeDeriv ;
      
      Array X;
      Array Y;
      Array testX;
      Array testY;
      Array CartX;
      Array CartY;
      Array testCartX;
      Array testCartY;      
      
      Field refTCoordField;
      Array refTCoord;
      
      Field normalField;
      Array normal;
      Array Nx,Ny,Tx,Ty;
      Array Nvect,Tvect;
      Array transfoNT;
      
      Field displField;
      Array displNT;
      Array displT;
      Array displN;
      Array testDisplNT;
      Array testDisplT;
      Array testDisplN;

      // Quantities related to coordinates systems:
      
      GlobLoc gl ;
      TensorCoordSys tcs ;
      Integ itg ;
      
      GlobLoc CartGL ;
      TensorCoordSys CartTCS ;
      Integ CartITG ;
      
      // Constraints:
      
      Constr BCDom,BCPoint1,BCPoint2 ;
      
      // Quantities mainly related to equations:
      
      Array surfaceTension ;
              
      // Constructor:
      
      kinCond1D(paramList PL) ;
      
      // Recalculation of the normals and reinitialization of displacements :
      
      void updateNormalDispl(void) ;
      
  } ;
    
}

#endif
