#ifndef ARRAY_H
#define ARRAY_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <string>
#include <iostream>

#include "types/allTypes.h"

namespace keyFE2 {
  
  class array ;
  class Field ;
  class function1 ;
  class GlobLoc ;
  class object ;
  class param ;
  class paramList ;
  class scal ;
  class step ;
  class stepList ;
  class TensorCoordSys ;
  class Object ;
  
  class Array {
    
    private :
      
      void *data ;
    
    public :
      
      Array(void) ;
      Array(const Array &in) ;
      Array(const Array &in,function1 &F) ;
      Array(param P) ;
      Array(paramList PL) ;
      ~Array(void) ;
      
      // array interface :
      
      void init(void) ;
      void reset(void) ;
      
      void setData(array*) ;
      array* getData(void) const ;
      void setName(std::string s) ;
      std::string getName(void) const ;
      void setLayer(Int nbr) ;
      Int getLayer(void) const ;
      Int getOrder(void) const ;
      Int getDim(Int index) const ;
      
      void setTCS(TensorCoordSys &TCS) ;
      TensorCoordSys getTCS(void) const ;
      
      Array& component(const Int &index0=-1,const Int &index1=-1,
                       const Int &index2=-1,const Int &index3=-1,
                       const Int &index4=-1,const Int &index5=-1,
                       const Int &index6=-1,const Int &index7=-1) ;
      
      Array makeDeriv(Int nbrScal,scal **scalList,GlobLoc &GL,
                      std::string derivType) const ;
      Array makeExplicitDeriv(Int nbrScal,scal **scalList) const ;
      Array makeCoordDeriv(GlobLoc &GL) const ;
      Array makeCorlocDeriv(Int nbrDP) const ;
      Array makePhysicalDeriv(GlobLoc &GL) const ;
      
      void write(void) const ;
      void writeValues(void) const ;
      void write(std::ostream &out) const ;
      void writeValues(std::ostream &out) const ;
      
      // Calculation :
      
      void makeAtPoint(Object &obj,Double *localCoord) ;
      void makeOnObject(Object &obj) ;
      
      // = operator :
      
      void operator=(const Array &) ;
      void operator=(Double d) ;
      
      // Recovering of data :
      
      Double value(void) ;
      
  } ;
  
}

#endif
