
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <typeinfo>
#include <stdio.h>

#include "ArrayOperators.h"
#include "array_base.h"
#include "arrayField.h"
#include "arrayFunctionsBuild.h"
#include "arrayInteg.h"
#include "arrayOp1.h"
#include "arrayOp2.h"
#include "arrayOpN.h"
#include "arrayTime.h"
#include "domain/Domain.h"
#include "domain/domain_base.h"
#include "err/err.h"
#include "coordSys/GlobLoc.h"
#include "coordSys/globLoc_base.h"
#include "integ/Integ.h"
#include "integ/integ_base.h"
#include "function/functions.h"
#include "util/utils.h"

using namespace std ;
using namespace keyFE2 ;


namespace keyFE2 {
  
  Array ArrayDMult(const Array &Ain,const Double x) {
    try {
      array *arrayIn=Ain.getData();
      if (arrayIn==NULL)
        errThrow("ArrayDMult","non initialized data");
      DMult *f=new DMult(x);
      array *newArray=functionBuild(*arrayIn,f);
      
      Array out;
      out.setData(newArray);
      
      return(out);   
    }
    catchAll("Array ArrayDMult","at the end of function")
  }
  
  
  
  Array operator+(const Array &Ain) {
    try {
      return(Ain);
    }
    catchAll("Array operator+","unary operator")
  }
  
  
  
  Array operator-(const Array &Ain) {
    try {
      return(ArrayDMult(Ain,-1.0));
    }
    catchAll("Array operator-","unary operator")
  }
  
  
  
  Array operator*(const Array &Ain,const Double x) {
    try {
      return(ArrayDMult(Ain,x));
    }
    catchAll("Array operator*(A,d)","at the end of function")
  }
  
  
  
  Array operator*(const Double x,const Array &Ain) {
    try {
      return(ArrayDMult(Ain,x));
    }
    catchAll("Array operator*(d,A)","at the end of function")
  }
  
  
  Array operator/(const Array &Ain,const Double x) {
    try {
      return(ArrayDMult(Ain,1.0/x));
    }
    catchAll("Array operator/(A,d)","at the end of function")
  }
  
  
  
  // *********************************************************************
  
  
  
  Array ArrayFunction(const Array &Ain,function1 *F) {
    try {
      array *arrayIn=Ain.getData();
      if (arrayIn==NULL)
        errThrow("ArrayFunction","non initialized data");
      array *newArray=functionBuild(*arrayIn,F);
      
      Array out;
      out.setData(newArray);
      
      return(out);
    }
    catchAll("ArrayFunction","at the end of function")
  }
  
  
  
  Array exp(const Array &Ain) {
    try {
      function1 *tmpF=new DExp;
      
      return(ArrayFunction(Ain,tmpF));
    }
    catchAll("Array exp","at the end of function")
  }
  
  
  
  Array pow(const Array &Ain,Double y) {
    try {
      function1 *tmpF=new DPowMult(1.0,y);
      
      return(ArrayFunction(Ain,tmpF));
    }
    catchAll("Array pow(const Array &Ain,Double y)","at the end of function")
  }
  
  
  
  Array pow(Double x,const Array &Ain,Double y) {
    try {
      function1 *tmpF=new DPowMult(x,y);
      
      return(ArrayFunction(Ain,tmpF));
    }
    catchAll("Array pow(Double x,const Array &Ain,Double y)",
	     "at the end of function")
  }
  
  
  
  Array sqrt(const Array &Ain) {
    try {
      function1 *tmpF=new DPowMult(1.0,0.5);
      
      return(ArrayFunction(Ain,tmpF));
    }
    catchAll("Array sqrt","at the end of function")
  }
  
  
  
  Array sincos(Double sinfact,Double cosfact,Double argfact,const Array &Ain) {
    try {
      function1 *tmpF=new DSinCos(sinfact,cosfact,argfact);
      
      return(ArrayFunction(Ain,tmpF));
    }
    catchAll("Array sincos","at the end of function")
  }
  
  
  
  Array sin(Double sinfact,Double argfact,const Array &Ain) {
    try {
      function1 *tmpF=new DSin(sinfact,argfact);
      
      return(ArrayFunction(Ain,tmpF));
    }
    catchAll("Array sin(Double sinfact,Double argfact,const Array &Ain)",
	     "at the end of function")
  }
  
  
  
  Array sin(const Array &Ain) {
    try {
      function1 *tmpF=new DSin(1.0,1.0);
      
      return(ArrayFunction(Ain,tmpF));
    }
    catchAll("Array sin(const Array &Ain)","")
  }
  
  
  
  Array cos(Double cosfact,Double argfact,const Array &Ain) {
    try {
      function1 *tmpF=new DCos(cosfact,argfact);
      
      return(ArrayFunction(Ain,tmpF));
    }
    catchAll("Array cos(Double cosfact,Double argfact,const Array &Ain)"
	     ,"at the end of function")
  }
  
  
  
  Array cos(const Array &Ain) {
    try {
      function1 *tmpF=new DCos(1.0,1.0);
      
      return(ArrayFunction(Ain,tmpF));
    }
    catchAll("Array cos(const Array &Ain)","")
  }
  
  
  
  Array sinh(Double sinhfact,Double argfact,const Array &Ain) {
    try {
      function1 *tmpF=new DSinh(sinhfact,argfact);
      
      return(ArrayFunction(Ain,tmpF));
    }
    catchAll("Array sinh(Double sinhfact,Double argfact,const Array &Ain)",
	     "at the end of function")
  }
  
  
  
  Array sinh(const Array &Ain) {
    try {
      function1 *tmpF=new DSinh(1.0,1.0);
      
      return(ArrayFunction(Ain,tmpF));
    }
    catchAll("Array sinh(const Array &Ain)","")
  }
  
  
  
  Array cosh(Double coshfact,Double argfact,const Array &Ain) {
    try {
      function1 *tmpF=new DCosh(coshfact,argfact);
      
      return(ArrayFunction(Ain,tmpF));
    }
    catchAll("Array cosh(Double coshfact,Double argfact,const Array &Ain)"
	     ,"at the end of function")
  }
  
  
  
  Array cosh(const Array &Ain) {
    try {
      function1 *tmpF=new DCosh(1.0,1.0);
      
      return(ArrayFunction(Ain,tmpF));
    }
    catchAll("Array cosh(const Array &Ain)","")
  }
  
  
  
  Array tanh(Double tanhfact,Double argfact,const Array &Ain) {
    try {
      function1 *tmpF=new DTanh(tanhfact,argfact);
      
      return(ArrayFunction(Ain,tmpF));
    }
    catchAll("Array tanh(Double tanhfact,Double argfact,const Array &Ain)"
	     ,"at the end of function")
  }
  
  
  
  Array tanh(const Array &Ain) {
    try {
      function1 *tmpF=new DTanh(1.0,1.0);
      
      return(ArrayFunction(Ain,tmpF));
    }
    catchAll("Array tanh(const Array &Ain)","")
  }
  
  
  
  Array pwLinear4(Double pa,
	          Double x1,Double y1,
	          Double x2,Double y2,
	          Double x3,Double y3,
	          Double x4,Double y4,
	          Double pb,
		  const Array &Ain) {
    try {
      function1 *tmpF=new DpwLinear4(pa,x1,y1,x2,y2,x3,y3,x4,y4,pb);
      
      return(ArrayFunction(Ain,tmpF));
    }
    catchAll("Array pwLinear4(const Array &Ain)","")
  }
  
  
  
  Array pwLinear3(Double pa,
	          Double x1,Double y1,
	          Double x2,Double y2,
	          Double x3,Double y3,
	          Double pb,
		  const Array &Ain) {
    try {
      function1 *tmpF=new DpwLinear3(pa,x1,y1,x2,y2,x3,y3,pb);
      
      return(ArrayFunction(Ain,tmpF));
    }
    catchAll("Array pwLinear3(const Array &Ain)","")
  }
  
  
  
  Array pwLinear2(Double pa,
	          Double x1,Double y1,
	          Double x2,Double y2,
	          Double pb,
		  const Array &Ain) {
    try {
      function1 *tmpF=new DpwLinear2(pa,x1,y1,x2,y2,pb);
      
      return(ArrayFunction(Ain,tmpF));
    }
    catchAll("Array pwLinear2(const Array &Ain)","")
  }
  
  
  
  Array pwLinear1(Double pa,
	          Double x1,Double y1,
	          Double pb,
		  const Array &Ain) {
    try {
      function1 *tmpF=new DpwLinear1(pa,x1,y1,pb);
      
      return(ArrayFunction(Ain,tmpF));
    }
    catchAll("Array pwLinear1(const Array &Ain)","")
  }
  
  
  
  Array pwConst4(Double pa,
	         Double x1,Double p12,
	         Double x2,Double p23,
	         Double x3,Double p34,
	         Double x4,Double pb,
		 const Array &Ain) {
    try {
      function1 *tmpF=new DpwConst4(pa,x1,p12,x2,p23,x3,p34,x4,pb);
      
      return(ArrayFunction(Ain,tmpF));
    }
    catchAll("Array pwConst4(const Array &Ain)","")
  }
  
  
  
  Array pwConst3(Double pa,
	         Double x1,Double p12,
	         Double x2,Double p23,
	         Double x3,Double pb,
		 const Array &Ain) {
    try {
      function1 *tmpF=new DpwConst3(pa,x1,p12,x2,p23,x3,pb);
      
      return(ArrayFunction(Ain,tmpF));
    }
    catchAll("Array pwConst3(const Array &Ain)","")
  }
  
  
  
  Array pwConst2(Double pa,
	         Double x1,Double p12,
	         Double x2,Double pb,
		 const Array &Ain) {
    try {
      function1 *tmpF=new DpwConst2(pa,x1,p12,x2,pb);
      
      return(ArrayFunction(Ain,tmpF));
    }
    catchAll("Array pwConst2(const Array &Ain)","")
  }
  
  
  
  Array pwConst1(Double pa,
	         Double x1,Double pb,
		 const Array &Ain) {
    try {
      function1 *tmpF=new DpwConst1(pa,x1,pb);
      
      return(ArrayFunction(Ain,tmpF));
    }
    catchAll("Array pwConst1(const Array &Ain)","")
  }
  
  
  
  Array makeCorlocDeriv(const Array &oldArray,Int nbrCoord) {
    try {
      Array newArray;
      array *new_array,*old_array;
      
      old_array=oldArray.getData();
      if (old_array==NULL)
        errThrow("makeCorlocDeriv","non initialized data");
      new_array=old_array->makeCorlocDeriv(nbrCoord);
      newArray.setData(new_array);
      
      return(newArray);
    }
    catchAll("Array makeCorlocDeriv","at the end of function")
  }
  
  
  
  Array contract(const Array &oldArray,Int index1,Int index2) {
    try {
      Array newArray;
      array *new_array,*old_array;
      
      old_array=oldArray.getData();
      if (old_array==NULL)
        errThrow("contract","non initialized data");
      if (index1==-1&&index2==-1) {
	if (old_array->getOrder()!=2)
	  errThrow("Array contract(Array)","   argument must have order 2.");
	
	index1=0;
	index2=1;
      }
      new_array=contractionBuild(*old_array,index1,index2);
      newArray.setData(new_array);
      
      return(newArray);
    }
    catchAll("Array contract","at the end of function")
  }
  
  
  
  Array transpose(const Array &oldArray,Int index1,Int index2) {
    try {
      Array newArray;
      array *new_array,*old_array;
      
      old_array=oldArray.getData();
      if (old_array==NULL)
        errThrow("transpose","non initialized data");
      if (index1==-1&&index2==-1) {
	index1=old_array->getOrder()-2;
	index2=old_array->getOrder()-1;
      }
      new_array=transpositionBuild(*old_array,index1,index2);
      newArray.setData(new_array);
      
      return(newArray);
    }
    catchAll("Array transpose","at the end of function")
  }
  
  
  
  Array inverse(const Array &oldArray) {
    try {
      Array newArray;
      array *new_array,*old_array;
      
      old_array=oldArray.getData();
      if (old_array==NULL)
        errThrow("inverse","non initialized data");
      new_array=inversionBuild(*old_array);
      newArray.setData(new_array);
      
      return(newArray);
    }
    catchAll("Array inverse","at the end of function")
  }
  
  
  
  Array discrTranspose(const Array &oldArray) {
    try {
      Array newArray;
      array *new_array,*old_array;
      
      old_array=oldArray.getData();
      if (old_array==NULL)
        errThrow("discrTranspose","non initialized data");
      new_array=discretizedTranspositionBuild(*old_array);
      newArray.setData(new_array);
      
      return(newArray);
    }
    catchAll("Array discrTranspose(const Array &oldArray)",
	     "at the end of function")
  }
  
  
  
  Array test(const Array &oldArray) {
    try {
      Array newArray;
      array *old_array,*new_array;
      arrayField *old_arrayField,*new_arrayField;
      
      old_array=oldArray.getData();
      if (old_array==NULL)
        errThrow("test","non initialized data");
      if (typeid(*old_array)!=typeid(arrayField))
	errThrow("test operator",
		 string("   the argument must be an arrayField and not an ")
		 +typeid(*old_array).name()+".");
      
      old_arrayField=dynamic_cast<arrayField*>(old_array);
      new_arrayField=new arrayField(*old_arrayField);
      new_arrayField->setLigDeriv(false);
      new_array=dynamic_cast<array*>(new_arrayField);
      newArray.setData(new_array);
      
      return(newArray);
    }
    catchAll("Array test(const Array &oldArray)","at the end of function")
  }
  
  
  
  Array grad(const Array &Arg) {
    try {
      GlobLoc GL=GlobLoc::getDefaultGL();
      
      if (GL.getData()->getTypeName()=="none"
	  ||GL.getData()->getTypeName()=="unknown")
	errThrow("Array grad(const Array &Arg)",
		 "default GlobLoc seems to be undefined");
      
      return(Arg.makePhysicalDeriv(GL));
    }
    catchAll("Array grad(const Array &Arg)","at the end of function")
  }
  
  
  
  Array grad(const Array &Arg,GlobLoc &GL) {
    try {
      return(Arg.makePhysicalDeriv(GL));
    }
    catchAll("Array grad(const Array &Arg,GlobLoc &GL)",
	     "at the end of function")
  }
  
  
  
  Array timeDeriv(const Array &arg,TimeDep &TD) {
    try {
      array *oldArray=arg.getData();
      if (oldArray==NULL)
        errThrow("timeDeriv","non initialized data");
      timeDep *tD=TD.getData();
      arrayTime *tmpArray=new arrayTime(*oldArray,*tD);
      Array NewArray;
      NewArray.setData(tmpArray);
      
      return(NewArray);
    }
    catchAll("Array timeDeriv","at the end of function")
  }
  
  
  
  Array operator+(const Array &Arg,Double x) {
    try {
      array *arg=Arg.getData();
      if (arg==NULL)
        errThrow("Array operator+(const Array &Arg,Double x)","non initialized data");
       if (arg->getOrder()!=0)
	errThrow("Array operator+(const Array &arg,Double x)",
		 "   order of the argument should be 0.");
      
      array *tmparray=new array(*arg);
      tmparray->component()=x;
      char sbuf[30];
      sprintf(sbuf,"%g",x);
      tmparray->setName(sbuf);
      Array tmpArray;
      tmpArray.setData(tmparray);
      
      return(Arg+tmpArray);
    }
    catchAll("Array operator+(const Array &Arg,Double x)",
	     "at the end of function")
  }
  
  
  
  Array operator-(const Array &Arg,Double x) {
    try {
      array *arg=Arg.getData();
      if (arg==NULL)
        errThrow("Array operator-(const Array &Arg,Double x)","non initialized data");
      if (arg->getOrder()!=0)
	errThrow("Array operator-(const Array &arg,Double x)",
		 "   order of the argument should be 0.");
      
      array *tmparray=new array(*arg);
      tmparray->component()=x;
      char sbuf[30];
      sprintf(sbuf,"%g",x);
      tmparray->setName(sbuf);
      Array tmpArray;
      tmpArray.setData(tmparray);
      
      return(Arg-tmpArray);
    }
    catchAll("Array operator-(const Array &Arg,Double x)",
	     "at the end of function")
  }
  
  
  
  Array operator+(Double x,const Array &Arg) {
    try {
      array *arg=Arg.getData();
      if (arg==NULL)
        errThrow("Array operator+(Double x,const Array &Arg)","non initialized data");
      if (arg->getOrder()!=0)
	errThrow("Array operator+(Double x,const Array &arg)",
		 "   order of the argument should be 0.");
      
      array *tmparray=new array(*arg);
      tmparray->component()=x;
      char sbuf[30];
      sprintf(sbuf,"%g",x);
      tmparray->setName(sbuf);
      Array tmpArray;
      tmpArray.setData(tmparray);
      
      return(tmpArray+Arg);
    }
    catchAll("Array operator+(Double x,const Array &Arg)",
	     "at the end of function")
  }
  
  
  
  Array operator-(Double x,const Array &Arg) {
    try {
      array *arg=Arg.getData();
      if (arg==NULL)
        errThrow("Array operator-(Double x,const Array &Arg)",
	         "non initialized data");
      if (arg->getOrder()!=0)
	errThrow("Array operator-(Double x,const Array &arg)",
		 "   order of the argument should be 0.");
      
      array *tmparray=new array(*arg);
      tmparray->component()=x;
      char sbuf[30];
      sprintf(sbuf,"%g",x);
      tmparray->setName(sbuf);
      Array tmpArray;
      tmpArray.setData(tmparray);
      
      return(tmpArray-Arg);
    }
    catchAll("Array operator-(Double x,const Array &Arg)",
	     "at the end of function")
  }
  
  
  
  Array operator+(const Array &first,const Array &second) {
    try {
      array *left_array,*right_array,*new_array;
      Array newArray;
      
      left_array=first.getData();
      right_array=second.getData();
      if (left_array==NULL)
        errThrow("Array operator+(const Array &first,const Array &second)",
	         "non initialized first data");
      if (right_array==NULL)
        errThrow("Array operator+(const Array &first,const Array &second)",
	         "non initialized second data");
      if (specialCombiliNeeded(left_array,right_array)) {
	new_array=dynamic_cast<array*>(specialCombiliBuild(1.0,left_array,1.0,right_array));
      }
      else
	new_array=dynamic_cast<array*>(additionBuild(left_array,right_array));
      newArray.setData(new_array);
      
      return(newArray);
    }
    catchAll("Array operator+(const Array &first,const Array &second)",
	     "at the end of function")
  }
  
  
  
  Array operator-(const Array &first,const Array &second) {
    try {
      array *left_array,*right_array,*new_array;
      Array newArray;
      
      left_array=first.getData();
      right_array=second.getData();
      if (left_array==NULL)
        errThrow("Array operator-(const Array &first,const Array &second)",
	         "non initialized first data");
      if (right_array==NULL)
        errThrow("Array operator-(const Array &first,const Array &second)",
	         "non initialized second data");
      if (specialCombiliNeeded(left_array,right_array)) {
	new_array=dynamic_cast<array*>(specialCombiliBuild(1.0,left_array,-1.0,right_array));
      }
      else
	new_array=dynamic_cast<array*>(soustractionBuild(left_array,right_array));
      newArray.setData(new_array);
      
      return(newArray);
    }
    catchAll("Array operator-(const Array &first,const Array &second)",
	     "at the end of function")
  }
  
  
  
  Array operator*(const Array &first,const Array &second) {
    try {
      array *left_array,*right_array,*new_array;
      Array newArray;
      
      left_array=first.getData();
      right_array=second.getData();
      if (left_array==NULL)
        errThrow("Array operator*(const Array &first,const Array &second)",
	         "non initialized first data");
      if (right_array==NULL)
        errThrow("Array operator*(const Array &first,const Array &second)",
	         "non initialized second data");
      new_array=dynamic_cast<array*>(exteriorMultiplicationBuild(left_array,right_array));
      newArray.setData(new_array);
      
      return(newArray);
    }
    catchAll("Array operator*(const Array &first,const Array &second)",
	     "at the end of function")
  }
  
  
  
  Array vectProduct(const Array &first,const Array &second) {
    try {
      array *left_array,*right_array,*new_array;
      Array newArray;
      
      left_array=first.getData();
      right_array=second.getData();
      if (left_array==NULL)
        errThrow("vectProduct","non initialized first data");
      if (right_array==NULL)
        errThrow("vectProduct","non initialized second data");
      new_array=dynamic_cast<array*>(vectProductBuild(left_array,right_array));
      newArray.setData(new_array);
      
      return(newArray);
    }
    catchAll("Array vectProduct(const Array &first,const Array &second)",
	     "at the end of function")
  }
  
  
  
  Array contractedMultiplication(const Array &first,const Array &second) {
    try {
      return(contractedMultiplication(first,-1,second,-1));
    }
    catchAll("Array contractedMultiplication",
	     "at the end of function")
  }
  
  
  
  Array contractedMultiplication(const Array &first,Int index1,
				 const Array &second,Int index2) {
    try {
      array *left_array,*right_array,*new_array;
      Array newArray;
      
      left_array=first.getData();
      right_array=second.getData();
      if (left_array==NULL)
        errThrow("contractedMultiplication","non initialized first data");
      if (right_array==NULL)
        errThrow("contractedMultiplication","non initialized second data");
      if (index1==-1&&index2==-1) {
	index1=left_array->getOrder()-1;
	index2=0;
      }
      new_array=dynamic_cast<array*>(contractedMultiplicationBuild(left_array,
								   index1,
								   right_array,
								   index2));
      newArray.setData(new_array);
      
      return(newArray);
    }
    catchAll("Array contractedMultiplication",
	     "at the end of function")
  }
  
  
  
  Array accumulation(const Array &first,const Array &second,Int nbrIndices) {
    try {
      array *left_array,*right_array,*new_array;
      Array newArray;
      
      left_array=first.getData();
      right_array=second.getData();
      if (left_array==NULL)
        errThrow("accumulation","non initialized first data");
      if (right_array==NULL)
        errThrow("accumulation","non initialized second data");
      if (nbrIndices==-1)
	nbrIndices=min(left_array->getOrder(),right_array->getOrder());
      new_array=dynamic_cast<array*>(accumulationBuild(left_array,right_array,
						       nbrIndices));
      newArray.setData(new_array);
      
      return(newArray);
    }
    catchAll("Array accumulation",
	     "at the end of function")
  }
  
  
  
  Array integrate(const Array &A,Domain &Dom,Integ &integration) {
    try {
      arrayInteg *a=new arrayInteg(A.getData(),Dom.getData(),
				   integration.getData());
      Array B;
      B.setData(a);
      
      return(B);
    }
    catchAll("Array integrate",
	     "at the end of function")
  }
  
  
  
  Array upwindF(Array &V,Array &rho,Array &k,Array &testU,Array &dXdt,
                GlobLoc &gl,Double dInterp) {
    try {
      arrayOpN *a=upwindF(V.getData(),
                          rho.getData(),
                          k.getData(),
			  testU.getData(),
			  dXdt.getData(),
			  gl.getData(),
			  dInterp);
      Array B;
      B.setData(a);
      
      return(B);			  
    }
    catchAll("Array upwindF",
	     "at the end of function")
  }
  
  
  
  Array upwindF(Array &V,Array &rho,Double k,Array &testU,Array &dXdt,
                GlobLoc &gl,Double dInterp) {
    try {
      arrayOpN *a=upwindF(V.getData(),
                          rho.getData(),
                          new array(k),
			  testU.getData(),
			  dXdt.getData(),
			  gl.getData(),
			  dInterp);
      Array B;
      B.setData(a);
      
      return(B);			  
    }
    catchAll("Array upwindF",
	     "at the end of function")
  }
  
  
  
  Array upwindF(Array &V,Array &rho,Array &k,Array &testU,Double dXdt,
                GlobLoc &gl,Double dInterp) {
    try {
      arrayOpN *a=upwindF(V.getData(),
                          rho.getData(),
                          k.getData(),
			  testU.getData(),
			  NULL,
			  gl.getData(),
			  dInterp);
      Array B;
      B.setData(a);
      
      return(B);			  
    }
    catchAll("Array upwindF",
	     "at the end of function")
  }
  
  
  
  Array upwindF(Array &V,Array &rho,Double k,Array &testU,Double dXdt,
                GlobLoc &gl,Double dInterp) {
    try {
      arrayOpN *a=upwindF(V.getData(),
                          rho.getData(),
                          new array(k),
			  testU.getData(),
			  NULL,
			  gl.getData(),
			  dInterp);
      Array B;
      B.setData(a);
      
      return(B);			  
    }
    catchAll("Array upwindF",
	     "at the end of function")
  }

  

  Array upwindF(Array &V,Double rho,Array &k,Array &testU,Array &dXdt,
                GlobLoc &gl,Double dInterp) {
    try {
      arrayOpN *a=upwindF(V.getData(),
                          new array(rho),
                          k.getData(),
			  testU.getData(),
			  dXdt.getData(),
			  gl.getData(),
			  dInterp);
      Array B;
      B.setData(a);
      
      return(B);			  
    }
    catchAll("Array upwindF",
	     "at the end of function")
  }
  
  
  
  Array upwindF(Array &V,Double rho,Double k,Array &testU,Array &dXdt,
                GlobLoc &gl,Double dInterp) {
    try {
      arrayOpN *a=upwindF(V.getData(),
                          new array(rho),
                          new array(k),
			  testU.getData(),
			  dXdt.getData(),
			  gl.getData(),
			  dInterp);
      Array B;
      B.setData(a);
      
      return(B);			  
    }
    catchAll("Array upwindF",
	     "at the end of function")
  }
  
  
  
  Array upwindF(Array &V,Double rho,Array &k,Array &testU,Double dXdt,
                GlobLoc &gl,Double dInterp) {
    try {
      arrayOpN *a=upwindF(V.getData(),
                          new array(rho),
                          k.getData(),
			  testU.getData(),
			  NULL,
			  gl.getData(),
			  dInterp);
      Array B;
      B.setData(a);
      
      return(B);			  
    }
    catchAll("Array upwindF",
	     "at the end of function")
  }
  
  
  
  Array upwindF(Array &V,Double rho,Double k,Array &testU,Double dXdt,
                GlobLoc &gl,Double dInterp) {
    try {
      arrayOpN *a=upwindF(V.getData(),
                          new array(rho),
                          new array(k),
			  testU.getData(),
			  NULL,
			  gl.getData(),
			  dInterp);
      Array B;
      B.setData(a);
      
      return(B);			  
    }
    catchAll("Array upwindF",
	     "at the end of function")
  }
}

