#ifndef ARRAY_OPERATORS_H
#define ARRAY_OPERATORS_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include "types/allTypes.h"
#include "array/Array.h"
#include "coordSys/GlobLoc.h"
#include "timeDep/TimeDep.h"

namespace keyFE2 {
  
  class Domain;
  class Integ;
  
  // Operators with one Array argument :
  
  Array operator+(const Array &) ;
  Array operator-(const Array &) ;
  Array operator*(const Array &,const Double) ;
  Array operator*(const Double,const Array &) ;
  Array operator/(const Array &,const Double) ;
  
  Array exp(const Array &) ;
  Array pow(const Array &,Double y) ;
  Array pow(Double x,const Array &,Double y) ;
  Array sqrt(const Array &) ;
  Array sincos(Double sinfact,Double cosfact,Double argfact,const Array &) ;
  Array sin(Double sinfact,Double argfact,const Array &) ;
  Array sin(const Array &) ;
  Array cos(Double cosfact,Double argfact,const Array &) ;
  Array cos(const Array &) ;
  Array sinh(Double sinhfact,Double argfact,const Array &) ;
  Array sinh(const Array &) ;
  Array cosh(Double coshfact,Double argfact,const Array &) ;
  Array cosh(const Array &) ;
  Array tanh(Double tanhfact,Double argfact,const Array &) ;
  Array tanh(const Array &) ;
  
  Array pwLinear4(Double pa,
	          Double x1,Double y1,
	          Double x2,Double y2,
	          Double x3,Double y3,
	          Double x4,Double y4,
	          Double pb,
		  const Array &in) ;
  Array pwLinear3(Double pa,
	          Double x1,Double y1,
	          Double x2,Double y2,
	          Double x3,Double y3,
	          Double pb,
		  const Array &in) ;
  Array pwLinear2(Double pa,
	          Double x1,Double y1,
	          Double x2,Double y2,
	          Double pb,
		  const Array &in) ;
  Array pwLinear1(Double pa,
	          Double x1,Double y1,
	          Double pb,
		  const Array &in) ;
  Array pwConst4(Double pa,
                 Double x1,Double p12,
                 Double x2,Double p23,
                 Double x3,Double p34,
                 Double x4,Double pb,
		 const Array &in);
  Array pwConst3(Double pa,
                 Double x1,Double p12,
                 Double x2,Double p23,
                 Double x3,Double pb,
		 const Array &in);
  Array pwConst2(Double pa,
                 Double x1,Double p12,
                 Double x2,Double pb,
		 const Array &in);
  Array pwConst1(Double pa,
                 Double x1,Double pb,
		 const Array &in);
  
  Array makeCorlocDeriv(const Array &,Int nbrCoord=3) ;
  Array contract(const Array &,Int index1=-1,Int index2=-1) ;
  Array transpose(const Array &,Int index1=-1,Int index2=-1) ;
  Array inverse(const Array &) ;
  Array discrTranspose(const Array &) ;
  
  Array test(const Array &) ;
  
  Array grad(const Array &) ;
  Array grad(const Array &,GlobLoc &GL) ;
  Array timeDeriv(const Array &,TimeDep &TD) ;
  
  // Operators with two arguments :
  
  Array operator+(const Array &,Double x) ;
  Array operator-(const Array &,Double x) ;
  Array operator+(Double x,const Array &) ;
  Array operator-(Double x,const Array &) ;
  Array operator+(const Array &,const Array &) ;
  Array operator-(const Array &,const Array &) ;
  Array operator*(const Array &,const Array &) ;
  Array vectProduct(const Array &,const Array &) ;
  Array contractedMultiplication(const Array &,Int index1,const Array &,
                                 Int index2) ;
  Array contractedMultiplication(const Array &,const Array &) ;
  Array accumulation(const Array &,const Array &,Int nbrIndices=-1) ;
  
  // Other operators :
  
  Array integrate(const Array &A,Domain &dom,Integ &integration) ;
  Array upwindF(Array &V,Array &rho,Array &k,Array &testU,
                Array &dXdt,GlobLoc &gl,Double dInterp=1.0) ;
  Array upwindF(Array &V,Array &rho,Double k,Array &testU,
                Array &dXdt,GlobLoc &gl,Double dInterp=1.0) ;
  Array upwindF(Array &V,Array &rho,Array &k,Array &testU,
                Double dXdt,GlobLoc &gl,Double dInterp=1.0) ;
  Array upwindF(Array &V,Array &rho,Double k,Array &testU,
                Double dXdt,GlobLoc &gl,Double dInterp=1.0) ;
  Array upwindF(Array &V,Double rho,Array &k,Array &testU,
                Array &dXdt,GlobLoc &gl,Double dInterp=1.0) ;
  Array upwindF(Array &V,Double rho,Double k,Array &testU,
                Array &dXdt,GlobLoc &gl,Double dInterp=1.0) ;
  Array upwindF(Array &V,Double rho,Array &k,Array &testU,
                Double dXdt,GlobLoc &gl,Double dInterp=1.0) ;
  Array upwindF(Array &V,Double rho,Double k,Array &testU,
                Double dXdt,GlobLoc &gl,Double dInterp=1.0) ;
  
  
}

#endif
