#ifndef ARRAY_FILTER_H
#define ARRAY_FILTER_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <set>
#include <string>

#include "array_base.h"

namespace keyFE2 {
  
  class Array ;
  class field ;
  class filter ;
  class object ;
  class printTree ;
  class timeDep ;
  
  class arrayFilter : public virtual array {
    
    friend class Array ;
    
    protected :
      
      array *expr ;
      Int nbrFilters ;
      filter **filterList ;
      
      std::set<Int> lines ; // Indices of lines to be cancelled.
      std::set<Int> cols ; // Indices of columns to be cancelled.
    
    public : 
      
      arrayFilter(void) ;
      arrayFilter(array &in) ;
      arrayFilter(arrayFilter &in) ;
      ~arrayFilter(void) ;
    
      void setExpr(array *A) ;
      array* getExpr(void) const ;
      void setNbrFilters(Int nbr) ;
      Int getNbrFilters(void) const ;
      void setFilter(Int pos,filter *F) ;
      filter* getFilter(Int pos) const ;
      
      array* makeCloneForArrayTime(timeDep &TD) ;
      array* makeDeriv(Int nbrScal,scal **scalList,globLoc *GL,
		       std::string derivType) const ;
      array* makeExplicitDeriv(Int nbrScal,scal **scalList) const ;
      array* makeCoordDeriv(globLoc *GL) const ;
      array* makeCorlocDeriv(Int nbrDP) const ;
      
      void setNewPoint(bool b) ;
      void accessNbrInit(void) ;
      void makeAtPoint(object *element,Int direct,Double *localCoord,
		       bool makeValue=true,bool makeDeriv=true) ;
      void makeOnObject(object *element,Int direct,bool makeValue,
                        bool makeDeriv) ;
      
      printTree getPrintTree(bool values) const ;
      
      stepList getStepList(void) ;
      std::set<field*>* buildActiveFieldList(void) ;
      std::set<parents*> getListOfType(std::string typeName) ;
      
      void generateFilters(void) ;
      void applyFiltersToComponents(void) ;
      void applyFiltersToScal(scal *tmpScal) const ;
      void cleanFilters(void) ;
    
  } ;
  
}

#endif


