
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <iomanip>
#include <typeinfo>

#include "array_base.h"
#include "err/err.h"
#include "scal/scal.h"
#include "util/utils.h"

using namespace std ;
using namespace keyFE2 ;



void array::transpositionExec(array *arrayArg,Int iParam1,Int iParam2) {
  try {
    Int i,j,index;
    Int argIndices[MAX_ORDER_ARRAY],indices[MAX_ORDER_ARRAY];
    
    for (i=0;i<MAX_ORDER_ARRAY;i++)
      argIndices[i]=indices[i]=-1;
    for (i=0;i<nbrVal;i++) {
      getIndicesFromIndex(i,indices);
      for (j=0;j<arrayArg->order;j++)
	if (j!=iParam1&&j!=iParam2)
	  argIndices[j]=indices[j];
      argIndices[iParam1]=indices[iParam2];
      argIndices[iParam2]=indices[iParam1];
      index=arrayArg->getIndexFromIndices(argIndices);
      componentValues[i]=arrayArg->componentValues[index];
    }
  }
  catchAll("array::transpositionExec","")
}



void array::contractionExec(array *arrayArg,Int iParam1,Int iParam2) {
  try {
    Int i,j,k,index;
    Int argIndices[MAX_ORDER_ARRAY],indices[MAX_ORDER_ARRAY];
    
    for (i=0;i<MAX_ORDER_ARRAY;i++)
      argIndices[i]=indices[i]=-1;
    for (i=0;i<nbrVal;i++) {
      componentValues[i]=0.0;
      getIndicesFromIndex(i,indices);
      k=0;
      for (j=0;j<arrayArg->order;j++) {
	if (j!=iParam1&&j!=iParam2)
	  argIndices[j]=indices[k];
	k++;
      }
      for (j=0;j<arrayArg->dim[iParam1];j++) {
	argIndices[iParam1]=j;
	argIndices[iParam2]=j;
	index=arrayArg->getIndexFromIndices(argIndices);
	componentValues[i]+=arrayArg->componentValues[index];
      }
    }
  }
  catchAll("array::contractionExec","")
}



void array::inversionExec(array *arrayArg) {
  try {
    Int i,j,k,l;
    static scal determinant,tmpScal;
    static array tmpArray;
    Int dim0,dim1,newDim;
    Int index0,index1;
    
    if (order==2) {
      if (dim[0]!=dim[1])
	errThrow("array::inversionExec",
		 "   the array must be square for inversion.");
      
      determinant=arrayArg->getDeterminant();
      
      if (tmpArray.order!=2)
	tmpArray.setOrder(2);
      if (tmpArray.dim[0]!=dim[0]-1)
	tmpArray.setDim(0,dim[0]-1);
      if (tmpArray.dim[1]!=dim[1]-1)
	tmpArray.setDim(1,dim[1]-1);
      
      dim0=arrayArg->dim[0];
      dim1=arrayArg->dim[1];
      newDim=dim0-1;
      for (i=0;i<dim0;i++) {
	for (j=0;j<dim1;j++) {
	  // One calculates the ijth cofactor of the argument matrix :
	  index0=0;
	  for (k=0;k<dim0;k++) {
	    if (k!=i) {
	      index1=0;
	      for (l=0;l<dim1;l++) {
		if (l!=j) {
		  tmpArray.componentValues[newDim*index0+index1]
		    =arrayArg->componentValues[dim0*k+l];
		  index1++;
		} // if (l!=j)
	      } // for l
	      index0++;
	    } // if (k!=i)
	  } // for k
	  if (((i+j)%2)==1)
	    componentValues[i+dim1*j]=(tmpArray.getDeterminant()/determinant)
	      *(-1.0);
	  else
	    componentValues[i+dim1*j]=tmpArray.getDeterminant()/determinant;
	} // for j
      } // for i
    }
    else if (order==0) {
      scal tmpUnit=1.0;
      componentValues[0]=tmpUnit/(arrayArg->componentValues[0]);
    }
    else
      errThrow("array::inversionExec",
	       "   order of the array must be 0 or 2 for inversion.");
  }
  catchAll("array::inversionExec","")
}



void array::functionExec(array *arrayArg,function1 *fParam,
			 bool makeValue,bool makeDeriv) {
  try {
    Int i;
    scal *oldScal,*newScal;
    
    for (i=0;i<nbrVal;i++) {
      newScal=componentValues+i;
      oldScal=arrayArg->componentValues+i;
      
      newScal->functionExec(oldScal,fParam,makeValue,makeDeriv);
    }
  }
  catchAll("array::functionExec","")
}



void array::indicesPermutationExec(array *arrayArg,Int *IList) {
  try {
    Int i,j,index;
    Int indices[MAX_ORDER_ARRAY],argIndices[MAX_ORDER_ARRAY];
    
    for (i=0;i<MAX_ORDER_ARRAY;i++)
      argIndices[i]=indices[i]=-1;
    for (i=0;i<nbrVal;i++) {
      arrayArg->getIndicesFromIndex(i,argIndices);
      for (j=0;j<order;j++)
	indices[j]=argIndices[IList[j]];
      index=getIndexFromIndices(indices);
      componentValues[index]=arrayArg->componentValues[i];
    }
  }
  catchAll("array::indicesPermutationExec","")
}



void array::discretizedTranspositionExec(array *arrayArg) {
  try {
    for (Int i=0;i<nbrVal;i++)
      componentValues[i].setToTransposedOf(arrayArg->componentValues[i]);
  }
  catchAll("array::discretizedTranspositionExec","")
}



void array::specialContractExec(array *arrayArg1,array *arrayArg2,
				Int IParam1) {
  
  // Contraction des IParam1 premiers indices de arrayArg1 et arrayArg2.
  
  try {
    Int i,j;
    Int arg1Indices[MAX_ORDER_ARRAY],arg2Indices[MAX_ORDER_ARRAY],
				     indices[MAX_ORDER_ARRAY];
    Int index1,index2;
    
    Int order1=arrayArg1->order;
    Int order2=arrayArg2->order;
    
    if (IParam1>arrayArg1->order)
      errThrow("array::specialContractExec",
	       string("   IParam1>arrayArg1->order (")+strFromInt(IParam1)
	       +">"+strFromInt(arrayArg1->order)+").");
    
    if (IParam1>arrayArg2->order)
      errThrow("array::specialContractExec",
	       string("   IParam1>arrayArg2->order (")+strFromInt(IParam1)
	       +">"+strFromInt(arrayArg2->order)+").");
    
    if (order!=arrayArg1->order+arrayArg2->order-IParam1)
      errThrow("array::specialContractExec",
	       string("   order!=arrayArg1->order+arrayArg2->order-IParam1 (")
	       +strFromInt(order)+"!="
	       +strFromInt(arrayArg1->order+arrayArg2->order-IParam1)+").");
    
    Int index=1;
    for (i=0;i<IParam1;i++) {
      if (arrayArg1->dim[i]!=arrayArg2->dim[i])
	errThrow("array::specialContractExec",
		 string("   arrayArg1->dim[")+strFromInt(i)
		 +"]!=arrayArg2->dim["+strFromInt(i)+"] ("
		 +strFromInt(arrayArg1->dim[i])+"!="
		 +strFromInt(arrayArg2->dim[i])+").");
      index*=arrayArg1->dim[i];
    }
    for (i=IParam1;i<arrayArg1->order;i++)
      index*=arrayArg1->dim[i];
    for (i=IParam1;i<arrayArg2->order;i++)
      index*=arrayArg2->dim[i];
    if (nbrVal!=index)
      errThrow("array::specialContractExec",
	       string("   nbrVal!=index (")+strFromInt(nbrVal)+"!="
	       +strFromInt(index)+").");
    
    for (i=0;i<MAX_ORDER_ARRAY;i++) {
      arg1Indices[i]=-1;
      arg2Indices[i]=-1;
      indices[i]=-1;
    }
    for (i=0;i<nbrVal;i++) {
      getIndicesFromIndex(i,indices);
      for (j=0;j<IParam1;j++) {
	arg1Indices[j]=indices[j];
	arg2Indices[j]=indices[j];
      }
      for (j=IParam1;j<order1;j++)
	arg1Indices[j]=indices[j];
      index=order1;
      for (j=IParam1;j<order2;j++) {
	arg2Indices[j]=indices[index];
	index++;
      }
      index1=arrayArg1->getIndexFromIndices(arg1Indices);
      index2=arrayArg2->getIndexFromIndices(arg2Indices);
      componentValues[i]=arrayArg1->componentValues[index1]
	*arrayArg2->componentValues[index2];
    }
  }
  catchAll("array::specialContractExec","")
}



void array::additionExec(array *arrayArg1,array *arrayArg2) {
  try {
    Int nbrVal1=arrayArg1->nbrVal;
    Int nbrVal2=arrayArg2->nbrVal;
    
    if (nbrVal1!=nbrVal2)
      errThrow("array::additionExec",
	       string("nbrVal1!=nbrVal2 (")+strFromInt(nbrVal1)+"!="
	       +strFromInt(nbrVal2)+").");
    if (nbrVal!=nbrVal2)
      errThrow("array::additionExec",
	       string("nbrVal!=nbrVal2 (")+strFromInt(nbrVal)+"!="
	       +strFromInt(nbrVal2)+").");
    
    for (Int i=0;i<nbrVal;i++) {
      componentValues[i]=arrayArg1->componentValues[i]
	+arrayArg2->componentValues[i];
    }
  }
  catchAll("array::additionExec","")
}



void array::soustractionExec(array *arrayArg1,array *arrayArg2) {
  try {
    Int nbrVal1=arrayArg1->nbrVal;
    Int nbrVal2=arrayArg2->nbrVal;
    
    if (nbrVal1!=nbrVal2)
      errThrow("array::soustractionExec",
	       string("nbrVal1!=nbrVal2 (")+strFromInt(nbrVal1)+"!="
	       +strFromInt(nbrVal2)+").");
    if (nbrVal!=nbrVal2)
      errThrow("array::soustractionExec",
	       string("nbrVal!=nbrVal2 (")+strFromInt(nbrVal)+"!="
	       +strFromInt(nbrVal2)+").");
    
    for (Int i=0;i<nbrVal;i++) {
      componentValues[i]=keyFE2::operator-(arrayArg1->componentValues[i],
	arrayArg2->componentValues[i]);
    }
  }
  catchAll("array::soustractionExec","")
}



void array::exteriorMultiplicationExec(array *arrayArg1,array *arrayArg2) {
  try {
    Int i,j;
    
    Int nbrVal1=arrayArg1->nbrVal;
    Int nbrVal2=arrayArg2->nbrVal;
    
    if (nbrVal!=nbrVal1*nbrVal2)
      errThrow("array::exteriorMultiplication",
	       string("   nbrVal!=nbrVal1*nbrVal2 (")+strFromInt(nbrVal)
	       +"!="+strFromInt(nbrVal1*nbrVal2)+").");
    
    for (i=0;i<nbrVal1;i++) {
      for (j=0;j<nbrVal2;j++) {
	componentValues[i*nbrVal2+j]=arrayArg1->componentValues[i]
	  *arrayArg2->componentValues[j];
      }
    }
  }
  catchAll("array::exteriorMultiplicationExec","")
}



void array::vectProductExec(array *arrayArg1,array *arrayArg2) {
  try {
    Int nbrVal1=arrayArg1->nbrVal;
    Int nbrVal2=arrayArg2->nbrVal;
    
    if (!(((getOrder()==0)&&(nbrVal1==2)&&(nbrVal2==2))
	  ||((getOrder()==1)&&(nbrVal1==3)&&(nbrVal2==3)))
	||(arrayArg1->getOrder()!=1)||(arrayArg2->getOrder()!=1))
      errThrow("array::vectProduct","   invalid arguments.");
    
    if (getOrder()==0) {
      componentValues[0]
	=arrayArg1->componentValues[0]*arrayArg2->componentValues[1]
	  -arrayArg1->componentValues[1]*arrayArg2->componentValues[0];
    }
    else if (getOrder()==1) {
      componentValues[0]
	=arrayArg1->componentValues[1]*arrayArg2->componentValues[2]
	  -arrayArg1->componentValues[2]*arrayArg2->componentValues[1];
      componentValues[1]
	=arrayArg1->componentValues[2]*arrayArg2->componentValues[0]
	  -arrayArg1->componentValues[0]*arrayArg2->componentValues[2];
      componentValues[2]
	=arrayArg1->componentValues[0]*arrayArg2->componentValues[1]
	  -arrayArg1->componentValues[1]*arrayArg2->componentValues[0];
    }
  }
  catchAll("array::vectProductExec","")
}



void array::contractedMultiplicationExec(array *arrayArg1,array *arrayArg2,
					 Int IParam1,Int IParam2) {
  try {
    Int i,j;
    
    Int arg1Indices[MAX_ORDER_ARRAY],arg2Indices[MAX_ORDER_ARRAY],
				     indices[MAX_ORDER_ARRAY];
    Int index,index1,index2;
    
    Int nbrVal1=arrayArg1->nbrVal;
    Int order1=arrayArg1->order;
    Int nbrVal2=arrayArg2->nbrVal;
    Int order2=arrayArg2->order;
    
    if (order!=order1+order2-2)
      errThrow("array::contractedMultiplicationExec",
	       string("order!=order1+order2-2 (")+strFromInt(order)+"!="
	       +strFromInt(order1+order2-2)+").");
    
    if (IParam1>=order1)
      errThrow("array::contractedMultiplicationExec",
	       string("IParam1!=order1 (")+strFromInt(IParam1)+">="
	       +strFromInt(order1)+").");
    
    if (IParam2>=order2) 
      errThrow("array::contractedMultiplicationExec",
	       string("IParam2!=order2 (")+strFromInt(IParam2)+">="
	       +strFromInt(order2)+").");
    
    if (arrayArg1->dim[IParam1]!=arrayArg2->dim[IParam2])
      errThrow("array::contractedMultiplicationExec",
	       "   incompatible number of array components.");
    
    if (nbrVal!=nbrVal1*nbrVal2/arrayArg1->dim[IParam1]
	/arrayArg2->dim[IParam2])
      errThrow("array::contractedMultiplicationExec",
	       "   incompatible number of array components.");
    
    for (i=0;i<MAX_ORDER_ARRAY;i++)
      arg1Indices[i]=arg2Indices[i]=indices[i]=-1;
    for (i=0;i<nbrVal;i++) {
      componentValues[i]=0.0;
      getIndicesFromIndex(i,indices);
      index=0;
      for (j=0;j<order1;j++)
	if (j!=IParam1) {
	arg1Indices[j]=indices[index];
	index++;
      }
      for (j=0;j<order2;j++)
	if (j!=IParam2) {
	arg2Indices[j]=indices[index];
	index++;
      }
      for (j=0;j<arrayArg1->dim[IParam1];j++) {
	arg1Indices[IParam1]=j;
	arg2Indices[IParam2]=j;
	index1=arrayArg1->getIndexFromIndices(arg1Indices);
	index2=arrayArg2->getIndexFromIndices(arg2Indices);
	componentValues[i]+=arrayArg1->componentValues[index1]
	  *arrayArg2->componentValues[index2];
      }
    }
  }
  catchAll("array::contractedMultiplicationExec","")
}



void array::accumulationExec(array *arrayArg1,array *arrayArg2,Int IParam1) {
  
  // Accumulation des IParam1 premiers indices :
  
  try {
    Int i,j,k;
    
    Int nbrVal1=arrayArg1->nbrVal;
    Int order1=arrayArg1->order;
    Int order2=arrayArg2->order;
    
    if (IParam1>order1)
      errThrow("array::accumulationExec",
	       string("IParam1!=order1 (")+strFromInt(IParam1)+">"
	       +strFromInt(order1)+").");
    
    
    if (IParam1>order2) 
      errThrow("array::accumulationExec",
	       string("IParam1!=order2 (")+strFromInt(IParam1)+">"
	       +strFromInt(order2)+").");
    
    
    if (order!=arrayArg1->order+arrayArg2->order-2*IParam1)
      errThrow("array::accumulationExec",
	       string("   order!=arrayArg1->order+arrayArg2->order-2*IParam1 (")
	       +strFromInt(order)+"!="
	       +strFromInt(arrayArg1->order+arrayArg2->order-2*IParam1)
	       +").");
    
    for (i=0;i<IParam1;i++) {
      if (arrayArg1->dim[i]!=arrayArg2->dim[i])
	errThrow("array::accumulationExec",string("   arrayArg1->dim[")
		 +strFromInt(i)+"]!=arrayArg2->dim["+strFromInt(arrayArg2->dim[i])
		 +").");
    }
    
    Int index=1;
    for (i=IParam1;i<arrayArg1->order;i++)
      index*=arrayArg1->dim[i];
    Int nbrC1=index;
    for (i=IParam1;i<arrayArg2->order;i++)
      index*=arrayArg2->dim[i];
    Int nbrC2=index/nbrC1;
    if (nbrVal!=index)
      errThrow("array::accumulationExec",string("   nbrVal!=index (")
	       +strFromInt(nbrVal)+"!="+strFromInt(index)+").");
    
    Int compression=nbrVal1/nbrC1;
    for (i=0;i<nbrC1;i++) {
      for (j=0;j<nbrC2;j++) {
	componentValues[i*nbrC2+j]=0.0;
	for (k=0;k<compression;k++)
	  componentValues[i*nbrC2+j]+=
	    arrayArg1->componentValues[k*nbrC1+i]*
	      arrayArg2->componentValues[k*nbrC2+j];
      }
    }
  }
  catchAll("array::accumulationExec","")
}



void array::substitutionExec(array *arrayArg1,array *arrayArg2,Int IParam1,
			     Int IParam2) {
  try {
    if (IParam1>arrayArg1->order)
      errThrow("array::substitutionExec",string("   IParam1>arrayArg1->order (")
	       +strFromInt(IParam1)+">"+strFromInt(arrayArg1->order)+").");
    
    if (IParam2<0||IParam2>1)
      errThrow("array::substitutionExec","   second parameter must be 0 or 1.");
    
    if (arrayArg2->getOrder()!=2)
      errThrow("array::substitutionExec",
	       "   order of the second array must be 2.");
    
    if (arrayArg2->getDim(0)!=arrayArg2->getDim(1))
      errThrow("array::substitutionExec","   the second array must be square.");
    
    if (arrayArg2->getDim(0)!=arrayArg1->getDim(IParam1))
      errThrow("array::substitutionExec",
	       "   invalid dimension in index substitution.");
    
    if (order!=arrayArg1->order)
      errThrow("array::substitutionExec",
	       string("   order!=arrayArg1->order (")+strFromInt(order)+"!="
	       +strFromInt(arrayArg1->order)+").");
    
    Int i,j;
    
    for (i=0;i<order;i++) {
      if (dim[i]!=arrayArg1->dim[i])
	errThrow("array::substitutionExec",string("   dim[")+strFromInt(i)
		 +"]!=arrayArg1->dim["+strFromInt(i)+"] ("
		 +strFromInt(arrayArg1->dim[i])+"!="
		 +strFromInt(arrayArg2->dim[i])+").");
    }
    
    Int arg1Indices[MAX_ORDER_ARRAY],arg2Indices[MAX_ORDER_ARRAY],
				     indices[MAX_ORDER_ARRAY];
    Int index1,index2;
    
    for (i=0;i<MAX_ORDER_ARRAY;i++)
      arg1Indices[i]=arg2Indices[i]=indices[i]=-1;
    
    for (i=0;i<nbrVal;i++) {
      getIndicesFromIndex(i,indices);
      for (j=0;j<order;j++)
	arg1Indices[j]=indices[j];
      arg2Indices[1-IParam2]=indices[IParam1];
      componentValues[i]=0.0;
      for (j=0;j<dim[IParam1];j++) {
	arg1Indices[IParam1]=j;
	index1=getIndexFromIndices(arg1Indices);
	arg2Indices[IParam2]=j;
	index2=getIndexFromIndices(arg2Indices);	
	componentValues[i]+=arrayArg1->componentValues[index1]
	  *arrayArg2->componentValues[index2];
      }
    }
  }
  catchAll("array::substitutionExec","");
}
