
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <iomanip>

#include "arrayList.h"
#include "array_base.h"
#include "err/err.h"

using namespace std ;
using namespace keyFE2 ;



arrayList::arrayList(void) {
  capacity=10;
  size=0;
  if (capacity>0)
    data=new array* [capacity];
  else
    data=NULL;
}



arrayList::arrayList(const arrayList &in) {
  capacity=2*in.size;
  size=in.size;
  if (capacity>0)
    data=new array* [capacity];
  else
    data=NULL;
  for (Int i=0;i<size;i++)
    data[i]=in.data[i];
}



arrayList::~arrayList(void) {
  capacity=0;
  size=0;
  delete[] data;
  data=NULL;
}



void arrayList::init(void) {
  size=0;
}



void arrayList::pushBack(array *in) {
  if (size==capacity) {
    array **tmpData=data;
    capacity=2*(size+1);
    if (capacity>0)
      data=new  array* [capacity];
    else
      data=NULL;
    for (Int i=0;i<size;i++)
      data[i]=tmpData[i];
    delete[] tmpData;
  }
  data[size]=in;
  size++;
}



void arrayList::insert(array *in) {
  for (Int i=0;i<size;i++)
    if (in==data[i])
      return;
  pushBack(in);
}



inline void swap(array *arg1,array *arg2) {
  array *tmp=arg1;
  arg1=arg2;
  arg2=tmp;
}



void arrayList::sort(void) {
  // On se satisfait pour l'instant d'un tri  bulles :
  Int i,j;
  for (i=size-1;i>0;i--)
    for (j=0;j<i;j++)
      if (data[j]<data[j+1])
	swap(data[j],data[j+1]);
}



void arrayList::write(ostream &myCout) const {
   for (Int i=0;i<size;i++)
     myCout << data[i] << endl ;
}



void arrayList::operator=(const arrayList &in) {
  if (this==&in)
    return;
  
  capacity=2*in.size;
  size=in.size;
  delete[] data;
  if (capacity>0)
    data=new array* [capacity];
  else
    data=NULL;
  for (Int i=0;i<size;i++)
    data[i]=in.data[i];
}



void arrayList::operator+=(const arrayList &in) {
  arrayList output=(*this)+in;
  (*this)=output;
}



arrayList arrayList::operator+(const arrayList &in) {
  Int index1=0;
  Int index2=0;
  arrayList output;
  
  while (index1<size&&index2<in.size) {
    if (data[index1]==in.data[index2]) {
      output.pushBack(data[index1]);
      index1++;
      index2++;
    }
    else if (data[index1]<in.data[index2]) {
      output.pushBack(data[index1]);
      index1++;
    }
    else {
      output.pushBack(in.data[index2]);
      index2++;
    }
  }
  
  while (index1<size)
    output.pushBack(data[index1++]);
  
  while (index2<in.size)
    output.pushBack(in.data[index2++]);
  
  return(output);
}



array* arrayList::operator[](Int index) const {
  if (index<0||index>size) 
    errThrow("arrayList::operator[]",
	     string("   invalid index : ")+strFromInt(index)+".");

  return(data[index]);
}




