#ifndef GLOBLOC_H
#define GLOBLOC_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <string>

#include "types/types.h"

namespace keyFE2 {
  
  class Array ;
  class TensorCoordSys ;
  class globLoc ;
  class param ;
  class paramList ;
  
  class GlobLoc {
    
    private :
      
      void *data ;
    
      static GlobLoc defaultGL ;
    
    public :
      
      GlobLoc(void) ;
      GlobLoc(std::string str) ;
      GlobLoc(param P) ;
      GlobLoc(paramList PL) ;
      GlobLoc(const GlobLoc &) ;
      ~GlobLoc(void) ;
      
      void setData(globLoc*) ;
      globLoc* getData(void) const ;
      void setName(std::string s) ;
      std::string getName(void) const ;
      
      GlobLoc operator=(const GlobLoc &) ;
      
      std::string getTypeName(void) const ;
      
      static Int getGlobDimFromName(std::string &name) ;
      static Int getLocDimFromName(std::string &name) ;
      
      static void setDefaultGL(const GlobLoc &GL) ;
      static void unsetDefaultGL(void) ;
      static GlobLoc getDefaultGL(void) ;
      
      Array& getNormal(const TensorCoordSys &TCS) const ;
      Array& getTangeant(const TensorCoordSys &TCS) const ;
      Array& getVector1(const TensorCoordSys &TCS) const ;
      Array& getVector2(const TensorCoordSys &TCS) const ;
      Array& getVector3(const TensorCoordSys &TCS) const ;
      
      Array& getNormal(void) const ;
      Array& getTangeant(void) const ;
      Array& getVector1(void) const ;
      Array& getVector2(void) const ;
      Array& getVector3(void) const ;
      
      void write(void) const ;
      void writeValues(void) const ;
      void write(std::ostream &out) const ;
      void writeValues(std::ostream &out) const ;
      
  } ;
  
}

#endif
