
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <iomanip>
#include <typeinfo>
 
#include "TensorCoordSys.h"
#include "tensorCoordSys_base.h"
#include "array/Array.h"
#include "array/array_base.h"
#include "err/err.h"
#include "param/param.h"
#include "param/paramList.h"

using namespace std ;
using namespace keyFE2 ;



TensorCoordSys::TensorCoordSys(void) {
  try {
    tensorCoordSys *TCS=new tensorCoordSys;
    data=NULL;
    setData(TCS);
  }
  catchAll("TensorCoordSys::TensorCoordSys(void)","")
}



TensorCoordSys::TensorCoordSys(const TensorCoordSys &in) {
  try {
    data=NULL;
    setData(in.getData());
  }
  catchAll("TensorCoordSys::TensorCoordSys(const TensorCoordSys &in)","")
}



TensorCoordSys::TensorCoordSys(param P) {
  try {
    paramList PL;
    PL.pushBack(P);
    TensorCoordSys TCS(PL);
    data=NULL;
    setData(TCS.getData());
  }
  catchAll("TensorCoordSys::TensorCoordSys(param P)","")
}



TensorCoordSys::TensorCoordSys(paramList PL) {
  try {
    string tcsName="";
    string tcsType="";
    array *coord=NULL;
    
    paramList plName=PL.getListOfType(NAME);
    PL=PL.getListNotOfType(NAME);
    paramList plTCST=PL.getListOfType(TCSTYPE);
    PL=PL.getListNotOfType(TCSTYPE);
    paramList plCoord=PL.getListOfType(COORD);
    PL=PL.getListNotOfType(COORD);
    
    // Checks for illegal arguments :
    
    if (PL.getNbrParams()!=0) {
      string tmpString=strFromInt(PL.getNbrParams())+" invalid parameters :";
      for (Int i=0;i<PL.getNbrParams();i++) {
	tmpString+=" ";
	tmpString+=PL[i].getName();
      }
      tmpString+=".\n";
      errWrite("TensorCoordSys construction",tmpString);
    }
    
    // Sets the name of the tensorCoordSys :
    
    if (plName.getNbrParams()>1) {
      errWrite("TensorCoordSys construction",
	       "only one NAME argument is allowed.");
    }
    else if (plName.getNbrParams()==1) {
      tcsName=plName[0].getStringData();
      if (tcsName=="")
	errWrite("TensorCoordSys construction",
		 "NAME= has to be followed by a string argument.");
    }
    
    // Sets the type of the globloc :
    
    if (plTCST.getNbrParams()!=1) {
      errWrite("TensorCoordSys construction",
	       "exactly one GLTYPE argument is needed.");
    }
    else {
      tcsType=plTCST[0].getStringData();
      if (tcsType=="")
	errWrite("TensorCoordSys construction",
		 "TCSTYPE= has to be followed by a string argument.");
    }
    
    // Gets the coordinates of the globloc :
    
    if (plCoord.getNbrParams()!=1) {
      errWrite("TensorCoordSys construction",
	       "exactly one COORD argument is needed.");
    }
    else {
      parents *tmpParent=plCoord[0].getParentData();
      coord=dynamic_cast<array*>(tmpParent);
      if (coord==NULL)
	errWrite("TensorCoordSys construction",
		 "COORD= has to be followed by an Array argument.");
    }
    
    if (!err::getOk())
      errThrow("TensorCoordSys construction","bailing out due to preceding errors.");
    
    if (tcsType!="Cartesian1D"&&tcsType!="Cartesian2D"
	&&tcsType!="Cartesian3D"&&tcsType!="CylindricalRZT")
      errThrow("TensorCoordSys construction","illegal TCSTYPE: "+tcsType+".");
    
    if (coord->getOrder()!=1)
      errThrow("TensorCoordSys construction","invalid order of coord array.");

    if (tcsType=="Cartesian1D"&&coord->getDim(0)!=1) {
      errThrow("TensorCoordSys construction","dim of coord array must be 1.");
    }
    else if (tcsType=="Cartesian2D"&&coord->getDim(0)!=2) {
      errThrow("TensorCoordSys construction","dim of coord array must be 2.");
    }
    else if ((tcsType=="Cartesian3D"||tcsType=="CartesianRZT")
	     &&coord->getDim(0)!=3) {
      errThrow("TensorCoordSys construction","dim of coord array must be 3.");
    }
    
    tensorCoordSys *outputTCS;
    if (tcsType=="Cartesian1D")
      outputTCS=build1DCartesianTCS(coord);
    else if (tcsType=="Cartesian2D")
      outputTCS=build2DCartesianTCS(coord);
    else if (tcsType=="Cartesian3D")
      outputTCS=build3DCartesianTCS(coord);
    else if (tcsType=="CylindricalRZT")
      outputTCS=buildRZTCylindricalTCS(coord);
    else
      errThrow("TensorCoordSys construction",
	       string("illegal TCSTYPE argument : ")+tcsType+".");
    if (tcsName!="");
    outputTCS->setName(tcsName);
    
    data=NULL;
    setData(outputTCS);
  }
  catchAll("TensorCoordSys::TensorCoordSys(paramList PL)","")
}



TensorCoordSys::~TensorCoordSys(void) {
  try {
    setData(NULL);
  }
  catchAll("TensorCoordSys::~TensorCoordSys(void)","")
}



void TensorCoordSys::setData(tensorCoordSys *newTCS) {
  try {
    tensorCoordSys *TCS=getData();
    delinkAndDelete(TCS,&data);
    if (newTCS!=NULL)
      newTCS->addParent(&data);
    data=static_cast<void*>(newTCS);
  }
  catchAll("TensorCoordSys::setData","")
}



tensorCoordSys* TensorCoordSys::getData(void) const {
  return(static_cast<tensorCoordSys*>(data));
}



void TensorCoordSys::setName(string s) {
  try {
    tensorCoordSys *TCS=getData();
    TCS->setName(s);
  }
  catchAll("TensorCoordSys::setName","")
}



string TensorCoordSys::getName(void) const {
  try {
    tensorCoordSys *TCS=getData();
    return(TCS->getName());
  }
  catchAll("TensorCoordSys::getName","")
}



void TensorCoordSys::write(void) const {
  try {
    tensorCoordSys *tensorCoordSysData=getData();
    tensorCoordSysData->write();
  }
  catchAll("TensorCoordSys::write","");
}



void TensorCoordSys::writeValues(void) const {
  try {
    tensorCoordSys *tensorCoordSysData=getData();
    tensorCoordSysData->writeValues();
  }
  catchAll("TensorCoordSys::writeValues","");
}



void TensorCoordSys::write(ostream &out) const {
  try {
    tensorCoordSys *tensorCoordSysData=getData();
    tensorCoordSysData->write(out);
  }
  catchAll("TensorCoordSys::write","");
}



void TensorCoordSys::writeValues(ostream &out) const {
  try {
    tensorCoordSys *tensorCoordSysData=getData();
    tensorCoordSysData->writeValues(out);
  }
  catchAll("TensorCoordSys::writeValues","");
}



void TensorCoordSys::operator=(const TensorCoordSys &in) {
  if (this==&in)
    return;
  
  try {
    setData(in.getData());
  }
  catchAll("TensorCoordSys::operator=","")
} 



TensorCoordSys build1DCartesianTCS(Array &Coord) {
  try {
    array* coord=Coord.getData();
    TensorCoordSys TCS;
    tensorCoordSys *tcs;
    tcs=build1DCartesianTCS(coord);
    TCS.setData(tcs);
    
    return(TCS);
  }
  catchAll("build1DCartesianTCS","")
}



TensorCoordSys build2DCartesianTCS(Array &Coord) {
  try {
    array* coord=Coord.getData();
    TensorCoordSys TCS;
    tensorCoordSys *tcs;
    tcs=build2DCartesianTCS(coord);
    TCS.setData(tcs);
    
    return(TCS);
  }
  catchAll("build2DCartesianTCS","")
}



TensorCoordSys build3DCartesianTCS(Array &Coord) {
  try {
    array* coord=Coord.getData();
    TensorCoordSys TCS;
    tensorCoordSys *tcs;
    tcs=build3DCartesianTCS(coord);
    TCS.setData(tcs);
    
    return(TCS);
  }
  catchAll("build3DCartesianTCS","")
}



TensorCoordSys buildRZTCylindricalTCS(Array &Coord) {
  try {
    array* coord=Coord.getData();
    TensorCoordSys TCS;
    tensorCoordSys *tcs;
    tcs=buildRZTCylindricalTCS(coord);
    TCS.setData(tcs);
    
    return(TCS);
  }
  catchAll("buildRZTCylindricalTCS","")
}



