
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <iomanip>
#include <typeinfo>
#include <string>
#include <stdio.h>

#include "globLoc_base.h"
#include "array/array_base.h"
#include "array/arrayComp.h"
#include "array/arrayFunctionsBuild.h"
#include "err/err.h"
#include "function/functions.h"

using namespace std ;
using namespace keyFE2 ;



namespace keyFE2 {
  
  
  
  globLoc* build0D0DGlobLoc(array *Coord) {
    try {
      globLoc *GL;
      
      GL=new globLoc;
      GL->setType(7);
      GL->setName("globloc: 0D");
      
      GL->setCoord(Coord);
      
      return(GL);
    }
    catchAll("build0D0DGlobLoc","")
  }
  
  
  
  globLoc* build1D1DGlobLoc(array *Coord) {
    try {
      array *tmp;
      globLoc *GL;
      
      if (Coord->getOrder()!=1)
	errThrow("build1D1DGlobLoc","coordinate array must be of order 1.");
      if (Coord->getDim(0)!=1)
	errThrow("build1D1DGlobLoc",
		 "number of components of coordinate array must be 1.");
      
      GL=new globLoc;
      GL->setType(1);
      GL->setName("globloc: 1D1D");
      
      GL->setCoord(Coord);
      
      tmp=Coord->makeCorlocDeriv(1);
      GL->setCoordDeriv(tmp);
      
      tmp=new array;
      tmp->setOrder(2);
      tmp->setDim(0,1);
      tmp->setDim(1,1);
      GL->setJac(tmp);
      
      tmp=new array;
      tmp->setOrder(2);
      tmp->setDim(0,1);
      tmp->setDim(1,1);
      GL->setInvJac(tmp);
      
      return(GL);
    }
    catchAll("build1D1DGlobLoc","")
  }
  
  
  
  globLoc* build1D0DGlobLoc(array *Coord) {
    try {
      globLoc *GL;
      
      if (Coord->getOrder()!=1)
	errThrow("build1D0DGlobLoc","coordinate array must be of order 1.");
      if (Coord->getDim(0)!=1)
	errThrow("build1D0DGlobLoc",
		 "number of components of coordinate array must be 1.");
      
      GL=new globLoc;
      GL->setType(8);
      GL->setName("globloc: 1D0D");
      
      GL->setCoord(Coord);
      
      return(GL);
    }
    catchAll("build1D0DGlobLoc","")
  }
  
  
  
  globLoc* build2D2DGlobLoc(array *Coord) {
    try {
      array *tmp;
      globLoc *GL;
      
      if (Coord->getOrder()!=1)
	errThrow("build2D2DGlobLoc","coordinate array must be of order 1.");
      if (Coord->getDim(0)!=2)
	errThrow("build2D2DGlobLoc",
		 "number of components of coordinate array must be 2.");
      
      GL=new globLoc;
      GL->setType(2);
      GL->setName("globloc: 2D2D");
      
      GL->setCoord(Coord);
      
      tmp=Coord->makeCorlocDeriv(2);
      GL->setCoordDeriv(tmp);
      
      tmp=new array;
      tmp->setOrder(2);
      tmp->setDim(0,2);
      tmp->setDim(1,2);
      GL->setJac(tmp);
      
      tmp=new array;
      tmp->setOrder(2);
      tmp->setDim(0,2);
      tmp->setDim(1,2);
      GL->setInvJac(tmp);
      
      return(GL);
    }
    catchAll("build2D2DGlobLoc","")
  }
  
  
  
  globLoc* build2D1DGlobLoc(array *Coord) {
    try {
      array *tmp;
      globLoc *GL;
      
      if (Coord->getOrder()!=1)
	errThrow("build2D1DGlobLoc",
		 "coordinate array must be of order 1.");
      if (Coord->getDim(0)!=2)
	errThrow("build2D1DGlobLoc",
		 "number of components of coordinate array must be 2.");
      
      GL=new globLoc;
      GL->setType(3);
      GL->setName("globloc: 2D1D");
      
      GL->setCoord(Coord);
      
      tmp=Coord->makeCorlocDeriv(1);
      GL->setCoordDeriv(tmp);
      
      tmp=new array;
      tmp->setOrder(2);
      tmp->setDim(0,2);
      tmp->setDim(1,2);
      GL->setJac(tmp);
      
      tmp=new array;
      tmp->setOrder(2);
      tmp->setDim(0,2);
      tmp->setDim(1,2);
      GL->setInvJac(tmp);
      
      return(GL);
    }
    catchAll("build2D1DGlobLoc","")
  }
  
  
  
  globLoc* build2D0DGlobLoc(array *Coord) {
    try {
      globLoc *GL;
      
      if (Coord->getOrder()!=1)
	errThrow("build2D0DGlobLoc",
		 "coordinate array must be of order 1.");
      if (Coord->getDim(0)!=2)
	errThrow("build2D0DGlobLoc",
		 "number of components of coordinate array must be 2.");
      
      GL=new globLoc;
      GL->setType(9);
      GL->setName("globloc: 2D0D");
      
      GL->setCoord(Coord);

      return(GL);
    }
    catchAll("build2D0DGlobLoc","")
  }
  
  
  
  globLoc* build3D3DGlobLoc(array *Coord) {
    try {
      array *tmp;
      globLoc *GL;
      
      if (Coord->getOrder()!=1)
	errThrow("build3D3DGlobLoc","coordinate array must be of order 1.");
      if (Coord->getDim(0)!=3)
	errThrow("build3D3DGlobLoc",
		 "number of components of coordinate array must be 3.");
      
      GL=new globLoc;
      GL->setType(4);
      GL->setName("globloc: 3D3D");
      
      GL->setCoord(Coord);
      
      tmp=Coord->makeCorlocDeriv(3);
      GL->setCoordDeriv(tmp);
      
      tmp=new array;
      tmp->setOrder(2);
      tmp->setDim(0,3);
      tmp->setDim(1,3);
      GL->setJac(tmp);
      
      tmp=new array;
      tmp->setOrder(2);
      tmp->setDim(0,3);
      tmp->setDim(1,3);
      GL->setInvJac(tmp);
      
      return(GL);
    }
    catchAll("build3D3DGlobLoc","")
  }
  
  
  
  
  globLoc* build3D2DGlobLoc(array *Coord) {
    // One makes the assumption that only the two first coordinates are not 
    //    zero (i.e. domain in plane XY).
    try {
      array *tmp;
      globLoc *GL;
      
      if (Coord->getOrder()!=1)
	errThrow("build3D2DGlobLoc","coordinate array must be of order 1.");
      if (Coord->getDim(0)!=3)
	errThrow("build3D2DGlobLoc",
		 "number of components of coordinate array must be 3.");
      
      GL=new globLoc;  
      GL->setType(5);
      GL->setName("globloc: 3D2D");
      
      GL->setCoord(Coord);
      
      tmp=Coord->makeCorlocDeriv(2);
      GL->setCoordDeriv(tmp);
      
      tmp=new array;
      tmp->setOrder(2);
      tmp->setDim(0,3);
      tmp->setDim(1,3);
      GL->setJac(tmp);
      
      tmp=new array;
      tmp->setOrder(2);
      tmp->setDim(0,3);
      tmp->setDim(1,3);
      GL->setInvJac(tmp);
      
      return(GL);
    }
    catchAll("build3D2DGlobLoc","")
  }
  
  
  
  
  globLoc* build3D1DGlobLoc(array *Coord) {
    // One makes the assumption that only the two first coordinates are not 
    //    zero (i.e. domain in plane XY).
    try {
      array *tmp;
      globLoc *GL;
      
      if (Coord->getOrder()!=1)      
	errThrow("build3D1DGlobLoc",
		 "coordinate array must be of order 1.");
      if (Coord->getDim(0)!=3)
	errThrow("build3D1DGlobLoc",
		 "number of components of coordinate array must be 3.");
      
      GL=new globLoc;
      GL->setType(6);
      GL->setName("globloc: 3D1D");
      
      GL->setCoord(Coord);
      
      tmp=Coord->makeCorlocDeriv(1);
      GL->setCoordDeriv(tmp);
      
      tmp=new array;
      tmp->setOrder(2);
      tmp->setDim(0,3);
      tmp->setDim(1,3);
      GL->setJac(tmp);
      
      tmp=new array;
      tmp->setOrder(2);
      tmp->setDim(0,3);
      tmp->setDim(1,3);
      GL->setInvJac(tmp);
      
      return(GL);
    }
    catchAll("build3D1DGlobLoc","")
  }
  
  
  
  globLoc* build3D0DGlobLoc(array *Coord) {
    // One makes the assumption that only the two first coordinates are not 
    //    zero (i.e. domain in plane XY).
    try {
      globLoc *GL;
      
      if (Coord->getOrder()!=1)      
	errThrow("build3D0DGlobLoc",
		 "coordinate array must be of order 1.");
      if (Coord->getDim(0)!=3)
	errThrow("build3D0DGlobLoc",
		 "number of components of coordinate array must be 3.");
      
      GL=new globLoc;
      GL->setType(10);
      GL->setName("globloc: 3D0D");
      
      GL->setCoord(Coord);

      return(GL);
    }
    catchAll("build3D0DGlobLoc","")
  }
  
  
  
}

