#ifndef TENSOR_COORDSYS_BASE_H
#define TENSOR_COORDSYS_BASE_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <string>

#include "types/params.h"
#include "types/types.h"

#include "print/print.h"
#include "util/name.h"
#include "util/newPoint.h"
#include "util/parents.h"

namespace keyFE2 {
  
  class array;
  class object;
  class scal;
  class printTree;
  
  class tensorCoordSys : public virtual name, public virtual newPoint, 
			 public virtual parents, public virtual print {
									
    friend class array;
    
    private :
      
      array *Coord;
      Int CoordComponentsNbr;
      scal **CoordComponents;
      
      array *CovaMetric;
      array *ContraMetric;
      
      array *Christoffel;
      
      Int type;
      
    public :
      
      void init(void);
    
      tensorCoordSys(void);
      tensorCoordSys(const tensorCoordSys &);
      ~tensorCoordSys(void);
      
      void setCoord(array*);
      array* getCoord(void) const;
      Int getCoordComponentsNbr(void) const ;
      scal** getCoordComponents(void) const ;
      scal* getCoordComponents(Int index) const ;
      
      void setCovaMetric(array*);
      array* getCovaMetric(void) const;
      void setContraMetric(array*);
      array* getContraMetric(void) const;
      
      void setChristoffel(array*);
      array* getChristoffel(void) const;
      
      void setType(Int) ;
      void setType(std::string) ;
      Int getType(void) const ;
      std::string getTypeName(void) const ;
      
      tensorCoordSys *makeCopyForCoordSys(void) const ;
      void makeAtPoint(object *element,Int direct,Double *localCoord,
		       bool makeValue=true,bool makeDeriv=true);
      
      void setNewPoint(bool);
      void accessNbrInit(void);
      
      virtual printTree getPrintTreeData(void) const ;
      virtual printTree getPrintTreeValues(void) const ;
      virtual printTree getPrintTree(bool values) const ;
      
  } ;
  
  tensorCoordSys* build1DCartesianTCS(array *coord);
  tensorCoordSys* build2DCartesianTCS(array *coord);
  tensorCoordSys* build3DCartesianTCS(array *coord);
  tensorCoordSys* buildRZTCylindricalTCS(array *coord);
  
}

#endif




