
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//


#include "Domain.h"
#include "domain_base.h"

#include "err/err.h"

using namespace std ;
using namespace keyFE2 ;



Domain::Domain(void) {
  try {
    domain *dom=new domain;
    data=NULL;
    setData(dom);
  }
  catchAll("Domain::Domain(void)","")
}



Domain::Domain(string str) {
  try {
    domain *dom=new domain(str);
    data=NULL;
    setData(dom);
  }
  catchAll("Domain::Domain(string str)","")
}



Domain::Domain(const Domain &DomIn) {
  try {
    data=NULL;
    setData(DomIn.getData());
  }
  catchAll("Domain::Domain(const Domain &DomIn)","")
}



Domain::~Domain(void) {
  try {
    setData(NULL);
  }
  catchAll("Domain::~Domain(void)","")
}



void Domain::setData(domain *newDomainData) {
  try {
    domain *dom=getData();
    delinkAndDelete(dom,&data);
    if (newDomainData!=NULL)
      newDomainData->addParent(&data);
    data=static_cast<void*>(newDomainData);
  }
  catchAll("void Domain::setData(domain *newDomainData)","")
}



domain* Domain::getData(void) const {
  return(static_cast<domain*>(data));
}



void Domain::setName(string str) {
  try {
    domain *dom=getData();
    dom->setName(str);
  }
  catchAll("void Domain::setName(string str)","")
}



string Domain::getName(void) const {
  try {
    return(getData()->getName());
  }
  catchAll("string Domain::getName(void) const","")
}



Int Domain::getDim(void) const {
  try {
    return(getData()->getDim());
  }
  catchAll("Int Domain::getDim(void) const","")
}



mesh* Domain::getBaseMesh(void) const {
  try {
    return(getData()->getBaseMesh());
  }
  catchAll("mesh* Domain::getBaseMesh(void) const","")
}



Int Domain::getObjectsNbr(void) const {
  try {
    return(getData()->getObjectsNbr());
  }
  catchAll("Int Domain::getObjectsNbr(void) const","")
}



Object Domain::getObject(Int index) const {
  try {
    Object obj;
    obj.setData(getData()->getObject(index));
    obj.setDirect(getData()->getObjectDirect(index));
    return obj;
  }
  catchAll("Object Domain::getObject(Int index) const","")
}



//object* Domain::getObject(Int index) const {
//  try {
//    return(getData()->getObject(index));
//  }
//  catchAll("object* Domain::getObject(Int index) const","")
//}



//Int Domain::getObjectDirect(Int index) const {
//  try {
//    return(getData()->getObjectDirect(index));
//  }
//  catchAll("Int Domain::getObjectDirect(Int index) const","")
//}



Int Domain::getAllObjectsNbr(void) const {
  try {
    return(getData()->getAllObjectsNbr());
  }
  catchAll("Int Domain::getAllObjectsNbr(void) const","")
}



object* Domain::getAllObject(Int index) const {  
  try {
    return(getData()->getAllObject(index));
  }
  catchAll("object* Domain::getAllObject(Int index) const","")
}



Int Domain::getAllObject(object *obj) const {
  try {
    return(getData()->getAllObject(obj));
  }
  catchAll("Int Domain::getAllObject(object *obj) const","")
}



void Domain::makeAllObjectsLst(void) {
  try {
    getData()->makeAllObjectsLst();
  }
  catchAll("void Domain::makeAllObjectsLst(void)","")
}



void Domain::write(void) const {
  try {
    domain *domainData=getData();
    domainData->write();
  }
  catchAll("Domain::write","");
}



void Domain::writeValues(void) const {
  try {
    domain *domainData=getData();
    domainData->writeValues();
  }
  catchAll("Domain::writeValues","");
}



void Domain::write(ostream &out) const {
  try {
    domain *domainData=getData();
    domainData->write(out);
  }
  catchAll("Domain::write","");
}



void Domain::writeValues(ostream &out) const {
  try {
    domain *domainData=getData();
    domainData->writeValues(out);
  }
  catchAll("Domain::writeValues","");
}



void Domain::splitWrite(ostream &myCout) {
  try {
    getData()->splitWrite(myCout);
  }
  catchAll("void Domain::splitWrite(ostream &myCout)","")
}



void Domain::splitWrite(string outFile) {
  try {
    getData()->splitWrite(outFile);
  }
  catchAll("void Domain::splitWrite(string outFile)","")
}



void Domain::operator=(const Domain &Darg) {
  if (this==&Darg)
    return;
  
  try {
    setData(Darg.getData());
  }
  catchAll("void Domain::operator=(const Domain &Darg)","")
}



Domain Domain::operator+(const Domain &Darg) const {
  try {
    domain *dom1=getData();
    domain *dom2=Darg.getData();
    domain dom=(*dom1)+(*dom2);
    domain *domOut=new domain(dom);
    Domain Dom;
    Dom.setData(domOut);
    return(Dom);
  }
  catchAll("Domain Domain::operator+(const Domain &Darg) const","")
}



Domain Domain::operator-(const Domain &Darg) const {
  try {
    domain *dom1=getData();
    domain *dom2=Darg.getData();
    domain dom=(*dom1)-(*dom2);
    domain *domOut=new domain(dom);
    Domain Dom;
    Dom.setData(domOut);
    return(Dom);
  }
  catchAll("Domain Domain::operator-(const Domain &Darg) const","")
}



Domain Domain::operator-() const {
  try {
    domain dom=-(*(getData()));
    domain *domOut=new domain(dom);
    Domain Dom;
    Dom.setData(domOut);
    return(Dom);
  }
  catchAll("Domain Domain::operator-() const","")
}



Domain Domain::operator*(const Domain &Darg) const {
  try {
    domain *dom1=getData();
    domain *dom2=Darg.getData();
    domain dom=(*dom1)*(*dom2);
    domain *domOut=new domain(dom);
    Domain Dom;
    Dom.setData(domOut);
    return(Dom);
  }
  catchAll("Domain Domain::operator*(const Domain &Darg) const","")
}



Domain Domain::getBoundary(void) const {
  try {
    domain* dom1=getData();
    domain dom2=dom1->getBoundary();
    domain *domOut=new domain(dom2);
    Domain Dom;
    Dom.setData(domOut);
    return(Dom);
  }
  catchAll("Domain Domain::getBoundary(void) const","")
}



void Domain::getExtremities(Domain &D1,Domain &D2) const {
  try {
    domain *tmpDom=getData();
    if (tmpDom==NULL)
      errThrow("Domain::getExtremities","Domain must be made of a single open line.");
    
    domain *dom1=new domain;
    domain *dom2=new domain;
    tmpDom->getExtremities(*dom1,*dom2);
    
    D1.setData(dom1);
    D2.setData(dom2);
  }
  catchAll("Domain Domain::getExtremities(void) const","")
}



Domain Domain::getFirstNode(void) const {
  Domain Dom;
  try {
    domain* dom1=getData();
    domain* dom2=dom1->getFirstNode();
    Dom.setData(dom2);
    return(Dom);
  }
  catchAll("Domain Domain::getFirstNode(void) const","")
}



