
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iomanip>
#include <list>
#include <map>

#include "domain_base.h"
#include "err/err.h"
#include "object/object_base.h"
#include "object/objectTypes.h"

using namespace std ;
using namespace keyFE2 ;



inline Int sign(Int a) {
  if (a>0) return 1 ;
  if (a<0) return -1 ;
  return 0 ;
}



void domain::reorder1DDomain(const domain &arg) {
  
  try {
    
    // Only 1D domains composed of a single line succession of edges is 
    // reordered.
    
    if (arg.dim!=1)
      errThrow("domain::reorder1DDomain",
	       string("dimension of the domain should be 1 and not ")
	       +strFromInt(arg.dim)+".");
    Int i;
    object *tmpEdge;
    map<object*,Int> nodeList;
    map<object*,object*> nodeListDown;
    map<object*,Int> nodeEdge;
    for (i=0;i<arg.objectsNbr;i++) {
      tmpEdge=arg.objectsLst[i];
      nodeList[tmpEdge->minusLst[0]]++;
      nodeList[tmpEdge->minusLst[1]]++;
      if (arg.objectsDirect[i]==2) {
	nodeEdge[tmpEdge->minusLst[0]]=i;
	nodeListDown[tmpEdge->minusLst[0]]=tmpEdge->minusLst[1];
      }
      else if (arg.objectsDirect[i]==-2) {
	nodeEdge[tmpEdge->minusLst[1]]=i;
	nodeListDown[tmpEdge->minusLst[1]]=tmpEdge->minusLst[0];
      }
      else
	errThrow("domain::reorder1DDomain",
		 string("invalid edge direction : ")
		 +strFromInt(arg.objectsDirect[i])+".");
    }
    
    map<object*,Int>::iterator nodIt;
    list<object*> firstNodes;
    for (nodIt=nodeList.begin();nodIt!=nodeList.end();nodIt++) {
      if (nodIt->second==1&&nodeListDown.find(nodIt->first)!=nodeListDown.end())
	firstNodes.push_back(nodIt->first);
      if (nodIt->second>2) {
	// One fait rien s'il y a des branchements dans le domaine 1D.
	operator=(arg);
	return;
      }
    }
    firstNodes.sort();
    
    if (firstNodes.size()<=1) {
      object *node1=NULL;
      Int counter,index;
      
      // On initialise this  arg :
      operator=(arg);
      
      // On dtermine le premier noeud du nouveau domaine :
      if (firstNodes.size()==1) {
	// domain ouvert :
	node1=*(firstNodes.begin());
      }
      else {
	// domaine ferm :
	node1=nodeListDown[(nodeListDown.begin())->first];
      }
      
      counter=0;
      while(nodeListDown.find(node1)!=nodeListDown.end()&&counter!=objectsNbr) {
	index=nodeEdge[node1];
	setObject(counter,arg.objectsLst[index],arg.objectsDirect[index]);
	node1=nodeListDown[node1];
	counter++;
      }
      if (counter!=objectsNbr)
	errThrow("domain::reorder1DDomain","counter error in reordering.");
    }
    else {
      // On ne trie pas les domaines 1D non connexes :
      operator=(arg);
      return;
    }
  }
  catchAll("void domain::reorder1DDomain(const domain &arg)","")
}


Int domain::setDim(void) {
  try {
    Int nbr=getObjectsNbr();  
    if (dim==-1&&nbr>0) {
      dim=objectsLst[0]->getDim();
      for (Int i=0;i<nbr;i++)
	if (dim!=objectsLst[i]->getDim())
	  errThrow("domain::setDim","all objects must have identical dimension.");
    }  
    return(dim);
  }
  catchAll("Int domain::setDim(void)","")
}



Int domain::getDim(void) const {
  return(dim);
}



void domain::operator=(const domain& dom) {
  try {
    if (this!=&dom) {
      baseMesh=dom.baseMesh;
      
      dim=dom.dim;
      objectsNbr=dom.objectsNbr;
      if (objectsLst!=NULL)
	delete[] objectsLst;
      if (objectsDirect!=NULL)
	delete[] objectsDirect;
      if (objectsNbr!=0) {
	objectsLst=new object* [objectsNbr];
	objectsDirect=new Int[objectsNbr];
      }
      else {
	objectsLst=NULL;
	objectsDirect=NULL;
      }
      for (Int i=0;i<objectsNbr;i++) {
	objectsLst[i]=dom.getObject(i);
	objectsDirect[i]=dom.getObjectDirect(i);
      }
      
      makeAllObjectsLst();
    }
  }
  catchAll("domain::operator=(const domain& dom)","")
}



domain domain::operator+(const domain &dom2) const {
  try {
    Int dim1=getDim();
    Int dim2=dom2.getDim();
    domain newDom;
    object *obj;
    
    if (dim1!=dim2)
      return(newDom);
    if (baseMesh!=dom2.baseMesh)
      return(newDom);
    
    map<object*,Int> elements;
    map<object*,Int>::iterator ELTSidx,ELTSend;
    
    Int nbr=getObjectsNbr();
    for (Int i=0;i<nbr;i++)
      elements[getObject(i)]=getObjectDirect(i);
    
    nbr=dom2.getObjectsNbr();
    for (Int i=0;i<nbr;i++) {
      obj=dom2.getObject(i);
      if (elements.find(obj)==elements.end())
	elements[obj]=dom2.getObjectDirect(i);
      else if (sign(elements[obj])!=sign(dom2.getObjectDirect(i)))
	elements.erase(elements.find(obj));
    }
    
    ELTSidx=elements.begin();
    ELTSend=elements.end();
    Int index=0;
    newDom.setObjectsNbr(elements.size());
    while (ELTSidx!=ELTSend) {
      newDom.setObject(index,(*ELTSidx).first,(*ELTSidx).second);
      index++;
      ELTSidx++;
    }
    
    newDom.setDim();
    newDom.makeAllObjectsLst();
    newDom.baseMesh=baseMesh;
    
    if (newDom.getDim()==1) {
      domain tmp;
      tmp.reorder1DDomain(newDom);
      return(tmp);
    }
    
    return(newDom); 
  }
  catchAll("domain domain::operator+(const domain &dom2) const","")
}



domain domain::operator-(const domain &dom2) const {
  try {
    Int dim1=getDim();
    Int dim2=dom2.getDim();
    domain newDom;
    object *obj;
    
    if (dim1!=dim2)
      return(newDom);
    if (baseMesh!=dom2.baseMesh)
      return(newDom);
    
    map<object*,Int> elements;
    map<object*,Int>::iterator ELTSidx,ELTSend;
    
    Int nbr=getObjectsNbr();
    for (Int i=0;i<nbr;i++)
      elements[getObject(i)]=getObjectDirect(i);
    
    nbr=dom2.getObjectsNbr();
    for (Int i=0;i<nbr;i++) {
      obj=dom2.getObject(i);
      ELTSidx=elements.find(obj);
      ELTSend=elements.end();
      if (ELTSidx!=ELTSend) {
	if (sign((*ELTSidx).second)==sign(dom2.getObjectDirect(i)))
	  elements.erase(ELTSidx);
      }
      else
	elements[obj]=(-dom2.getObjectDirect(i));
    }
    
    ELTSidx=elements.begin();
    ELTSend=elements.end();
    Int index=0;
    newDom.setObjectsNbr(elements.size());
    while (ELTSidx!=ELTSend) {
      newDom.setObject(index,(*ELTSidx).first,(*ELTSidx).second);
      index++;
      ELTSidx++;
    }
    
    newDom.setDim();
    newDom.makeAllObjectsLst();
    newDom.baseMesh=baseMesh;
    
    if (newDom.getDim()==1) {
      domain tmp;
      tmp.reorder1DDomain(newDom);
      return(tmp);
    }
    
    return(newDom);  
  }
  catchAll("domain domain::operator-(const domain &dom2) const","")
}



domain domain::operator-() const {
  try {
    domain newDom;
    Int nbr=getObjectsNbr();
    
    newDom.setObjectsNbr(nbr);
    for (Int i=0;i<nbr;i++)
      newDom.setObject(i,getObject(i),-getObjectDirect(i));
    
    newDom.setDim();
    newDom.makeAllObjectsLst();
    newDom.baseMesh=baseMesh;
    
    if (newDom.getDim()==1) {
      domain tmp;
      tmp.reorder1DDomain(newDom);
      return(tmp);
    }
    
    return(newDom); 
  }
  catchAll("domain domain::operator-() const","")
}



domain domain::operator*(const domain &dom2) const {
  try {
    Int dim1=getDim();
    Int dim2=dom2.getDim();
    domain newDom;
    object *obj;
    
    if (dim1!=dim2)
      return(newDom);
    if (baseMesh!=dom2.baseMesh)
      return(newDom);
    
    map<object*,Int> elements,elements2;
    map<object*,Int>::iterator ELTSidx,ELTSend;
    
    Int nbr=getObjectsNbr();
    for (Int i=0;i<nbr;i++)
      elements[getObject(i)]=getObjectDirect(i);
    
    nbr=dom2.getObjectsNbr();
    for (Int i=0;i<nbr;i++) {
      obj=dom2.getObject(i);
      if (elements.find(obj)!=elements.end()
	  &&sign(elements[obj])==sign(dom2.getObjectDirect(i)))
	elements2[obj]=elements[obj];
    }
    
    ELTSidx=elements2.begin();
    ELTSend=elements2.end();
    Int index=0;
    newDom.setObjectsNbr(elements2.size());
    while (ELTSidx!=ELTSend) {
      newDom.setObject(index,(*ELTSidx).first,(*ELTSidx).second);
      index++;
      ELTSidx++;
    }
    
    newDom.setDim();
    newDom.makeAllObjectsLst();
    newDom.baseMesh=baseMesh;
    
    if (newDom.getDim()==1) {
      domain tmp;
      tmp.reorder1DDomain(newDom);
      return(tmp);
    }
    
    return(newDom);
  }
  catchAll("domain domain::operator*(const domain &dom2) const","")
}



domain domain::getBoundary(void) const {
  try {
    map<object*,Int> boundaryCount;
    map<object*,Int>::iterator BCidx,BCend;
    map<object*,Int> boundaryDirect;
    
    Int nbr=getObjectsNbr();
    Int nbrMinus,i,j;
    Int thisDirect;
    object *obj;
    
    for (i=0;i<nbr;i++) {
      obj=getObject(i);
      thisDirect=getObjectDirect(i);
      nbrMinus=obj->getMinusNbr();
      for (j=0;j<nbrMinus;j++) {
	boundaryCount[obj->getMinus(j)]++;
	boundaryDirect[obj->getMinus(j)]=(obj->getMinusDirect(j)
					  *sign(thisDirect));
      }
    }
    
    BCidx=boundaryCount.begin();
    BCend=boundaryCount.end();
    nbr=0;
    while (BCidx!=BCend) {
      if ((*BCidx).second==1)
	nbr++;
      BCidx++;
    }
    
    domain boundary;
    boundary.setObjectsNbr(nbr);
    nbr=0;
    BCidx=boundaryCount.begin();
    BCend=boundaryCount.end();
    while (BCidx!=BCend) {
      if ((*BCidx).second==1) {
	obj=(*BCidx).first;
	boundary.setObject(nbr,obj,boundaryDirect[obj]);
	nbr++;
      }
      BCidx++;
    }
    
    boundary.setDim();
    boundary.makeAllObjectsLst();
    boundary.baseMesh=baseMesh;
    
    if (boundary.getDim()==1) {
      domain tmp;
      tmp.reorder1DDomain(boundary);
      return(tmp);
    }
    
    return(boundary);
  }
  catchAll("domain domain::getBoundary(void) const","")
}



void domain::getExtremities(domain &d1,domain &d2) const {
  if (dim!=1)
    errThrow("domain::getExtremities","domain must be made of a single open line.");
  
  domain tmpDom=getBoundary();
  if (tmpDom.objectsNbr!=2)
    errThrow("domain::getExtremities","domain must be made of a single open line.");
  
  domain point1;
  point1.setObjectsNbr(1);
  point1.setObject(0,tmpDom.objectsLst[0],tmpDom.objectsDirect[0]);
  point1.setDim();
  point1.makeAllObjectsLst();
  point1.baseMesh=baseMesh;
  
  domain point2;
  point2.setObjectsNbr(1);
  point2.setObject(0,tmpDom.objectsLst[1],tmpDom.objectsDirect[1]);
  point2.setDim();
  point2.makeAllObjectsLst();
  point2.baseMesh=baseMesh;
  
  d1=point1;
  d2=point2;
}



domain* domain::getFirstNode(void) const {  
  domain* point=new domain;
  
  point->setObjectsNbr(1);
  for (int i=0;i<allObjectsNbr;i++) {
    if (allObjectsLst[i]->getType()==node) {
      point->setObject(0,allObjectsLst[i],0);
      break;
    }
  }
  point->setDim();
  point->makeAllObjectsLst();
  point->baseMesh=baseMesh;
  
  return point;
}




