#ifndef FIELD_BASE_H
#define FIELD_BASE_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <string>

#include "types/types.h"
#include "print/print.h"
#include "util/name.h"
#include "util/parents.h"

namespace keyFE2 {
  
  class array ;
  class domain ;
  class object ;
  class arrayField ;
  class printTree ;
  class step ;
  
  class field : public virtual name, public virtual print,
		public virtual parents {
				       
    friend class arrayField ;
    friend class solver ;
    friend class plot ;
    friend class step ;
    
    private :
      
      domain *defDomain ;
    
      Int nbrComponents ;
      std::string *componentName ;
      
      Int interpolationType ;
      
      Int nodalValuesNbr ; 
      Int nodalValuesNbrPC ;
      Double *nodalValues ;
      Int objectNodalValuesIndexNbr ;
      Int* objectNodalValuesIndex ;
      
      object *currentObject ;
      Int currentObjectDirect ;
      
      Int localNodalValuesSize ;
      Int localNodalValuesCapacity ;
      Int *mappingLocalGlobal ;
      Int *mappingLocalScal ;
      Double *localNodalValues ;
      
      Int currentComponent ;
      
      static Int allFieldsCapacity ;
      static Int allFieldsSize ;
      static field **allFields ;
      static Int allNodalValuesSize ;
      Int firstNodalValueIndex ;
      
      Double minRefForRelativeVariation ;
      
    public :
      
      field(void) ;
      field(domain* dom) ;
      field(domain* dom,Int nbrCmp) ;
      field(domain* dom,Int nbrCmp,std::string interpolationType) ;
      field(const field&) ;
      ~field(void) ;
      
      void operator=(const field&) ;
      
      void setDefDomain(domain *dom) ;
      domain* getDefDomain(void) const ;
      
      void setNbrComponents(Int) ;
      Int getNbrComponents(void) const ;
      void setComponentName(Int,std::string) ;
      std::string getComponentName(Int) const ;
      
      void setInterpolationType(std::string) ;
      Int getInterpolationType(void) const ;
      std::string getInterpolationTypeName(void) const ;
      
      void makeObjectNodalValuesIndex(void) ;
      Int getNodalValuesNbr(void) const ;
      Int getNodalValuesNbrPC(void) const ;
      Double* getNodalValues(void) const ;
      void setNodalValue(Int Comp,Int index,Double v) ;
      Double getNodalValue(Int Comp,Int index) const ;
      
      void fillLocalNodalValues(object *element,Int direct,
				bool makeMappingLocalScal,
                                bool fillInAnyCase) ;
      void interpolate(array *origArray,domain *interpDom,array *Coord) ;
      void interpolate(field *origField,domain *interpDom,field *Coord) ;
      
      void setCurrentComponent(Int nbr) ;
      field& component(Int) ;
      Int getCurrentComponent(void) const ;
      
      void setMinRefForRelativeVariation(Double value) ;
      Double getMinRefForRelativeVariation(void) const ;
      
      void printNodalValues(std::ostream &fOut) const ;
      void printCompNodalValues(std::ostream &fOut,int comp) const ;
      
      void addToAllFields(field *f) ;
      void removeFromAllFields(field *f) ;
      
      static Int getAllFieldsSize(void) ;
      static field* getAllField(Int index) ;
      static Int getAllFieldIndex(field *F) ;
      
      static void updateAllFieldsIndices(void) ;
      static Int getAllNodalValuesSize(void) ;
      
      static void writeInfoOnNodalVariable(Int nodalIndex) ;
      static Double getNodalVariableValue(Int nodalIndex);
      static Double updateAllFieldsNodalValues(Double *delta,bool *activity,
					       bool verbose=true) ;
      static void backupAllFieldsNodalValues(Double *backup) ;
      static void restoreAllFieldsNodalValues(Double *backup) ;
      
      void setAllNodalValues(Double d) ;
      void setAllNodalValues(Double d1,field &f1) ;
      void setAllNodalValues(Double d1,field &f1,Double d2,field &f2) ;
      void setAllNodalValues(Double d1,field &f1,Double d2,field &f2,
			     Double d3,field &f3) ;
      
      printTree getPrintTreeData(void) const ;
      printTree getPrintTreeValues(void) const ;
      printTree getPrintTree(bool values) const ;
      printTree getPrintTreeSummary(void) const ;
      
  } ;
  
}

#endif

