
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include "Filter.h"
#include "filter_base.h"
#include "err/err.h"
#include "param/param.h"
#include "param/paramList.h"

using namespace std ;
using namespace keyFE2 ;



Filter::Filter(void) {
  try {
    filter *F=new filter;
    data=NULL;
    setData(F);
  }
  catchAll("Filter::Filter(void)","")
}



Filter::Filter(const Filter &in) {
  try {
    data=NULL;
    setData(in.getData());
  }
  catchAll("Filter::Filter(const Filter &in)","")
}



Filter::Filter(param P) {
  try {
    paramList PL;
    PL.pushBack(P);
    Filter F(PL);
    data=NULL;
    setData(F.getData());
  }
  catchAll("Filter::Filter(param P)","")
}



Filter::Filter(paramList PL) {
  try {
    filter *tmpFilter=new filter(PL);
    data=NULL;
    setData(tmpFilter);
  }
  catchAll("Filter::Filter(paramList PL)","")
}



Filter::~Filter(void) {
  try {
    setData(NULL);
  }
  catchAll("Filter::~Filter(void)","")
}



void Filter::setData(filter *newFilterData) {
  try {
    filter *F=getData();
    delinkAndDelete(F,&data);
    if (newFilterData!=NULL)
      newFilterData->addParent(&data);
    data=static_cast<void*>(newFilterData);
  }
  catchAll("void Filter::setData(filter *newFilterData)","")
}



filter* Filter::getData(void) const {
  return(static_cast<filter*>(data));
}



void Filter::setName(string s) {
  try {
    filter *filterData=getData();
    filterData->setName(s);
  }
  catchAll("void Filter::setName(string s)","")
}



string Filter::getName(void) const {
  try {
    filter *filterData=getData();
    return(filterData->getName());
  }
  catchAll("string Filter::getName(void) const","")
}



void Filter::operator=(const Filter &arg) {
  if (this==&arg)
    return;
  
  try {
    setData(arg.getData());
  }
  catchAll("void Filter::operator=(const Filter &arg)","")
}



void Filter::write(void) const {
  try {
    filter *filterData=getData();
    filterData->write();
  }
  catchAll("Filter::write","");
}



void Filter::writeValues(void) const {
  try {
    filter *filterData=getData();
    filterData->writeValues();
  }
  catchAll("Filter::writeValues","");
}



void Filter::write(ostream &out) const {
  try {
    filter *filterData=getData();
    filterData->write(out);
  }
  catchAll("Filter::write","");
}



void Filter::writeValues(ostream &out) const {
  try {
    filter *filterData=getData();
    filterData->writeValues(out);
  }
  catchAll("Filter::writeValues","");
}








