
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <cmath>
#include <stdio.h>

#include "DCos.h"
#include "DSin.h"
#include "DConst.h"

using namespace std ;
using namespace keyFE2 ;



DCos::DCos(Double cosfact,Double argfact) : function1() {
   CosFact=cosfact;
   ArgFact=argfact;
}



DCos::DCos(const DCos &in) : function1() {
   CosFact=in.CosFact;
   ArgFact=in.ArgFact;
}



DCos::~DCos(void) {
}



Double DCos::value(Double arg) const {
   return(CosFact*cos(ArgFact*arg));
}



Double DCos::deriv(Double arg) const {
   return(-ArgFact*CosFact*sin(ArgFact*arg));
}



void DCos::setCosFact(Double x) {
   CosFact=x;
}



Double DCos::getCosFact(void) const {
   return(CosFact);
}



void DCos::setArgFact(Double x) {
   ArgFact=x;
}



Double DCos::getArgFact(void) const {
   return(ArgFact);
}



function1* DCos::makeCopy(void) const {
   return(new DCos(*this));
}



function1* DCos::makeFunctionDeriv(void) const {
   if (CosFact==0.0||ArgFact==0.0) 
      return(new DConst(0.0));
   else
      return(new DSin(-CosFact*ArgFact,ArgFact));
}



string DCos::strPrint(void) const {
  char cbuf[STRING_LENGTH];
  string sbuf;
  if (getCosFact()==1.0)
    cbuf[0]=0;
  else if (getCosFact()==-1.0)
    sprintf(cbuf,"-");
  else
    sprintf(cbuf,"%g*",getCosFact());
  sbuf=cbuf;
  
  sbuf+="cos(";
  
  if (getArgFact()==1.0)
    sbuf+="";
  else if (getArgFact()==-1.0)
    sbuf+="-";
  else {
    sprintf(cbuf,"%g*",getArgFact());
    sbuf+=cbuf;
    return cbuf;
  }
  
  sprintf(cbuf,"arrayArg)");
  sbuf+=cbuf;
  
  return sbuf;
}  
  
