
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <cmath>
#include <stdio.h>

#include "DSin.h"
#include "DCos.h"
#include "DConst.h"

using namespace std ;
using namespace keyFE2 ;



DSin::DSin(Double sinfact,Double argfact) : function1() {
   SinFact=sinfact;
   ArgFact=argfact;
}



DSin::DSin(const DSin &in) : function1() {
   SinFact=in.SinFact;
   ArgFact=in.ArgFact;
}



DSin::~DSin(void) {
}



Double DSin::value(Double arg) const {
   return(SinFact*sin(ArgFact*arg));
}



Double DSin::deriv(Double arg) const {
   return(ArgFact*(SinFact*cos(ArgFact*arg)));
}



void DSin::setSinFact(Double x) {
   SinFact=x;
}



Double DSin::getSinFact(void) const {
   return(SinFact);
}



void DSin::setArgFact(Double x) {
   ArgFact=x;
}



Double DSin::getArgFact(void) const {
   return(ArgFact);
}



function1* DSin::makeCopy(void) const {
   return(new DSin(*this));
}



function1* DSin::makeFunctionDeriv(void) const {
   if (SinFact==0.0||ArgFact==0.0) 
      return(new DConst(0.0));
   else
      return(new DCos(SinFact*ArgFact,ArgFact));
}



string DSin::strPrint(void) const {
  char cbuf[STRING_LENGTH];
  string sbuf;
  
  if (getSinFact()==1.0)
    cbuf[0]=0;
  else if (getSinFact()==-1.0)
    sprintf(cbuf,"-");
  else
    sprintf(cbuf,"%g*",getSinFact());
  sbuf=cbuf;
  
  sbuf+="sin(";
  
  if (getArgFact()==1.0)
    sbuf+="";
  else if (getArgFact()==-1.0)
    sbuf+="-";
  else {
    sprintf(cbuf,"%g*",getArgFact());
    sbuf+=cbuf;
  }
  
  sprintf(cbuf,"arrayArg)");
  sbuf+=cbuf;
  
  return sbuf;
}


