
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <cmath>
#include <stdio.h>

#include "DTanh.h"
#include "DSinh.h"
#include "DCosh.h"
#include "DConst.h"
#include "err/err.h"

using namespace std ;
using namespace keyFE2 ;



DTanh::DTanh(Double tanhfact,Double argfact) : function1() {
   TanhFact=tanhfact;
   ArgFact=argfact;
}



DTanh::DTanh(const DTanh &in) : function1() {
   TanhFact=in.TanhFact;
   ArgFact=in.ArgFact;
}



DTanh::~DTanh(void) {
}



Double DTanh::value(Double arg) const {
   return(TanhFact*tanh(ArgFact*arg));
}



Double DTanh::deriv(Double arg) const {
   return(ArgFact*(TanhFact*pow(cosh(ArgFact*arg),-2)));
}



void DTanh::setTanhFact(Double x) {
   TanhFact=x;
}



Double DTanh::getTanhFact(void) const {
   return(TanhFact);
}



void DTanh::setArgFact(Double x) {
   ArgFact=x;
}



Double DTanh::getArgFact(void) const {
   return(ArgFact);
}



function1* DTanh::makeCopy(void) const {
   return(new DTanh(*this));
}



function1* DTanh::makeFunctionDeriv(void) const {
  errThrow("DTanh::makeFunctionDeriv","function undefined");
}



string DTanh::strPrint(void) const {
  char cbuf[STRING_LENGTH];
  string sbuf;
  
  if (getTanhFact()==1.0)
    cbuf[0]=0;
  else if (getTanhFact()==-1.0)
    sprintf(cbuf,"-");
  else
    sprintf(cbuf,"%g*",getTanhFact());
  sbuf=cbuf;
  
  sbuf+="tanh(";
  
  if (getArgFact()==1.0)
    sbuf+="";
  else if (getArgFact()==-1.0)
    sbuf+="-";
  else {
    sprintf(cbuf,"%g*",getArgFact());
    sbuf+=cbuf;
  }
  
  sprintf(cbuf,"arrayArg)");
  sbuf+=cbuf;
  
  return sbuf;
}


