
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <cmath>

#include "DUpwind.h"
#include "err/err.h"

using namespace std ;
using namespace keyFE2 ;



DUpwind::DUpwind(void) : function1() {
}



DUpwind::DUpwind(const DUpwind &in) : function1() {
}



DUpwind::~DUpwind(void) {
}



Double DUpwind::value(Double arg) const {
   if (abs(arg)>20.0)
      return arg/abs(arg) ;
   else if (abs(arg)>0.01) 
      return 1.0/tanh(arg)-1.0/arg;
   else
      return arg/3.0-arg*arg*arg/45.0;
}



Double DUpwind::deriv(Double arg) const {
   if (abs(arg)>20.0)
      return 0.0;
   else if (abs(arg)>0.01) 
      return -1.0/(sinh(arg)*sinh(arg))+1.0/(arg*arg);
   else
      return 1.0/3.0-arg*arg/15.0;
}



function1* DUpwind::makeCopy(void) const {
   return(new DUpwind(*this));
}



function1* DUpwind::makeFunctionDeriv(void) const {
  errThrow("DUpwind::::makeFunctionDeriv",
	   "cannot make a derivative function of DUpwind:: object.");
}



string DUpwind::strPrint(void) const {
  return "DUpwind(arrayArg)";
}
 
