
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <cmath>
#include <stdio.h>

#include "DpwConst2.h"
#include "DConst.h"

using namespace std ;
using namespace keyFE2 ;



DpwConst2::DpwConst2(void) : function1() {
  PA=0.0;
  X1=0.0;
  P12=0.0;
  X2=0.0;
  PB=0.0;
}



DpwConst2::DpwConst2(Double pa,
                   Double x1,Double p12,
                   Double x2,Double pb) : function1() {
  PA=pa;
  X1=x1;
  P12=p12;
  X2=x2;
  PB=pb;
}



DpwConst2::DpwConst2(const DpwConst2 &in) : function1() {
  PA=in.PA;
  X1=in.X1;
  P12=in.P12;
  X2=in.X2;
  PB=in.PB;
}



DpwConst2::~DpwConst2(void) {
  PA=0.0;
  X1=0.0;
  P12=0.0;
  X2=0.0;
  PB=0.0;
}



Double DpwConst2::value(Double arg) const {
   if (arg<X1)
      return PA;
   else if (arg<X2)
      return P12;
   else
      return PB;
}



Double DpwConst2::deriv(Double arg) const {
   return 0.0;
}



void DpwConst2::setPA(Double x) {
  PA=x;
}



Double DpwConst2::getPA(void) const {
  return(PA);
}



void DpwConst2::setX1(Double x) {
  X1=x;
}



Double DpwConst2::getX1(void) const {
  return(X1);
}



void DpwConst2::setP12(Double x) {
  P12=x;
}



Double DpwConst2::getP12(void) const {
  return(P12);
}



void DpwConst2::setX2(Double x) {
  X2=x;
}



Double DpwConst2::getX2(void) const {
  return(X2);
}



void DpwConst2::setPB(Double x) {
  PB=x;
}



Double DpwConst2::getPB(void) const {
  return(PB);
}



function1* DpwConst2::makeCopy(void) const {
  return(new DpwConst2(*this));
}



function1* DpwConst2::makeFunctionDeriv(void) const {
    return(new DConst(0.0));

}



string DpwConst2::strPrint(void) const {
  char cbuf[STRING_LENGTH];
  sprintf(cbuf,"pwConst2(pa=%g,x1=%g,p12=%g,x2=%g,pb=%g,arrayArg)",
          PA,X1,P12,X2,PB);
  return cbuf;
}
 

