
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//



#include "Integ.h"
#include "integ_base.h"
#include "coordSys/GlobLoc.h"
#include "coordSys/globLoc_base.h"
#include "coordSys/TensorCoordSys.h"
#include "coordSys/tensorCoordSys_base.h"
#include "err/err.h"
#include "object/object_base.h"
#include "param/param.h"
#include "param/paramList.h"
#include "scal/scal.h"

using namespace std ;
using namespace keyFE2 ;



Integ::Integ(void) {
  try {
    integ *argInteg=new integ;
    data=NULL;
    setData(argInteg);
  }
  catchAll("Integ::Integ(void)","at the very end")
}



Integ::Integ(param P) {
  try {
    paramList PL;
    PL.pushBack(P);
    Integ I(PL);
    data=NULL;
    setData(I.getData());
  }
  catchAll("Integ::Integ(param P)","at the very end")
}



Integ::Integ(paramList PL) {
  try {
    string integName="";
    Int integRule=0;
    string integRuleName="";
    tensorCoordSys *integTCS=NULL;
    globLoc *integGL=NULL;
    
    paramList plName=PL.getListOfType(NAME);
    PL=PL.getListNotOfType(NAME);
    paramList plGL=PL.getListOfType(GLOBLOC);
    PL=PL.getListNotOfType(GLOBLOC);
    paramList plTCS=PL.getListOfType(TENSORCOORDSYS);
    PL=PL.getListNotOfType(TENSORCOORDSYS);
    paramList plRule=PL.getListOfType(RULE);
    PL=PL.getListNotOfType(RULE);
    paramList plRuleName=PL.getListOfType(RULENAME);
    PL=PL.getListNotOfType(RULENAME);
    
    // Checks for illegal arguments :
    
    if (PL.getNbrParams()!=0) {
      string tmpString=strFromInt(PL.getNbrParams())+" invalid parameters :";
      for (Int i=0;i<PL.getNbrParams();i++) {
	tmpString+=" ";
	tmpString+=PL[i].getName();
      }
      tmpString+=".\n";
      errWrite("Integ construction",tmpString);
    }
    
    // Sets the name of the Integ object :
    
    if (plName.getNbrParams()>1) {
      errWrite("Integ construction","only one NAME argument is allowed.");
    }
    else if (plName.getNbrParams()==1) {
      integName=plName[0].getStringData();
      if (integName=="")
	errWrite("Integ construction",
		 "NAME= has to be followed by a string argument.");
    }
    
    // Sets the rule of the globloc :
    
    if ((plRuleName.getNbrParams()+plRule.getNbrParams())!=1) {
      errWrite("Integ construction",
	       "exactly one RULE or RULENAME argument is needed.");
    }
    else if (plRuleName.getNbrParams()==1) {
      integRuleName=plRuleName[0].getStringData();
      if (integRuleName=="")
	errWrite("Integ construction",
		 "RULENAME= has to be followed by a string argument.");
    }
    else if (plRule.getNbrParams()==1) {
      integRule=plRule[0].getIntData();
      if (integRule==-1)
	errWrite("Integ construction",
		 "RULE= has to be followed by an integer argument.");
    }
        
    // Sets the tensorCoordSys of the integ :
    
    if (plTCS.getNbrParams()!=1) {
      errWrite("Integ construction",
	       "exactly one TENSORCOORDSYS argument is needed.");
    }
    else if (plTCS.getNbrParams()==1) {
      parents *tmpP=plTCS[0].getParentData();
      integTCS=dynamic_cast<tensorCoordSys*>(tmpP);
      if (integTCS==NULL)
	errWrite("Integ construction",
		 "TENSORCOORDSYS= has to be followed by a TensorCoordSys argument.");
    }
    
    // Sets the globLoc of the integ :
    
    if (plGL.getNbrParams()!=1) {
      errWrite("Integ construction",
	       "exactly one GLOBLOC argument is needed.");
    }
    else if (plGL.getNbrParams()==1) {
      parents *tmpP=plGL[0].getParentData();
      integGL=dynamic_cast<globLoc*>(tmpP);
      if (integGL==NULL)
	errWrite("Integ construction",
		 "GLOBLOC= has to be followed by a GlobLoc argument.");
    }
    
    {
      string glTypeName=integGL->getTypeName();
      int globDim=GlobLoc::getGlobDimFromName(glTypeName);
      string tcsTypeName=integTCS->getTypeName();
      switch (globDim) {
        case 0 :
	  errWrite("Integ construction",
	           "GLOBLOC= cannot be followed by a 0D GlobLoc argument.");
	  break;
	case 1 :
	  if (tcsTypeName!="Cartesian1D")
	    errWrite("Integ construction",
	             "Incompatible GlobLoc and TensorCoordSys: "+glTypeName
		     +","+tcsTypeName+".");
          break;
	case 2 :
	  if (tcsTypeName!="Cartesian2D")
	    errWrite("Integ construction",
	             "Incompatible GlobLoc and TensorCoordSys: "+glTypeName
		     +","+tcsTypeName+".");
          break;
	case 3 :
	  if (tcsTypeName!="Cartesian3D"&&tcsTypeName!="CylindricalRZT")
	    errWrite("Integ construction",
	             "Incompatible GlobLoc and TensorCoordSys: "+glTypeName
		     +","+tcsTypeName+".");
          break;
        default :
	  errWrite("Integ construction",
	           "Incompatible GlobLoc and TensorCoordSys: "+glTypeName
		   +","+tcsTypeName+".");
	  break;
      }
    }
    
    if (!err::getOk())
      errThrow("Integ construction","bailing out due to preceding errors.");
    
    integ *outputI;
    if (integRuleName!="") {
      outputI=new integ(integGL,integTCS,integRuleName);
    }
    else
      outputI=new integ(integGL,integTCS,integRule);
    outputI->setName(integName);
        
    data=NULL;
    setData(outputI);
  }
  catchAll("Integ::Integ(paramList PL)","at the very end")
}



Integ::Integ(const Integ &ArgInteg) {
  try {
    data=NULL;
    setData(ArgInteg.getData());
  }
  catchAll("Integ::Integ(const Integ &ArgInteg)","at the very end")
}



Integ::~Integ(void) {
  try {
    setData(NULL);
  }
  catchAll("Integ::~Integ(void)","at the very end")
}



void Integ::setData(integ *newInteg) {
  try {
    integ *old=getData();
    delinkAndDelete(old,&data);
    if (newInteg!=NULL)
      newInteg->addParent(&data);
    data=static_cast<void*>(newInteg);
  }
  catchAll("void Integ::setData(integ *newInteg)","at the very end")
}



integ* Integ::getData(void) const {
  return(static_cast<integ*>(data));
}



void Integ::setName(string str) {
  try {
    integ *arg=getData();
    arg->setName(str);
  }
  catchAll("void Integ::setName(string str)","at the very end")
}



string Integ::getName(void) const {
  try {
    return(getData()->getName());
  }
  catchAll("string Integ::getName(void) const","at the very end")
}



void Integ::write(void) const {
  try {
    if (getData()==NULL)
      cerr << "Integ::write: Integ contains no data." << endl ;
    else {
      integ *integData=getData();
      integData->write();
    }
  }
  catchAll("Integ::write","at the very end");
}



void Integ::writeValues(void) const {
  try {
    if (getData()==NULL)
      cerr << "Integ::writeValues: Integ contains no data." << endl ;
    else {
      integ *integData=getData();
      integData->writeValues();
    }
  }
  catchAll("Integ::writeValues","at the very end");
}



void Integ::write(ostream &out) const {
  try {
    if (getData()==NULL)
      cerr << "Integ::write: Integ contains no data." << endl ;
    else {
      integ *integData=getData();
      integData->write(out);
    }
  }
  catchAll("Integ::write","at the very end");
}



void Integ::writeValues(ostream &out) const {
  try {
    if (getData()==NULL)
      cerr << "Integ::writeValues: Integ contains no data." << endl ;
    else {
      integ *integData=getData();
      integData->writeValues(out);
    }
  }
  catchAll("Integ::writeValues","at the very end");
}



void Integ::operator=(const Integ &ArgInteg) {
  if (this==&ArgInteg)
    return;
  
  try {
    data=NULL;
    setData(ArgInteg.getData());
  }
  catchAll("Integ::Integ(const Integ &ArgInteg)","at the very end")
}




