
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include "Mesh.h"
#include "mesh_base.h"
#include "array/Array.h"
#include "array/array_base.h"
#include "domain/Domain.h"
#include "domain/domain_base.h"
#include "err/err.h"
#include "field/Field.h"

using namespace std ;
using namespace keyFE2 ;



Mesh::Mesh(void) {
  try {
    mesh *m=new mesh;
    data=NULL;
    setData(m);
  }
  catchAll("Mesh::Mesh(void)","")
}



Mesh::Mesh(const Mesh &M) {
  try {
    data=NULL;
    setData(M.getData());
  }
  catchAll("Mesh::Mesh(const Mesh &M)","")
}



Mesh::~Mesh(void) {
  try {
    setData(NULL);
  }
  catchAll("Mesh::~Mesh(void)","")
}



void Mesh::setData(mesh *newMeshData) {
  try {
    mesh *m=getData();
    delinkAndDelete(m,&data);
    if (newMeshData!=NULL)
      newMeshData->addParent(&data);
    data=static_cast<void*>(newMeshData);
  }
  catchAll("void Mesh::setData(mesh *newMeshData)","")
}



mesh* Mesh::getData(void) const {
  return(static_cast<mesh*>(data));
}



void Mesh::setName(string str) {
  try {
    mesh *m=getData();
    m->setName(str);
  }
  catchAll("void Mesh::setName(string str)","")
}



string Mesh::getName(void) const {
  try {
    return(getData()->getName());
  }
  catchAll("string Mesh::getName(void) const","")
}



void Mesh::operator=(const Mesh &M) {
  if (this==&M)
    return;
  
  try {
    setData(M.getData());
  }
  catchAll("void Mesh::operator=(const Mesh &M)","")
}



void Mesh::read(string fileName,string typeMesh) {
  try {
    getData()->read(fileName,typeMesh);
  }
  catchAll("void Mesh::read(string fileName,string typeMesh)","")
}



void Mesh::write(string fileName,string typeMesh) {
  try {
    getData()->write(fileName,typeMesh);
  }
  catchAll("void Mesh::write(string fileName,string typeMesh)","")
}



Int Mesh::getDomainsNbr(void) {
  try {
    return(getData()->getDomainsNbr());
  }
  catchAll("Int Mesh::getDomainsNbr(void)","")
}



Domain Mesh::getDomain(Int index) {
  try {
    return(getData()->getDomain(index));
  }
  catchAll("Domain Mesh::getDomain(Int index)","")
}



Domain Mesh::getDomainFromIntId(Int ident) {
  try {
    return(getData()->getDomainFromIntId(ident));
  }
  catchAll("Domain Mesh::getDomainFromIntId(Int ident)","")
}



Int Mesh::getDomainIntId(Int ident) {
  try {
    return(getData()->getDomainIntId(ident));
  }
  catchAll("Int Mesh::getDomainIntId(Int ident)","")
}



void Mesh::writeDomain(ofstream &outFile,Domain &Dom) {
  try {
    getData()->writeDomain(outFile,Dom);
  }
  catchAll("void Mesh::writeDomain(ofstream &outFile,Domain &Dom)","")
}



void Mesh::writeDomain(string fileName,Domain &Dom) {
  try {
    getData()->writeDomain(fileName,Dom);
  }
  catchAll("void Mesh::writeDomain(string fileName,Domain &Dom)","")
}



Domain Mesh::getDefDomain(void) {
  try {
    return(getData()->getDefDomain());
  }
  catchAll("Domain Mesh::getDefDomain(void)","")
}



Field Mesh::getCoordinateField(Domain &Dom) {
  try {
    return(getData()->getCoordinateField(Dom));
  }
  catchAll("Field Mesh::getCoordinateField(Domain &Dom)","")
}



namespace keyFE2 {
  
  Mesh renumberDefDomainFrontal(const Mesh &Marg,Domain Darg) {
    try {
      mesh *m=new mesh;
      mesh *marg=Marg.getData();
      domain *darg=Darg.getData();
      m->renumberDefDomainFrontal(*marg,*darg);
      Mesh M;
      M.setData(m);
      return(M);
    }
    catchAll("Mesh renumberDefDomainFrontal(const Mesh &Marg,Domain Darg)","")
  }
  
  
  Mesh renumberDefDomainArray(const Mesh &Marg,Array Aarg) {
    try {
      mesh *m=new mesh;
      mesh *marg=Marg.getData();
      array *aarg=Aarg.getData();
      m->renumberDefDomainArray(*marg,*aarg);
      Mesh M;
      M.setData(m);
      return(M);
    }
    catchAll("Mesh renumberDefDomainArray(const Mesh &Marg,Array Aarg)","")
  }
  
}



void Mesh::write(ostream &os) const {
  try {
    dynamic_cast<print*>(getData())->write(os);
  }
  catchAll("void Mesh::write(ostream &os) const","")
}



void Mesh::write(void) const {
  try {
    dynamic_cast<print*>(getData())->write();
  }
  catchAll("void Mesh::write(void) const","")
}



void Mesh::writeValues(ostream &os) const {
  try {
    dynamic_cast<print*>(getData())->writeValues(os);
  }
  catchAll("void Mesh::writeValues(ostream &os) const","")
}



void Mesh::writeValues(void) const {
  try {
    dynamic_cast<print*>(getData())->writeValues();
  }
  catchAll("void Mesh::writeValues(void) const","")
}


