
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iostream>
#include <stdio.h>

#include "mesh_base.h"
#include "meshTypes.h"
#include "domain/domain_base.h"
#include "err/err.h"
#include "object/object_base.h"
#include "license/license.h"
#include "print/printTree.h"

using namespace std ;
using namespace keyFE2 ;



mesh::mesh(void) : name(), parents() {
  license brol;

  type=none;
  
  nodesNbr=0;
  coord=NULL;
  
  objectsNbr=0;
  objectsLst=NULL;
  
  domainsNbr=0;
  domainsLst=NULL;
  domainsIntId=NULL;
}   



mesh::~mesh(void) {
  try {
    if (coord!=NULL) {
      delete[] coord;
      coord=NULL;
    }
    if (objectsLst!=NULL) {
      delete[] objectsLst;
      objectsLst=NULL;
    }
    setDomainsNbr(0);
  }
  catchAll("mesh::~mesh(void)","")
}



object* mesh::setObjectsNbr(Int nbr) {
  try {
    if (objectsLst!=NULL)
      delete[] objectsLst;
    if (nbr>0)
      objectsLst=new object[nbr];
    else
      objectsLst=NULL;
    objectsNbr=nbr;
    
    return(objectsLst);
  }
  catchAll("object* mesh::setObjectsNbr(Int nbr)","")
}



Int mesh::getObjectsNbr(void) const {
  return(objectsNbr);
}



object* mesh::getObject(Int index) const {
  return(objectsLst+index);
}



Int mesh::getObjectIndex(object *obj) const {
  return(obj-objectsLst);
}



object* mesh::getObjectsLst(void) const {
  return(objectsLst);
}



printTree mesh::getPrintTreeData(void) const {
  try {
    char cbuf[STRING_LENGTH];
    string sbuf;
    printTree pt;
  
    if (getName()=="")
      sprintf(cbuf,"   anonymous");
    else
      sprintf(cbuf,"   name = %s",getName().c_str());
    sbuf=cbuf;
    pt+=sbuf;
    
    sbuf="   type = ";
    sprintf(cbuf,"%d",type);
    sbuf+=cbuf;
    pt+=sbuf;
    
    return(pt);
  }
  catchAll("mesh::getPrintTreeData","")
}



printTree mesh::getPrintTreeValues(void) const { 
  try {
    printTree pt,ptArg;
    char cbuf[STRING_LENGTH];
    string sbuf;
    Int i;
    
    sprintf(cbuf," | nodesNbr = %d",nodesNbr);
    sbuf=cbuf;
    pt+=sbuf;
    
    sprintf(cbuf," | objectsNbr = %d",objectsNbr);
    sbuf=cbuf;
    pt+=sbuf;
    
    pt+=" | ";
    sprintf(cbuf," | domainsNbr = %d",domainsNbr);
    sbuf=cbuf;
    pt+=sbuf;
    pt+=" | ";
    
    pt.prepareForArguments();
    
    for (i=0;i<domainsNbr;i++) {
      sprintf(cbuf,"domain (pos=%d,intId=%d)",i,domainsIntId[i]);
      ptArg=domainsLst[i].getFirstPrintTree(cbuf,false);
      if (i+1==domainsNbr)
        ptArg.prepareForAddLast();
      else
        ptArg.prepareForAdd();
      pt+=ptArg;
    }
    
    return pt;
  }
  catchAll("mesh::getPrintTreeValues","")
}



printTree mesh::getPrintTree(bool values) const {
  try {
    char cbuf[STRING_LENGTH];
    string sbuf;
    printTree pt,ptArg;
    
    sprintf(cbuf,"mesh %p",static_cast<const void*>(this));
    sbuf=cbuf;
    pt+=sbuf;
    
    ptArg=getPrintTreeData();
    pt+=ptArg;
    
    if (values) {
      pt.prepareForArguments();
      pt+=getPrintTreeValues();
    }
    sbuf="   ";
    pt+=sbuf;
    
    return(pt);
  }
  catchAll("printTree mesh::getPrintTree(bool values) const","")
}
