#ifndef MESH_BASE_H
#define MESH_BASE_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <fstream>
#include <string>

#include "types/types.h"
#include "types/params.h"

#include "util/name.h"
#include "util/parents.h"
#include "print/print.h"

namespace keyFE2 {
  
  class array ;
  class Domain ;
  class domain ;
  class Field ;
  class field ;
  class object ;
  
  class mesh : public virtual name, public virtual parents,
	       public virtual print {
	       
    private:
    
       class intList {
         
	 public:
	 
	   Int size ;
	   Int ilist[10] ;
	   
	 public:
	 
	   intList(void);
	   intList(const intList &in);
	   ~intList(void);
	   
	   void setSize(Int nbr) ;
	   const mesh::intList& operator=(const mesh::intList &in) ;
	   
	   bool operator==(const mesh::intList &in) const ;
	   bool operator!=(const mesh::intList &in) const ;
	   bool operator<=(const mesh::intList &in) const ;
	   bool operator>=(const mesh::intList &in) const ;
	   bool operator<(const mesh::intList &in) const ;
	   bool operator>(const mesh::intList &in) const ;
	   
	   void write(void) const ;
	 
       } ;
				      
    private:
      
      Int type ;
    
      Int nodesNbr ;
      Double (*coord)[3] ;
    
      Int objectsNbr ;
      object *objectsLst ;
    
      Int domainsNbr ;
      domain *domainsLst ;
      Int *domainsIntId ;
    
    public:
      
      mesh(void) ;
      ~mesh(void) ;
    
    private:
      
      void read_rsMesh(std::istream &outFile) ;
      void write_rsMesh(std::ostream &outFile) const ;
      void read_2DMesh(std::istream &outFile) ;
      void write_2DMesh(std::ostream &outFile) const ;
      void read_emc2Mesh(std::istream &outFile) ;
      void write_emc2Mesh(std::ostream &outFile) const ;
      void read_gmsh2dMesh(std::istream &outFile) ;
      void write_gmsh2dMesh(std::ostream &outFile) const ;
    
    public:
      
      void read(std::string fileName,std::string typeMesh) ;
      void write(std::string fileName,std::string typeMesh) const ;
    
      object* setObjectsNbr(Int nbr) ;
      Int getObjectsNbr(void) const ;
      object* getObject(Int index) const ;
      Int getObjectIndex(object *obj) const ;
      object* getObjectsLst(void) const ;
      
      domain* setDomainsNbr(Int nbr) ;
      Int getDomainsNbr(void) const ;
      Domain getDefDomain(void) const ;
      Domain getDomain(Int) const ;
      Domain getDomainFromIntId(Int) const ;
      Int getDomainIntId(Int) const ;
      void writeDomain(std::ostream &myCout,Domain &Dom) const ;
      void writeDomain(std::string fileName,Domain &Dom) const ;
      void writeDomain(std::ostream &myCout,domain &dom) const ;
      void writeDomain(std::string fileName,domain &dom) const ;
      
      Field getCoordinateField(Domain &Dom) const ;
      field* getCoordinateField(domain *dom) const ;
      
      void renumberDefDomainFrontal(const mesh &arg,domain &dom) ;
      void renumberDefDomainArray(const mesh &arg,array &A) ;
      
      virtual printTree getPrintTreeData(void) const ;
      virtual printTree getPrintTreeValues(void) const ;
      virtual printTree getPrintTree(bool values) const ;
      
  } ; 
  
}

#endif
