#ifndef OBJECT_BASE_H
#define OBJECT_BASE_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <string>

#include "types/types.h"

namespace keyFE2 {
  
  class object {
    
    friend class domain ;
    friend class mesh ;
    
    private :
      
      Int type;
    
      Int minusNbr;
      Int *minusDirect;
      object **minusLst;
    
      Int plusNbr;
      object **plusLst;
    
    public : 
      
      object(void);
      object(std::string);
      ~object(void);
      
      void setType(Int);
      void setType(std::string);
      Int getType(void) const ;
      std::string getTypeName(void) const ;
      
      void setMinusNbr(Int);
      void setMinus(Int,object*,Int);
      Int getMinusNbr(void) const ;
      object* getMinus(Int) const ;
      Int getMinusDirect(Int) const ;
      
      void setPlusNbr(Int);
      void setPlus(Int,object*);
      Int getPlusNbr(void) const ;
      object* getPlus(Int) const ;
      
      Int split(Int thisDirect,object **lst,Int *Direct) ;
      void getPlusMinMax(object* &objMin,object* &objMax) ;
      
      Int getDim(void) const ;
      static Int getDim(const Int &objectType) ;
      
  } ;
  
}

#endif
