#ifndef RESULT_BASE_H
#define PROBLEM_BASE_H

//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include "print/print.h"
#include "util/parents.h"
#include "types/allTypes.h"
#include "util/utils.h"

namespace keyFE2 {
  
  class problem : public virtual name, public virtual parents,
		  public virtual print, public virtual newPoint {

    friend class array ;
    friend class constr ;
		
    protected :
    
      Int arrayCapacity ;
      Int arraySize ;
      array **aList ;
      Int constrCapacity ;
      Int constrSize ;
      constr **cList ;
    
    public :
      
      problem(void) ;
      problem(const problem &in) ;
      virtual ~problem(void) ; 
      
      void addArray(array *inArray) ;
      void addConstr(constr *inConstr) ; 
      
      Int getNbrArrays(void) const ; 
      array* getArray(Int pos) const ;  
      Int getNbrConstrs(void) const ;   
      constr* getConstr(Int pos) const ;  
      
      void setNewPoint(bool b) ;
      void accessNbrInit(void) ;
      
      printTree getPrintTreeData(void) const ;
      printTree getPrintTreeValues(void) const ;
      printTree getPrintTree(bool values) const ;
      printTree getPrintTreeSummary(void) const ;
      
  } ;
   
}

#endif
    
    
