
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <fstream>
#include <iomanip>
#include <iostream>
#include <string>

#include "scal.h"
#include "err/err.h"

using namespace std ;
using namespace keyFE2 ;



inline void swap(Double &a,Double &b) {
  static Double tmp;
  tmp=a;
  a=b;
  b=tmp;
}

inline void swap(Double* &a,Double* &b) {
  static Double *tmp;  
  tmp=a;
  a=b;
  b=tmp;
}

inline void swap(Double** &a,Double** &b) {
  static Double **tmp;  
  tmp=a;
  a=b;
  b=tmp;
}

inline void swap(Int &a,Int &b) {
  static Int tmp;
  tmp=a;
  a=b;
  b=tmp;
}

inline void swap(Int* &a,Int* &b) {
  static Int *tmp;
  tmp=a;
  a=b;
  b=tmp;
}

inline void swap(bool &a,bool &b) {
  static bool tmp;
  tmp=a;
  a=b;
  b=tmp;
}

inline void swap(bool* &a,bool* &b) {
  static bool* tmp;
  tmp=a;
  a=b;
  b=tmp;
}



namespace keyFE2 {
  
  Int scal::mapCapacity=100;
  Int scal::mapSize=0;
//#warning modifier quand l erreur interne de compilateur sera corrige!
  Int* scal::map=new Int[100]; // 100=scal::mapCapacity=100
  bool scal::white=false;
  
  Int scal::stackCapacity=10;
  Int scal::stackSize=0;
//#warning modifier quand l erreur interne de compilateur sera corrige!
  Double* scal::dStack=new Double[10]; // 10=scal::stackCapacity
//#warning modifier quand l erreur interne de compilateur sera corrige!
  scal** scal::sStack=new scal* [10]; // 10=scal::stackCapacity
  
  void exchange(scal &s1,scal &s2) {
    static string tmpString;
    static bool tmpBool;
    
    tmpBool=s1.boolZeroValue;
    s1.boolZeroValue=s2.boolZeroValue;
    s2.boolZeroValue=tmpBool;
    
    swap(s1.val,s2.val);
    
    swap(s1.ligCapacity,s2.ligCapacity);
    swap(s1.ligSize,s2.ligSize);
    swap(s1.ligIndex,s2.ligIndex);
    swap(s1.ligBool,s2.ligBool);
    swap(s1.lig,s2.lig);
    
    swap(s1.colCapacity,s2.colCapacity);
    swap(s1.colSize,s2.colSize);
    swap(s1.colIndex,s2.colIndex);
    swap(s1.colBool,s2.colBool);
    swap(s1.col,s2.col);
    
    swap(s1.mat,s2.mat);
  }
  
}



using namespace keyFE2 ;

scal::scal(void) : boolZero() {
  val=0.0;
  
  ligCapacity=0;
  ligSize=0;
  ligIndex=0;
  ligBool=NULL;
  lig=NULL;
  
  colCapacity=0;
  colSize=0;
  colIndex=0;
  colBool=NULL;
  col=NULL;
  
  mat=NULL;
  
  boolZeroValue=true;
}



scal::scal(Double d) : boolZero() {
  val=d;
  
  ligCapacity=0;
  ligSize=0;
  ligIndex=0;
  ligBool=NULL;
  lig=NULL;
  
  colCapacity=0;
  colSize=0;
  colIndex=0;
  colBool=NULL;
  col=NULL;
  
  mat=NULL;
  
  if (val==0.0)
    boolZeroValue=true;
  else
    boolZeroValue=false;
}



scal::scal(const scal &in) : boolZero(in) {
  Int i;
  register Double *diter1,*diter2;
  register bool *biter1,*biter2,*bFinal;
  register Double *dFinal;

  ligCapacity=0;
  ligSize=0;
  ligIndex=0;
  ligBool=NULL;
  lig=NULL;
  
  colCapacity=0;
  colSize=0;
  colIndex=0;
  colBool=NULL;
  col=NULL;
  
  mat=NULL;
  
  setLigColSize(in.ligSize,in.colSize);
  colIndex=in.colIndex;
  ligIndex=in.ligIndex;
  
  if (white) {
    val=in.val;
    
    biter1=colBool;
    biter2=in.colBool;
    bFinal=biter1+colSize;
    while (biter1<bFinal)
      *(biter1++)=*(biter2++);
    
    biter1=ligBool;
    biter2=in.ligBool;
    bFinal=biter1+ligSize;
    while (biter1<bFinal)
      *(biter1++)=*(biter2++);
    
    if (val==0.0&&ligSize==0&&colSize==0)
      boolZeroValue=true;
    else
      boolZeroValue=false;
    
    return;
  }
  
  val=in.val;
  
  biter1=colBool;
  biter2=in.colBool;
  diter1=col;
  diter2=in.col;
  dFinal=diter1+colSize;
  while (diter1<dFinal) {
    *(biter1++)=*(biter2++);
    *(diter1++)=*(diter2++);
  }
  
  biter1=ligBool;
  biter2=in.ligBool;
  diter1=lig;
  diter2=in.lig;  
  dFinal=diter1+ligSize;
  while (diter1<dFinal) {
    *(biter1++)=*(biter2++);
    *(diter1++)=*(diter2++);
  }
  
  
  for (i=0;i<colSize;i++) {
    if (colBool[i]) {
      diter1=mat[i];
      diter2=in.mat[i];
      dFinal=diter1+ligSize;
      while (diter1<dFinal)
        *(diter1++)=*(diter2++);
    }
    else {
      diter1=mat[i];
      dFinal=diter1+ligSize;
      while (diter1<dFinal)
        *(diter1++)=0.0;
    }
  }
  
  boolZeroValue=in.boolZeroValue;
}



scal::~scal(void) {
  delete[] ligBool;
  ligBool=NULL;
  delete[] lig;
  lig=NULL;
  
  delete[] colBool;
  colBool=NULL;
  delete[] col;
  col=NULL;
  
  if (mat!=NULL)
    delete[] mat[0];
  delete[] mat;
  mat=NULL;
}



Int* scal::getMap(void) const {
  return(map);
}



Int scal::getPosInMap(Int nbr) const {
  Int i;
  
  for (i=0;i<mapSize;i++)
    if (map[i]==nbr)
      return(i);
  
  errThrow("scal::getPosInMap",strFromInt(nbr)+" not in map.");
}



void scal::setWhite(bool b) {
  white=b;
}



bool scal::getWhite(void) {
  return(white);
}



void scal::setLigColCapacity(Int nbr1,Int nbr2) {
  Double *tmp;
  Int i;
  
  ligCapacity=nbr1;
  delete[] lig;
  if (nbr1>0)
    lig=new Double[nbr1];
  else
    lig=NULL;
  
  colCapacity=nbr2;
  delete[] col;
  if (nbr2>0)
    col=new Double[nbr2];
  else
    col=NULL;
  
  if (mat!=NULL)
    delete[] mat[0];
  delete[] mat;
  if (colCapacity>0&&ligCapacity>0) {
    tmp=new Double[colCapacity*ligCapacity];
    mat=new Double* [colCapacity];
    for (i=0;i<colCapacity;i++)
      mat[i]=tmp+i*ligCapacity;
  }
  else 
    mat=NULL;
  
  if (ligBool!=NULL)
    delete[] ligBool;
  if (ligCapacity>0)
    ligBool=new bool[ligCapacity];
  else
    ligBool=NULL;
  
  if (colBool!=NULL)
    delete[] colBool;
  if (colCapacity>0)
    colBool=new bool[colCapacity];
  else
    colBool=NULL;
}



void scal::setLigCapacity(Int nbr) {
  Double *tmp;
  Int i;
  
  ligCapacity=nbr;
  
  delete[] lig;
  if (nbr>0)
    lig=new Double[nbr];
  else
    lig=NULL;
  
  if (mat!=NULL)
    delete[] mat[0];
  delete[] mat;
  if (colCapacity>0&&ligCapacity>0) {
    tmp=new Double[colCapacity*ligCapacity];
    mat=new Double* [colCapacity];
    for (i=0;i<colCapacity;i++)
      mat[i]=tmp+i*ligCapacity;
  }
  else
    mat=NULL;
  
  if (ligBool!=NULL)
    delete[] ligBool;
  if (ligCapacity>0)
    ligBool=new bool[ligCapacity];
  else
    ligBool=NULL;
}



void scal::setColCapacity(Int nbr) {
  Double *tmp;
  Int i;
  
  colCapacity=nbr;
  
  delete[] col;
  if (nbr>0)
    col=new Double[nbr];
  else
    col=NULL;
  
  if (mat!=NULL)
    delete[] mat[0];
  delete[] mat;
  if (colCapacity>0&&ligCapacity>0) {
    tmp=new Double[colCapacity*ligCapacity];
    mat=mat=new Double* [colCapacity];
    for (i=0;i<colCapacity;i++)
      mat[i]=tmp+i*ligCapacity;
  }
  else
    mat=NULL;
  
  if (colBool!=NULL)
    delete[] colBool;
  if (colCapacity>0)
    colBool=new bool[colCapacity];
  else
    colBool=NULL;
}



void scal::setLigColSize(Int nbr1,Int nbr2) {
  ligSize=nbr1;
  colSize=nbr2;
  if (ligSize>ligCapacity&&colSize>colCapacity)
    setLigColCapacity(nbr1,nbr2);
  else if (ligSize>ligCapacity)
    setLigCapacity(nbr1);
  else if (colSize>colCapacity)
    setColCapacity(nbr2);
}



void scal::setLigSize(Int nbr) {
  ligSize=nbr;
  if (ligSize>ligCapacity)
    setLigCapacity(nbr);
}



void scal::setColSize(Int nbr) {
  colSize=nbr;
  if (colSize>colCapacity)
    setColCapacity(nbr);
}



void scal::setLigIndex(Int nbr) {
  ligIndex=nbr;
}



void scal::setColIndex(Int nbr) {
  colIndex=nbr;
}



void scal::zeroInit(void) {
  register Double *diter;
  register bool *biter,*bFinal;
  register Double *dFinal;
  static Int i;
  
  if (white) {
    val=0.0;
    
    biter=ligBool;
    bFinal=biter+ligSize;
    while (biter<bFinal)
    *(biter++)=false;
  
    biter=colBool;
    bFinal=biter+colSize;
    while (biter<bFinal)
      *(biter++)=false;
    
    boolZeroValue=true;
    
    return;
  }
  
  val=0.0;
  
  diter=lig;
  biter=ligBool;
  dFinal=diter+ligSize;
  while (diter<dFinal) {
    *(biter++)=false;
    *(diter++)=0.0;
  }
  
  diter=col;
  biter=colBool;
  dFinal=diter+colSize;
  while (diter<dFinal) {
    *(biter++)=false;
    *(diter++)=0.0;
  }
  
  if (colSize>0&&ligSize>0) {
    for (i=0;i<colCapacity;i++) {
      diter=mat[i];
      dFinal=diter+ligSize;
      while (diter<dFinal)
        *(diter++)=0.0;
    }
  }
  
  boolZeroValue=true;
}



Double scal::getVal(void) const {
  return(val);
}



Int scal::getLigSize(void) const {
  return(ligSize);
}



Int scal::getLigIndex(void) const {
  return(ligIndex);
}



bool* scal::getLigBool(void) const {
  return(ligBool);
}



Double* scal::getLig(void) const {
  return(lig);
}



Int scal::getColSize(void) const {
  return(colSize);
}



Int scal::getColIndex(void) const {
  return(colIndex);
}



bool* scal::getColBool(void) const {
  return(colBool);
}



Double* scal::getCol(void) const {
  return(col);
}



Double** scal::getMat(void) const {
  return(mat);
}



bool scal::updateBoolZeroValue(void) {
  register Double *diter;
  register Double *dFinal;
  static Int i;

  if (white) {
    if (val==0.0&&ligSize==0&&colSize==0)
      boolZeroValue=true;
    else
      boolZeroValue=false;
    
    return(boolZeroValue);
  }
  
  if (val==0.0) {
    if ((colSize==0)&&(ligSize==0)) {
      setBoolZeroValue(true);
      return(boolZeroValue);
    }
    else {
      diter=lig;
      dFinal=diter+ligSize;
      while (diter<dFinal) {
	if ((*(diter++))!=0.0) {
	  setBoolZeroValue(false);
	  return(boolZeroValue);
        }
      }
      diter=col;
      dFinal=diter+colSize;
      while (diter<dFinal) {
	if ((*(diter++))!=0.0) {
	  setBoolZeroValue(false);
	  return(boolZeroValue);
        }
      }
      if (colSize>0&&ligSize>0) {
        for (i=0;i<colCapacity;i++) {
          diter=mat[i];
          dFinal=diter+ligSize;
          while (diter<dFinal) {
	    if ((*(diter++))!=0.0) {
	      setBoolZeroValue(false);
	      return(boolZeroValue);
	    }
	  }
        }
      }
      setBoolZeroValue(true);
      setLigColSize(0,0);
      return(boolZeroValue);
    }
  }
  else {
    boolZeroValue=false;
    return(boolZeroValue);
  }
}



void scal::writeMat(ostream &myCout) const {
  static Int i,j;
  for (i=0;i<colSize;i++) {
    if (colBool[i]) {
      for (j=0;j<ligSize;j++) {
	if (ligBool[j]) {
	  myCout << setw(15) << mat[i][j] ;
	}
      }
      myCout << endl ;
    }
  }
}



void scal::writeCol(ostream &myCout) const {
  static Int i;
  for (i=0;i<colSize;i++)
    if (colBool[i])
      myCout << setw(6) << map[colIndex+i] << setw(15) << col[i] << endl ;
}




void scal::writeLig(ostream &myCout) const {
  static Int i;
  for (i=0;i<ligSize;i++)
    if (ligBool[i])
      myCout << setw(6) << map[ligIndex+i] << setw(15) << lig[i] ;
  myCout << endl ;
}




void scal::writeVal(ostream &myCout) const {
  myCout << setw(15) << val << endl ;
}



void scal::writeMat(string &fileName) const {
  ofstream fout(fileName.c_str());
  writeMat(fout);
}



void scal::writeCol(string &fileName) const {
  ofstream fout(fileName.c_str());
  writeCol(fout);
}



void scal::writeLig(string &fileName) const {
  ofstream fout(fileName.c_str());
  writeLig(fout);
}



void scal::writeVal(string &fileName) const {
  ofstream fout(fileName.c_str());
  writeVal(fout);
}



void scal::stackClear(void) {
  stackSize=0;
}



void scal::stackPushBack(Double dArg,scal &sArg) {
  if (stackSize==stackCapacity) {
    static Int i;
    static Double *newDStack=NULL;
    static scal **newSStack=NULL;
    
    newDStack=new Double[stackCapacity+1];
    newSStack=new scal* [stackCapacity+1];
    for (i=0;i<stackCapacity;i++) {
      newDStack[i]=dStack[i];
      newSStack[i]=sStack[i];
    }
    delete[] dStack;
    delete[] sStack;
    dStack=newDStack;
    sStack=newSStack;
    newDStack=NULL;
    newSStack=NULL;
    stackCapacity++;
  }
  dStack[stackSize]=dArg;
  sStack[stackSize++]=&sArg;
}



