
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <iomanip>
#include <iostream>

#include "scal.h"
#include "err/err.h"
#include "function/function1.h"

using namespace std ;
using namespace keyFE2 ;



void scal::assignValuesWhite(const scal &in) {
  
  // Identical to the operator=, but does not modify the mapping.
  // One assumes that the mapping of "in" is contained in this mapping.
  
  register bool *biter1,*biter2,*bFinal;
  
  zeroInit();
  
  // Assigning value :
  val=in.val;
  
  // Building of col :
  biter1=colBool+in.colIndex-colIndex;
  biter2=in.colBool;
  bFinal=biter1+in.colSize;
  while (biter1<bFinal)
    *(biter1++)=*(biter2++);
  
  // Building of lig :
  biter1=ligBool+in.ligIndex-ligIndex;
  biter2=in.ligBool;
  bFinal=biter1+in.ligSize;
  while (biter1<bFinal)
    *(biter1++)=*(biter2++);
  
  if (val==0.0&&ligSize==0&&colSize==0)
    boolZeroValue=true;
  else
    boolZeroValue=false;
}



void scal::assignValues(const scal &in) {
  
  // Identical to the operator=, but does not modify the mapping.
  // One assumes that the mapping of "in" is contained in this mapping.
  
  Int i;
  register Double *diter1,*diter2;
  register bool *biter1,*biter2;
  register Double *dFinal;
  
  if (white) {
    assignValuesWhite(in);
    return;
  }    
  
  zeroInit();
  
  // Building val :
  val=in.val;
  
  // Building of col :
  biter1=colBool+in.colIndex-colIndex;
  biter2=in.colBool;
  diter1=col+in.colIndex-colIndex;
  diter2=in.col;
  dFinal=diter1+in.colSize;
  while (diter1<dFinal) {
    *(biter1++)=*(biter2++);
    *(diter1++)=*(diter2++);
  }
  
  // Building of lig :
  biter1=ligBool+in.ligIndex-ligIndex;
  biter2=in.ligBool;
  diter1=lig+in.ligIndex-ligIndex;
  diter2=in.lig;  
  dFinal=diter1+in.ligSize;
  while (diter1<dFinal) {
    *(biter1++)=*(biter2++);
    *(diter1++)=*(diter2++);
  }
  
  // Building of mat :
  if (in.ligSize!=0) {
    for (i=0;i<in.colSize;i++) {
      if (in.colBool[i]) {
	diter1=mat[i+in.colIndex-colIndex]+in.ligIndex-ligIndex;
	diter2=in.mat[i];
	dFinal=diter1+in.ligSize;
	while (diter1<dFinal)
	  *(diter1++)=*(diter2++);
      }
    }
  }
  
  boolZeroValue=in.boolZeroValue;
}



void scal::uminusAssignValues(const scal &in) {
  
  // Identical to the operator-=, but does not modify the mapping.
  // One assumes that the mapping of "in" is contained in this mapping.
  
  Int i;
  register Double *diter1,*diter2;
  register bool *biter1,*biter2;
  register Double *dFinal;
  
  if (white) {
    assignValuesWhite(in);
    return;
  }    
  
  zeroInit();
  
  // Building val :
  val=-in.val;
  
  // Building of col :
  biter1=colBool+in.colIndex-colIndex;
  biter2=in.colBool;
  diter1=col+in.colIndex-colIndex;
  diter2=in.col;
  dFinal=diter1+in.colSize;
  while (diter1<dFinal) {
    *(biter1++)=*(biter2++);
    *(diter1++)=-*(diter2++);
  }
  
  // Building of lig :
  biter1=ligBool+in.ligIndex-ligIndex;
  biter2=in.ligBool;
  diter1=lig+in.ligIndex-ligIndex;
  diter2=in.lig;  
  dFinal=diter1+in.ligSize;
  while (diter1<dFinal) {
    *(biter1++)=*(biter2++);
    *(diter1++)=-*(diter2++);
  }
  
  // Building of mat :
  if (in.ligSize!=0) {
    for (i=0;i<in.colSize;i++) {
      if (in.colBool[i]) {
	diter1=mat[i+in.colIndex-colIndex]+in.ligIndex-ligIndex;
	diter2=in.mat[i];
	dFinal=diter1+in.ligSize;
	while (diter1<dFinal)
	  *(diter1++)=-*(diter2++);
      }
    }
  }
  
  boolZeroValue=in.boolZeroValue;
}



void scal::multAssignValues(Double factor,const scal &in) {
  
  // Identical to the operator=, but does not modify the mapping.
  // One assumes that the mapping of "in" is contained in this mapping.
  
  static Int i;
  register Double *diter1,*diter2;
  register bool *biter1,*biter2;
  register Double *dFinal;
  
  if (factor==0.0) {
    zeroInit();
    boolZeroValue=true;
  }
  
  if (white) {
    assignValuesWhite(in);
    return;
  }    
  
  zeroInit();

  // Building val :
  val=factor*in.val;
  
  // Building of col :
  biter1=colBool+in.colIndex-colIndex;
  biter2=in.colBool;
  diter1=col+in.colIndex-colIndex;
  diter2=in.col;
  dFinal=diter1+in.colSize;
  while (diter1<dFinal) {
    *(biter1++)=*(biter2++);
    *(diter1++)=factor*(*(diter2++));
  }
  
  // Building of lig :
  biter1=ligBool+in.ligIndex-ligIndex;
  biter2=in.ligBool;
  diter1=lig+in.ligIndex-ligIndex;
  diter2=in.lig;  
  dFinal=diter1+in.ligSize;
  while (diter1<dFinal) {
    *(biter1++)=*(biter2++);
    *(diter1++)=factor*(*(diter2++));
  }
  
  // Building of mat :
  if (in.ligSize!=0) {
    for (i=0;i<in.colSize;i++) {
      if (in.colBool[i]) {
	diter1=mat[i+in.colIndex-colIndex]+in.ligIndex-ligIndex;
	diter2=in.mat[i];
	dFinal=diter1+in.ligSize;
	while (diter1<dFinal)
	  *(diter1++)=factor*(*(diter2++));
      }
    }
  }
  
  boolZeroValue=in.boolZeroValue;
}



void scal::multAddValues(Double factor,const scal &in) {
  
  // Identical to the operator=, but does not modify the mapping.
  // One assumes that the mapping of "in" is contained in this mapping.
  
  static Int i;
  register Double *diter1,*diter2;
  register bool *biter1,*biter2;
  register Double *dFinal;
  
  if (factor==0.0)
    return;
  
  if (white) {
    incWhite(in);
    val+=factor*in.val;
    
    if (val==0.0&&ligSize==0&&colSize==0)
      boolZeroValue=true;
    else
      boolZeroValue=false;
    
    return;
  }    
  
  // Building val :
  val+=factor*in.val;
  
  // Building of col :
  biter1=colBool+in.colIndex-colIndex;
  biter2=in.colBool;
  diter1=col+in.colIndex-colIndex;
  diter2=in.col;
  dFinal=diter1+in.colSize;
  while (diter1<dFinal) {
    *(biter1++)|=*(biter2++);
    *(diter1++)+=factor*(*(diter2++));
  }
  
  // Building of lig :
  biter1=ligBool+in.ligIndex-ligIndex;
  biter2=in.ligBool;
  diter1=lig+in.ligIndex-ligIndex;
  diter2=in.lig;  
  dFinal=diter1+in.ligSize;
  while (diter1<dFinal) {
    *(biter1++)|=*(biter2++);
    *(diter1++)+=factor*(*(diter2++));
  }
  
  // Building of mat :
  if (in.ligSize!=0) {
    for (i=0;i<in.colSize;i++) {
      if (in.colBool[i]) {
	diter1=mat[i+in.colIndex-colIndex]+in.ligIndex-ligIndex;
	diter2=in.mat[i];
	dFinal=diter1+in.ligSize;
	while (diter1<dFinal)
	  *(diter1++)+=factor*(*(diter2++));
      }
    }
  }
  boolZeroValue=false;
}



void scal::assignWhite(const scal &in) {
  register bool *biter1,*biter2,*bFinal;
  
  setLigColSize(in.ligSize,in.colSize);
  colIndex=in.colIndex;
  ligIndex=in.ligIndex;
  
  // Building of val :
  val=in.val;
  
  // Building of col :
  biter1=colBool;
  biter2=in.colBool;
  bFinal=biter1+colSize;
  while (biter1<bFinal)
    *(biter1++)=*(biter2++);
  
  // Building of lig :
  biter1=ligBool;
  biter2=in.ligBool;
  bFinal=biter1+ligSize;
  while (biter1<bFinal)
    *(biter1++)=*(biter2++);
  
  if (val==0.0&&ligSize==0&&colSize==0)
    boolZeroValue=true;
  else
    boolZeroValue=false;
}



void scal::operator=(const scal &in) {
  Int i;
  register Double *diter1,*diter2,*dFinal;
  register bool *biter1,*biter2;
  
  if (in.isZero()) {
    setLigColSize(0,0);
    val=0.0;
    boolZeroValue=true;
    return;
  }

  if (white) {
    assignWhite(in);
    return;
  }
  
  setLigColSize(in.ligSize,in.colSize);
  colIndex=in.colIndex;
  ligIndex=in.ligIndex;
  
  // Building val :
  val=in.val;
  
  // Building of col :
  biter1=colBool;
  biter2=in.colBool;
  diter1=col;
  diter2=in.col;
  dFinal=diter1+colSize;
  while (diter1<dFinal) {
    *(biter1++)=*(biter2++);
    *(diter1++)=*(diter2++);
  }
  
  // Building of lig :
  biter1=ligBool;
  biter2=in.ligBool;
  diter1=lig;
  diter2=in.lig;  
  dFinal=diter1+ligSize;
  while (diter1<dFinal) {
    *(biter1++)=*(biter2++);
    *(diter1++)=*(diter2++);
  }
  
  // Building of mat :
  if (in.ligSize!=0) {
    for (i=0;i<colSize;i++) {
      if (colBool[i]) {
	diter1=mat[i];
	diter2=in.mat[i];
	dFinal=diter1+ligSize;
	while (diter1<dFinal)
	  *(diter1++)=*(diter2++);
      }
      else {
	diter1=mat[i];
	dFinal=diter1+ligSize;
	while (diter1<dFinal)
	  *(diter1++)=0.0;
      }
    }
  }
    
  boolZeroValue=in.boolZeroValue;
}



void scal::operator=(Double D) {
  setLigColSize(0,0);  
  val=D;
  
  if (val==0.0)
    boolZeroValue=true;
  else
    boolZeroValue=false;
}



void scal::setToTransposedOf(const scal &in) {
  register Double *diter1,*diter2,*dFinal;
  register bool *biter1,*biter2,*bFinal;
  
  if (in.isZero()) {
    setLigColSize(0,0);
    val=0.0;
    boolZeroValue=true;
    return;
  }

  if (white) {
    
    zeroInit();
    
    // Assigning value :
    val=in.val;
    
    setLigColSize(in.ligSize,in.colSize);
    colIndex=in.colIndex;
    ligIndex=in.ligIndex;
    
    // Building of col :
    biter1=colBool;
    biter2=in.colBool;
    bFinal=biter1+colSize;
    while (biter1<bFinal)
      *(biter1++)=*(biter2++);
    
    // Building of lig :
    biter1=ligBool;
    biter2=in.ligBool;
    bFinal=biter1+ligSize;
    while (biter1<bFinal)
      *(biter1++)=*(biter2++);
    
    if (val==0.0&&ligSize==0&&colSize==0)
      boolZeroValue=true;
    else
      boolZeroValue=false;
    
    return;
  }

  setLigColSize(in.colSize,in.ligSize);
  colIndex=in.ligIndex;
  ligIndex=in.colIndex;
  
  // Building val :
  val=in.val;
  
  // Building of col :
  if (colSize>0) {
    biter1=colBool;
    biter2=in.ligBool;
    diter1=col;
    diter2=in.lig;
    dFinal=diter1+colSize;
    while (diter1<dFinal) {
      *(biter1++)=*(biter2++);
      *(diter1++)=*(diter2++);
    }
  }
  
  // Building of lig :
  if (ligSize>0) {
    biter1=ligBool;
    biter2=in.colBool;
    diter1=lig;
    diter2=in.col;  
    dFinal=diter1+ligSize;
    while (diter1<dFinal) {
      *(biter1++)=*(biter2++);
      *(diter1++)=*(diter2++);
    }
  }
  
  // Building of mat :
  if (colSize>0&&ligSize>0)
    errThrow("scal::setToTransposedOf",
	     "only transposition of lines and columns has been programmed.");
//  if (in.ligSize!=0) {
//    for (i=0;i<colSize;i++) {
//      if (colBool[i]) {
//	diter1=mat[i];
//	diter2=in.mat[i];
//	dFinal=diter1+ligSize;
//	while (diter1<dFinal)
//	  *(diter1++)=*(diter2++);
//      }
//      else {
//	diter1=mat[i];
//	dFinal=diter1+ligSize;
//	while (diter1<dFinal)
//	  *(diter1++)=0.0;
//      }
//    }
//  }
    
  boolZeroValue=in.boolZeroValue;
}




void scal::mapMerge(const scal &left,const scal &right) {
  static Int indexMin,indexMax;
  static Int lSize,cSize;
  
  if (!(left.ligSize||right.ligSize)) {
    ligIndex=0;
    lSize=0;
  }
  else {
    indexMin=mapSize;
    if (left.ligSize)
      indexMin=min(indexMin,left.ligIndex);
    if (right.ligSize)
      indexMin=min(indexMin,right.ligIndex);
    indexMax=max(left.ligIndex+left.ligSize-1,right.ligIndex+right.ligSize-1);
    ligIndex=indexMin;
    lSize=indexMax-indexMin+1;
  }
  
  if (!(left.colSize||right.colSize)) {
    colIndex=0;
    cSize=0;
  }
  else {
    indexMin=mapSize;
    if (left.colSize)
      indexMin=min(indexMin,left.colIndex);
    if (right.colSize)
      indexMin=min(indexMin,right.colIndex);
    indexMax=max(left.colIndex+left.colSize-1,right.colIndex+right.colSize-1);
    colIndex=indexMin;
    cSize=indexMax-indexMin+1;
  }
  
  setLigColSize(lSize,cSize);
}



void scal::mapMerge(const scal &s1,const scal &s2,const scal &s3) {
  static Int indexMin,indexMax;
  static Int lSize,cSize;
  
  if (!(s1.ligSize||s2.ligSize||s3.ligSize)) {
    ligIndex=0;
    lSize=0;
  }
  else {
    indexMin=mapSize;
    if (s1.ligSize)
      indexMin=min(indexMin,s1.ligIndex);
    if (s2.ligSize)
      indexMin=min(indexMin,s2.ligIndex);
    if (s3.ligSize)
      indexMin=min(indexMin,s3.ligIndex);
    indexMax=max(s1.ligIndex+s1.ligSize-1,s2.ligIndex+s2.ligSize-1,
	         s3.ligIndex+s3.ligSize-1);
    ligIndex=indexMin;
    lSize=indexMax-indexMin+1;
  }
  
  if (!(s1.colSize||s2.colSize||s3.colSize)) {
    colIndex=0;
    cSize=0;
  }
  else {
    indexMin=mapSize;
    if (s1.colSize)
      indexMin=min(indexMin,s1.colIndex);
    if (s2.colSize)
      indexMin=min(indexMin,s2.colIndex);
    if (s3.colSize)
      indexMin=min(indexMin,s3.colIndex);
    indexMax=max(s1.colIndex+s1.colSize-1,s2.colIndex+s2.colSize-1,
  	         s3.colIndex+s3.colSize-1);
    colIndex=indexMin;
    cSize=indexMax-indexMin+1;
  }
  
  setLigColSize(lSize,cSize);
}



void scal::dyadicWhite(const scal &left,const scal &right) {
  register bool *biter1,*biter2,*bFinal;
  
  mapMerge(left,right);
  zeroInit();
  
  // Building of col :
  biter1=colBool+(left.colIndex-colIndex);
  biter2=left.colBool;
  bFinal=biter2+left.colSize;
  while(biter2<bFinal)
    *(biter1++)=*(biter2++);
  biter1=colBool+(right.colIndex-colIndex);
  biter2=right.colBool;
  bFinal=biter2+right.colSize;
  while(biter2<bFinal)
    *(biter1++)|=*(biter2++);
  
  // Building of lig :
  biter1=ligBool+(left.ligIndex-ligIndex);
  biter2=left.ligBool;
  bFinal=biter2+left.ligSize;
  while(biter2<bFinal)
    *(biter1++)=*(biter2++);
  biter1=ligBool+(right.ligIndex-ligIndex);
  biter2=right.ligBool;
  bFinal=biter2+right.ligSize;
  while(biter2<bFinal)
    *(biter1++)|=*(biter2++);
  
  boolZeroValue=false;
}



void scal::sum(const scal &left,const scal &right) {
  static Int i;
  register Double *diter1,*diter2,*dFinal;
  register bool *biter1,*biter2;
  
  if (left.isZero()) {
    this->operator=(right);
    return;
  }
  else if (right.isZero()) {
    this->operator=(left);
    return;
  }
  else {   
    
    if (white) {
      dyadicWhite(left,right);
      val=left.val+right.val;
      
      if (val==0.0&&ligSize==0&&colSize==0)
	boolZeroValue=true;
      else
	boolZeroValue=false;
      
      return;
    }    
    
    mapMerge(left,right);
    zeroInit();
    
    // Building val :
    val=left.val+right.val;
    
    // Building of col :
    diter1=col+(left.colIndex-colIndex);
    diter2=left.col;
    dFinal=diter2+left.colSize;
    biter1=colBool+(left.colIndex-colIndex);
    biter2=left.colBool;
    while(diter2<dFinal) {
      *(diter1++)=*(diter2++);
      *(biter1++)=*(biter2++);
    }
    diter1=col+(right.colIndex-colIndex);
    diter2=right.col;
    dFinal=diter2+right.colSize;
    biter1=colBool+(right.colIndex-colIndex);
    biter2=right.colBool;
    while(diter2<dFinal) {
      *(diter1++)+=*(diter2++);
      *(biter1++)|=*(biter2++);
    }
    
    // Building of lig :
    diter1=lig+(left.ligIndex-ligIndex);
    diter2=left.lig;
    dFinal=diter2+left.ligSize;
    biter1=ligBool+(left.ligIndex-ligIndex);
    biter2=left.ligBool;
    while(diter2<dFinal) {
      *(diter1++)=*(diter2++);
      *(biter1++)=*(biter2++);
    }
    diter1=lig+(right.ligIndex-ligIndex);
    diter2=right.lig;
    dFinal=diter2+right.ligSize;
    biter1=ligBool+(right.ligIndex-ligIndex);
    biter2=right.ligBool;
    while(diter2<dFinal) {
      *(diter1++)+=*(diter2++);
      *(biter1++)|=*(biter2++);
    }
    
    // Building of mat :
    if (left.ligSize!=0) {
      for (i=0;i<left.colSize;i++) {
	if (left.colBool[i]) {
	  diter1=mat[i+left.colIndex-colIndex]+(left.ligIndex-ligIndex);
	  diter2=left.mat[i];
	  dFinal=diter2+left.ligSize;
	  while (diter2<dFinal)
	    *(diter1++)=*(diter2++);
	}
      }
    }
    if (right.ligSize!=0) {
      for (i=0;i<right.colSize;i++) {
	if (right.colBool[i]) {
	  diter1=mat[i+right.colIndex-colIndex]+(right.ligIndex-ligIndex);
	  diter2=right.mat[i];
	  dFinal=diter2+right.ligSize;
	  while (diter2<dFinal)
	    *(diter1++)+=*(diter2++);
	}
      }
    }
    
    updateBoolZeroValue();
  }
}



void scal::diff(const scal &left,const scal &right) {
  static Int i;
  register Double *diter1,*diter2;
  register bool *biter1,*biter2;
  register Double *dFinal;
  
  if (right.isZero()) {
    this->operator=(left);
    return;
  }
  else {
    
    if (white) {
      dyadicWhite(left,right);
      val=left.val-right.val;
      
      if (val==0.0&&ligSize==0&&colSize==0)
	boolZeroValue=true;
      else
	boolZeroValue=false;
      
      return;
    }    
    
    mapMerge(left,right);
    zeroInit();
    
    // Building val :
    val=left.val-right.val;
    
    // Building of col :
    diter1=col+(left.colIndex-colIndex);
    diter2=left.col;
    dFinal=diter2+left.colSize;
    biter1=colBool+(left.colIndex-colIndex);
    biter2=left.colBool;
    while(diter2<dFinal) {
      *(diter1++)=*(diter2++);
      *(biter1++)=*(biter2++);
    }
    diter1=col+(right.colIndex-colIndex);
    diter2=right.col;
    dFinal=diter2+right.colSize;
    biter1=colBool+(right.colIndex-colIndex);
    biter2=right.colBool;
    while(diter2<dFinal) {
      *(diter1++)-=*(diter2++);
      *(biter1++)|=*(biter2++);
    }
    
    // Building of lig :
    diter1=lig+(left.ligIndex-ligIndex);
    diter2=left.lig;
    dFinal=diter2+left.ligSize;
    biter1=ligBool+(left.ligIndex-ligIndex);
    biter2=left.ligBool;
    while(diter2<dFinal) {
      *(diter1++)=*(diter2++);
      *(biter1++)=*(biter2++);
    }
    diter1=lig+(right.ligIndex-ligIndex);
    diter2=right.lig;
    dFinal=diter2+right.ligSize;
    biter1=ligBool+(right.ligIndex-ligIndex);
    biter2=right.ligBool;
    while(diter2<dFinal) {
      *(diter1++)-=*(diter2++);
      *(biter1++)|=*(biter2++);
    }
    
    // Building of mat :
    if (left.ligSize!=0) {
      for (i=0;i<left.colSize;i++) {
	if (left.colBool[i]) {
	  diter1=mat[i+left.colIndex-colIndex]+(left.ligIndex-ligIndex);
	  diter2=left.mat[i];
	  dFinal=diter2+left.ligSize;
	  while (diter2<dFinal)
	    *(diter1++)=*(diter2++);
	}
      }
    }
    if (right.ligSize!=0) {
      for (i=0;i<right.colSize;i++) {
	if (right.colBool[i]) {
	  diter1=mat[i+right.colIndex-colIndex]+(right.ligIndex-ligIndex);
	  diter2=right.mat[i];
	  dFinal=diter2+right.ligSize;
	  while (diter2<dFinal)
	    *(diter1++)-=*(diter2++);
	}
      }
    }
    
    updateBoolZeroValue();
  }
}



void scal::mult(const scal &left,const scal &right) {
  static Int i;
  register Double *diter1,*diter2;
  register bool *biter1,*biter2;
  register Double *dFinal;
  register Double factor;  
  
  if (left.isZero()||right.isZero()) {
    this->operator=(0.0);
    return;
  }
  else {
    
    if (white) {
      dyadicWhite(left,right);
      val=left.val*right.val;
      
      if (val==0.0&&ligSize==0&&colSize==0)
	boolZeroValue=true;
      else
	boolZeroValue=false;
      
      return;
    }    
    
    mapMerge(left,right);
    zeroInit();
    
    // Building of val :
    val=left.val*right.val;
    
    // Building of col :
    diter1=col+(left.colIndex-colIndex);
    diter2=left.col;
    dFinal=diter2+left.colSize;
    biter1=colBool+(left.colIndex-colIndex);
    biter2=left.colBool;
    factor=right.val;
    while(diter2<dFinal) {
      *(diter1++)=factor*(*(diter2++));
      *(biter1++)=*(biter2++);
    }
    diter1=col+(right.colIndex-colIndex);
    diter2=right.col;
    dFinal=diter2+right.colSize;
    biter1=colBool+(right.colIndex-colIndex);
    biter2=right.colBool;
    factor=left.val;
    while(diter2<dFinal) {
      *(diter1++)+=factor*(*(diter2++));
      *(biter1++)|=*(biter2++);
    }
    
    // Building of lig :
    diter1=lig+(left.ligIndex-ligIndex);
    diter2=left.lig;
    dFinal=diter2+left.ligSize;
    biter1=ligBool+(left.ligIndex-ligIndex);
    biter2=left.ligBool;
    factor=right.val;
    while(diter2<dFinal) {
      *(diter1++)=factor*(*(diter2++));
      *(biter1++)=*(biter2++);
    }
    diter1=lig+(right.ligIndex-ligIndex);
    diter2=right.lig;
    dFinal=diter2+right.ligSize;
    biter1=ligBool+(right.ligIndex-ligIndex);
    biter2=right.ligBool;
    factor=left.val;
    while(diter2<dFinal) {
      *(diter1++)+=factor*(*(diter2++));
      *(biter1++)|=*(biter2++);
    }
    
    // Building of mat :
    if (left.ligSize!=0) {
      factor=right.val;
      for (i=0;i<left.colSize;i++) {
        if (left.colBool[i]) {
          diter1=mat[i+left.colIndex-colIndex]+(left.ligIndex-ligIndex);
          diter2=left.mat[i];
          dFinal=diter2+left.ligSize;
          while (diter2<dFinal)
            *(diter1++)=factor*(*(diter2++));
        }
      }
    }
    if (right.ligSize!=0) {
      factor=left.val;
      for (i=0;i<right.colSize;i++) {
        if (right.colBool[i]) {
          diter1=mat[i+right.colIndex-colIndex]+(right.ligIndex-ligIndex);
          diter2=right.mat[i];
          dFinal=diter2+right.ligSize;
          while (diter2<dFinal)
            *(diter1++)+=factor*(*(diter2++));
	}
      }
    }
    if (right.ligSize!=0) {
      for (i=0;i<left.colSize;i++) {
	if (left.colBool[i]) {
	  factor=left.col[i];
	  diter1=mat[i+left.colIndex-colIndex]+(right.ligIndex-ligIndex);
	  diter2=right.lig;
	  dFinal=diter2+right.ligSize;
	  while (diter2<dFinal)
	    *(diter1++)+=factor*(*(diter2++));
	}
      }
    }
    if (left.ligSize!=0) {
      for (i=0;i<right.colSize;i++) {
	if (right.colBool[i]) {
	  factor=right.col[i];
	  diter1=mat[i+right.colIndex-colIndex]+(left.ligIndex-ligIndex);
	  diter2=left.lig;
	  dFinal=diter2+left.ligSize;
	  while (diter2<dFinal)
	    *(diter1++)+=factor*(*(diter2++));
	}
      }
    }
    
    updateBoolZeroValue();
  }
}



void scal::uminusMult(const scal &left,const scal &right) {
  static Int i;
  register Double *diter1,*diter2;
  register bool *biter1,*biter2;
  register Double *dFinal;
  register Double factor;
  
  if (left.isZero()||right.isZero()) {
    this->operator=(0.0);
    return;
  }
  else {
    
    if (white) {
      dyadicWhite(left,right);
      val=-left.val*right.val;
      
      if (val==0.0&&ligSize==0&&colSize==0)
	boolZeroValue=true;
      else
	boolZeroValue=false;
      
      return;
    }    
    
    mapMerge(left,right);
    zeroInit();
    
    // Building of val :
    val=-left.val*right.val;
    
    // Building of col :
    diter1=col+(left.colIndex-colIndex);
    diter2=left.col;
    dFinal=diter2+left.colSize;
    biter1=colBool+(left.colIndex-colIndex);
    biter2=left.colBool;
    factor=right.val;
    while(diter2<dFinal) {
      *(diter1++)=-factor*(*(diter2++));
      *(biter1++)=*(biter2++);
    }
    diter1=col+(right.colIndex-colIndex);
    diter2=right.col;
    dFinal=diter2+right.colSize;
    biter1=colBool+(right.colIndex-colIndex);
    biter2=right.colBool;
    factor=left.val;
    while(diter2<dFinal) {
      *(diter1++)-=factor*(*(diter2++));
      *(biter1++)|=*(biter2++);
    }
    
    // Building of lig :
    diter1=lig+(left.ligIndex-ligIndex);
    diter2=left.lig;
    dFinal=diter2+left.ligSize;
    biter1=ligBool+(left.ligIndex-ligIndex);
    biter2=left.ligBool;
    factor=right.val;
    while(diter2<dFinal) {
      *(diter1++)=-factor*(*(diter2++));
      *(biter1++)=*(biter2++);
    }
    diter1=lig+(right.ligIndex-ligIndex);
    diter2=right.lig;
    dFinal=diter2+right.ligSize;
    biter1=ligBool+(right.ligIndex-ligIndex);
    biter2=right.ligBool;
    factor=left.val;
    while(diter2<dFinal) {
      *(diter1++)-=factor*(*(diter2++));
      *(biter1++)|=*(biter2++);
    }
    
    // Building of mat :
    if (left.ligSize!=0) {
      factor=right.val;
      for (i=0;i<left.colSize;i++) {
        if (left.colBool[i]) {
          diter1=mat[i+left.colIndex-colIndex]+(left.ligIndex-ligIndex);
          diter2=left.mat[i];
          dFinal=diter2+left.ligSize;
          while (diter2<dFinal)
            *(diter1++)=-factor*(*(diter2++));
        }
      }
    }
    if (right.ligSize!=0) {
      factor=left.val;
      for (i=0;i<right.colSize;i++) {
        if (right.colBool[i]) {
          diter1=mat[i+right.colIndex-colIndex]+(right.ligIndex-ligIndex);
          diter2=right.mat[i];
          dFinal=diter2+right.ligSize;
          while (diter2<dFinal)
            *(diter1++)-=factor*(*(diter2++));
	}
      }
    }
    if (right.ligSize!=0) {
      for (i=0;i<left.colSize;i++) {
	if (left.colBool[i]) {
	  factor=left.col[i];
	  diter1=mat[i+left.colIndex-colIndex]+(right.ligIndex-ligIndex);
	  diter2=right.lig;
	  dFinal=diter2+right.ligSize;
	  while (diter2<dFinal)
	    *(diter1++)-=factor*(*(diter2++));
	}
      }
    }
    if (left.ligSize!=0) {
      for (i=0;i<right.colSize;i++) {
	if (right.colBool[i]) {
	  factor=right.col[i];
	  diter1=mat[i+right.colIndex-colIndex]+(left.ligIndex-ligIndex);
	  diter2=left.lig;
	  dFinal=diter2+left.ligSize;
	  while (diter2<dFinal)
	    *(diter1++)-=factor*(*(diter2++));
	}
      }
    }
    
    updateBoolZeroValue();
  }
}



void scal::uminusAssign(const scal &in) {
  Int i;
  register Double *diter1,*diter2;
  register bool *biter1,*biter2;
  register Double *dFinal;
  
  if (in.isZero()) {
    setLigColSize(0,0);
    val=0.0;
    boolZeroValue=true;
    return;
  }

  if (white) {
    assignWhite(in);
    val=-in.val;
    return;
  }
  
  setLigColSize(in.ligSize,in.colSize);
  colIndex=in.colIndex;
  ligIndex=in.ligIndex;
  
  // Building val :
  val=-in.val;
  
  // Building of col :
  biter1=colBool;
  biter2=in.colBool;
  diter1=col;
  diter2=in.col;
  dFinal=diter1+colSize;
  while (diter1<dFinal) {
    *(biter1++)=-*(biter2++);
    *(diter1++)=-*(diter2++);
  }
  
  // Building of lig :
  biter1=ligBool;
  biter2=in.ligBool;
  diter1=lig;
  diter2=in.lig;  
  dFinal=diter1+ligSize;
  while (diter1<dFinal) {
    *(biter1++)=-*(biter2++);
    *(diter1++)=-*(diter2++);
  }
  
  // Building of mat :
  if (in.ligSize!=0) {
    for (i=0;i<colSize;i++) {
      if (colBool[i]) {
	diter1=mat[i];
	diter2=in.mat[i];
	dFinal=diter1+ligSize;
	while (diter1<dFinal)
	  *(diter1++)=-*(diter2++);
      }
      else {
	diter1=mat[i];
	dFinal=diter1+ligSize;
	while (diter1<dFinal)
	  *(diter1++)=0.0;
      }
    }
  }
    
  boolZeroValue=in.boolZeroValue;
}



void scal::div(const scal &left,const scal &right) {
  static Int i;
  register Double *diter1,*diter2;
  register bool *biter1,*biter2;
  register Double *dFinal;
  register Double factor;
  register Double rightVal,rightVal2,rightVal3;
  
  if (left.isZero()) {
    this->operator=(0.0);
    return;
  }
  else if (right.isZero()) {
    errThrow("scal::div","division by zero not allowed.");
  }
  else {
    
    if (white) {
      dyadicWhite(left,right);
      val=left.val/right.val;
      
      if (val==0.0&&ligSize==0&&colSize==0)
	boolZeroValue=true;
      else
	boolZeroValue=false;
      
      return;
    }    
    
    rightVal=right.val;
    rightVal2=rightVal*rightVal;
    rightVal3=rightVal2*rightVal;
    
    mapMerge(left,right);
    zeroInit();
    
    // Building of val :
    val=left.val/rightVal;
    
    // Building of col :
    diter1=col+(left.colIndex-colIndex);
    diter2=left.col;
    dFinal=diter2+left.colSize;
    biter1=colBool+(left.colIndex-colIndex);
    biter2=left.colBool;
    factor=1.0/rightVal;
    while(diter2<dFinal) {
      *(diter1++)=factor*(*(diter2++));
      *(biter1++)=*(biter2++);
    }
    diter1=col+(right.colIndex-colIndex);
    diter2=right.col;
    dFinal=diter2+right.colSize;
    biter1=colBool+(right.colIndex-colIndex);
    biter2=right.colBool;
    factor=-left.val/rightVal2;
    while(diter2<dFinal) {
      *(diter1++)+=factor*(*(diter2++));
      *(biter1++)|=*(biter2++);
    }
    
    // Building of lig :
    diter1=lig+(left.ligIndex-ligIndex);
    diter2=left.lig;
    dFinal=diter2+left.ligSize;
    biter1=ligBool+(left.ligIndex-ligIndex);
    biter2=left.ligBool;
    factor=1.0/rightVal;
    while(diter2<dFinal) {
      *(diter1++)=factor*(*(diter2++));
      *(biter1++)=*(biter2++);
    }
    diter1=lig+(right.ligIndex-ligIndex);
    diter2=right.lig;
    dFinal=diter2+right.ligSize;
    biter1=ligBool+(right.ligIndex-ligIndex);
    biter2=right.ligBool;
    factor=-left.val/rightVal2;
    while(diter2<dFinal) {
      *(diter1++)+=factor*(*(diter2++));
      *(biter1++)|=*(biter2++);
    }
    
    // Building of mat :
    if (left.ligSize!=0) {
      factor=1.0/rightVal;
      for (i=0;i<left.colSize;i++) {
	if (left.colBool[i]) {
	  diter1=mat[i+left.colIndex-colIndex]+(left.ligIndex-ligIndex);
	  diter2=left.mat[i];
	  dFinal=diter2+left.ligSize;
	  while (diter2<dFinal)
	    *(diter1++)=factor*(*(diter2++));
	}
      }
    }
    if (right.ligSize!=0) {
      factor=-left.val/rightVal2;
      for (i=0;i<right.colSize;i++) {
	if (right.colBool[i]) {
	  diter1=mat[i+right.colIndex-colIndex]+(right.ligIndex-ligIndex);
	  diter2=right.mat[i];
	  dFinal=diter2+right.ligSize;
	  while (diter2<dFinal)
	    *(diter1++)+=factor*(*(diter2++));
	}
      }
    }
    if (right.ligSize!=0) {
      for (i=0;i<left.colSize;i++) {
	if (left.colBool[i]) {
	  factor=-left.col[i]/rightVal2;
	  diter1=mat[i+left.colIndex-colIndex]+(right.ligIndex-ligIndex);
	  diter2=right.lig;
	  dFinal=diter2+right.ligSize;
	  while (diter2<dFinal)
	    *(diter1++)+=factor*(*(diter2++));
	}
      }
    }
    if (left.ligSize!=0) {
      for (i=0;i<right.colSize;i++) {
	if (right.colBool[i]) {
	  factor=-right.col[i]/rightVal2;
	  diter1=mat[i+right.colIndex-colIndex]+(left.ligIndex-ligIndex);
	  diter2=left.lig;
	  dFinal=diter2+left.ligSize;
	  while (diter2<dFinal)
	    *(diter1++)+=factor*(*(diter2++));
	}
      }
    }
    if (right.ligSize!=0) {
      for (i=0;i<right.colSize;i++) {
	if (right.colBool[i]) {
	  factor=(2.0*left.val/rightVal3)*right.col[i];
	  diter1=mat[i+right.colIndex-colIndex]+(right.ligIndex-ligIndex);
	  diter2=right.lig;
	  dFinal=diter2+right.ligSize;
	  while (diter2<dFinal)
	    *(diter1++)+=factor*(*(diter2++));
	}
      }
    }
    
    updateBoolZeroValue();
  }
}



void scal::mult(const scal &left,Double right) {
  Int i;
  register Double *diter1,*diter2;
  register bool *biter1,*biter2;
  register Double *dFinal;
  register Double factor;
  
  if (left.isZero()||right==0.0) {
    this->operator=(0.0);
    return;
  }
  else {
    if (white) {
      assignWhite(left);
      val=left.val*right;
      
      if (val==0.0&&ligSize==0&&colSize==0)
	boolZeroValue=true;
      else
	boolZeroValue=false;
      
      return;
    }
    
    setLigColSize(left.ligSize,left.colSize);
    colIndex=left.colIndex;
    ligIndex=left.ligIndex;
    zeroInit();
    
    factor=right;
    
    // Building of val :
    val=factor*left.val;
    
    // Building of col :
    biter1=colBool;
    biter2=left.colBool;
    diter1=col;
    diter2=left.col;
    dFinal=diter1+colSize;
    while (diter1<dFinal) {
      *(biter1++)=*(biter2++);
      *(diter1++)=factor*(*(diter2++));
    }
    
    // Building of lig :
    biter1=ligBool;
    biter2=left.ligBool;
    diter1=lig;
    diter2=left.lig;  
    dFinal=diter1+ligSize;
    while (diter1<dFinal) {
      *(biter1++)=*(biter2++);
      *(diter1++)=factor*(*(diter2++));
    }
    
    // Building of mat :
    if (left.ligSize!=0) {
      for (i=0;i<colSize;i++) {
	if (colBool[i]) {
	  diter1=mat[i];
	  diter2=left.mat[i];
	  dFinal=diter1+ligSize;
	  while (diter1<dFinal)
	    *(diter1++)=factor*(*(diter2++));
	}
	else {
	  diter1=mat[i];
	  dFinal=diter1+ligSize;
	  while (diter1<dFinal)
	    *(diter1++)=0.0;
	}
      }
    }
    
    updateBoolZeroValue();
  }
}



void scal::div(const scal &left,Double right) {
  if (left.isZero()) {
    this->operator=(0.0);
    return;
  }
  else if (right==0.0) {
    errThrow("scal::div","division by zero not allowed.");
  }
  else {
    
    if (white) {
      assignWhite(left);
      return;
    }
    
    mult(left,1.0/right);
  }
}



void scal::doubleDiff(const Double &d,const scal &left,const scal &right) {
  static Int i;
  register Double *diter1,*diter2;
  register bool *biter1,*biter2;
  register Double *dFinal;
  
  if (d==0.0) {
    this->operator=(0.0);
  }
  else if (right.isZero()) {
    this->operator=(d*left);
    return;
  }
  else if (left.isZero()) {
    this->operator=((-d)*right);
    return;
  }    
  else {
    
    if (white) {
      dyadicWhite(left,right);
      val=d*(left.val-right.val);
      
      if (val==0.0&&ligSize==0&&colSize==0)
	boolZeroValue=true;
      else
	boolZeroValue=false;
      
      return;
    }    
    
    mapMerge(left,right);
    zeroInit();
    
    // Building val :
    val=d*(left.val-right.val);
    
    // Building of col :
    diter1=col+(left.colIndex-colIndex);
    diter2=left.col;
    dFinal=diter2+left.colSize;
    biter1=colBool+(left.colIndex-colIndex);
    biter2=left.colBool;
    while(diter2<dFinal) {
      *(diter1++)=d*(*(diter2++));
      *(biter1++)=*(biter2++);
    }
    diter1=col+(right.colIndex-colIndex);
    diter2=right.col;
    dFinal=diter2+right.colSize;
    biter1=colBool+(right.colIndex-colIndex);
    biter2=right.colBool;
    while(diter2<dFinal) {
      *(diter1++)-=d*(*(diter2++));
      *(biter1++)|=*(biter2++);
    }
    
    // Building of lig :
    diter1=lig+(left.ligIndex-ligIndex);
    diter2=left.lig;
    dFinal=diter2+left.ligSize;
    biter1=ligBool+(left.ligIndex-ligIndex);
    biter2=left.ligBool;
    while(diter2<dFinal) {
      *(diter1++)=d*(*(diter2++));
      *(biter1++)=*(biter2++);
    }
    diter1=lig+(right.ligIndex-ligIndex);
    diter2=right.lig;
    dFinal=diter2+right.ligSize;
    biter1=ligBool+(right.ligIndex-ligIndex);
    biter2=right.ligBool;
    while(diter2<dFinal) {
      *(diter1++)-=d*(*(diter2++));
      *(biter1++)|=*(biter2++);
    }
    
    // Building of mat :
    if (left.ligSize!=0) {
      for (i=0;i<left.colSize;i++) {
	if (left.colBool[i]) {
	  diter1=mat[i+left.colIndex-colIndex]+(left.ligIndex-ligIndex);
	  diter2=left.mat[i];
	  dFinal=diter2+left.ligSize;
	  while (diter2<dFinal)
	    *(diter1++)=d*(*(diter2++));
	}
      }
    }
    if (right.ligSize!=0) {
      for (i=0;i<right.colSize;i++) {
	if (right.colBool[i]) {
	  diter1=mat[i+right.colIndex-colIndex]+(right.ligIndex-ligIndex);
	  diter2=right.mat[i];
	  dFinal=diter2+right.ligSize;
	  while (diter2<dFinal)
	    *(diter1++)-=d*(*(diter2++));
	}
      }
    }
    
    updateBoolZeroValue();
  }
}



void scal::MultDiffMult(const scal &s1,const scal &s2,const scal &s3,
			const scal &s4) {
  static scal tmpS1,tmpS2;
  
  tmpS1.mult(s1,s2);
  tmpS2.mult(s3,s4);
  this->diff(tmpS1,tmpS2);
}



void scal::MultSumMult(const scal &s1,const scal &s2,const scal &s3,
			const scal &s4) {
  static scal tmpS1,tmpS2;
  
  tmpS1.mult(s1,s2);
  tmpS2.mult(s3,s4);
  this->sum(tmpS1,tmpS2);
}



void scal::incWhite(const scal &right) {
  // One makes the assumption that the mapping of the right operand
  // is included in this mapping.
  
  register bool *biter1,*biter2,*bFinal;
  
  biter1=colBool+(right.colIndex-colIndex);
  biter2=right.colBool;
  bFinal=biter2+right.colSize;
  while(biter2<bFinal)
    *(biter1++)|=*(biter2++);
  
  biter1=ligBool+(right.ligIndex-ligIndex);
  biter2=right.ligBool;
  bFinal=biter2+right.ligSize;
  while(biter2<bFinal)
    *(biter1++)|=*(biter2++);
  
  if (val==0.0&&ligSize==0&&colSize==0)
    boolZeroValue=true;
  else
    boolZeroValue=false;
  
  return;
}



void scal::operator+=(const scal &right) {
  static bool ok; 
  
  if (right.isZero()) {
    return;
  }

  ok=true;
  if (right.colSize) {
    ok=(colIndex<=right.colIndex
	&&(colIndex+colSize)>=(right.colIndex+right.colSize));
  }
  if (ok&&right.ligSize) {
    ok=(ligIndex<=right.ligIndex
	&&(ligIndex+ligSize)>=(right.ligIndex+right.ligSize));
  }
  
  if (ok) {
    static Int i;
    register Double *diter1,*diter2;
    register bool *biter1,*biter2;
    register Double *dFinal;
    
    if (white) {
      incWhite(right);
      
      val+=right.val;
      
      if (val==0.0&&ligSize==0&&colSize==0)
	boolZeroValue=true;
      else
	boolZeroValue=false;
      
      return;
    }
    
    // Building val :
    val+=right.val;
    
    // Building of col :
    diter1=col+(right.colIndex-colIndex);
    diter2=right.col;
    dFinal=diter2+right.colSize;
    biter1=colBool+(right.colIndex-colIndex);
    biter2=right.colBool;
    while(diter2<dFinal) {
      *(diter1++)+=*(diter2++);
      *(biter1++)|=*(biter2++);
    }
    
    // Building of lig :
    diter1=lig+(right.ligIndex-ligIndex);
    diter2=right.lig;
    dFinal=diter2+right.ligSize;
    biter1=ligBool+(right.ligIndex-ligIndex);
    biter2=right.ligBool;
    while(diter2<dFinal) {
      *(diter1++)+=*(diter2++);
      *(biter1++)|=*(biter2++);
    }
    
    // Building of mat :
    if (right.ligSize!=0) {
      for (i=0;i<right.colSize;i++) {
	if (right.colBool[i]) {
	  diter1=mat[i+right.colIndex-colIndex]+(right.ligIndex-ligIndex);
	  diter2=right.mat[i];
	  dFinal=diter2+right.ligSize;
	  while (diter2<dFinal)
	    *(diter1++)+=*(diter2++);
	}
      }
    }
    
    updateBoolZeroValue();      
  }
  else {
    static scal tmpScal;
    exchange(*this,tmpScal);
    sum(tmpScal,right);
  }
}



void scal::operator-=(const scal &right) {
  static bool ok; 
  
  if (right.isZero()) {
    return;
  }
  
  ok=true;
  if (right.colSize) {
    ok=(colIndex<=right.colIndex
	&&(colIndex+colSize)>=(right.colIndex+right.colSize));
  }
  if (ok&&right.ligSize) {
    ok=(ligIndex<=right.ligIndex
	&&(ligIndex+ligSize)>=(right.ligIndex+right.ligSize));
  }
  
  if (ok) {
    static Int i;
    register Double *diter1,*diter2;
    register bool *biter1,*biter2;
    register Double *dFinal;
    
    if (white) {
      incWhite(right);
      val-=right.val;
      
      if (val==0.0&&ligSize==0&&colSize==0)
	boolZeroValue=true;
      else
	boolZeroValue=false;   
      
      return;
    }
    
    // Building val :
    val-=right.val;
    
    // Building of col :
    diter1=col+(right.colIndex-colIndex);
    diter2=right.col;
    dFinal=diter2+right.colSize;
    biter1=colBool+(right.colIndex-colIndex);
    biter2=right.colBool;
    while(diter2<dFinal) {
      *(diter1++)-=*(diter2++);
      *(biter1++)|=*(biter2++);
    }
    
    // Building of lig :
    diter1=lig+(right.ligIndex-ligIndex);
    diter2=right.lig;
    dFinal=diter2+right.ligSize;
    biter1=ligBool+(right.ligIndex-ligIndex);
    biter2=right.ligBool;
    while(diter2<dFinal) {
      *(diter1++)-=*(diter2++);
      *(biter1++)|=*(biter2++);
    }
    
    // Building of mat :
    if (right.ligSize!=0) {
      for (i=0;i<right.colSize;i++) {
	if (right.colBool[i]) {
	  diter1=mat[i+right.colIndex-colIndex]+(right.ligIndex-ligIndex);
	  diter2=right.mat[i];
	  dFinal=diter2+right.ligSize;
	  while (diter2<dFinal)
	    *(diter1++)-=*(diter2++);
	}
      }
    }
    
    updateBoolZeroValue();
  }
  else {
    static scal tmpScal;
    exchange(*this,tmpScal);
    diff(tmpScal,right);
  }
}



void scal::operator*=(const scal &right) {
  static scal tmpScal; 

  if (isZero()||right.isZero()) {
    this->operator=(0.0);
    return;
  }

  exchange(*this,tmpScal);
  mult(tmpScal,right);
}



void scal::operator/=(const scal &right) {
  static scal tmpScal;

  if (isZero())
    return;

  exchange(*this,tmpScal);
  div(tmpScal,right);
}



void scal::operator*=(Double right) {
  Int i;
  register Double *diter1;
  register Double *dFinal;
  register Double factor;

  factor=right;
  
  if (isZero()||factor==0.0) {
    boolZeroValue=true; 
    setLigColSize(0,0);
    val=0.0;
    return;
  }
  
  if (white) {
    val*=right;
    if (right==0.0) {
      setLigColSize(0,0);
      boolZeroValue=true;
    }
    return;
  }    
  
  // Building val :
  val*=factor;
  
  // Building of col :
  diter1=col;
  dFinal=diter1+colSize;
  while (diter1<dFinal)
    *(diter1++)*=factor;
  
  // Building of lig :
  diter1=lig;
  dFinal=diter1+ligSize;
  while (diter1<dFinal)
    *(diter1++)*=factor;
  
  // Building of mat :
  if (ligSize!=0) {
    for (i=0;i<colSize;i++) {
      if (colBool[i]) {
	diter1=mat[i];
	dFinal=diter1+ligSize;
	while (diter1<dFinal)
	  *(diter1++)*=factor;
      }
    }
  }
}



void scal::operator/=(Double right) {
  if (isZero())
    return;

  (*this)*=(1.0/right);
}



scal& scal::operator-(const scal &right) const {
  static scal newScal;
  newScal.diff(*this,right);
  return(newScal);
}



scal& scal::operator/(const scal &right) const {
  static scal newScal;
  newScal.div(*this,right);
  return(newScal);
}



scal& scal::operator*(Double right) const {
  static scal newScal;
  newScal.mult(*this,right);
  return(newScal);
}



scal& scal::operator/(Double right) const {
  static scal newScal;
  newScal.div(*this,right);
  return(newScal);
}



bool scal::operator==(Double d) const {
  return(val==d);
}



bool scal::operator!=(Double d) const {
  return(val!=d);
}



bool scal::operator<=(Double d) const {
  return(val<=d);
}



bool scal::operator>=(Double d) const {
  return(val>=d);
}



bool scal::operator<(Double d) const {
  return(val<d);
}



bool scal::operator>(Double d) const {
  return(val>d);
}



void scal::functionExec(scal *arg,function1 *fParam,
			bool makeValue,bool makeDeriv) {  
  Int i;
  register Double *diter1,*diter2;
  register bool *biter1,*biter2;
  register Double *dFinal;
  register Double factor;
  
  if (white) {
    if (makeDeriv) {
      assignWhite(*arg);
      val=fParam->value(arg->val);
      
      if (val==0.0&&ligSize==0&&colSize==0)
	boolZeroValue=true;
      else
	boolZeroValue=false;   
    }
    else {
      setLigColSize(0,0);
      zeroInit();
      val=fParam->value(arg->val);
      if (val==0.0)
	boolZeroValue=true;
      else
	boolZeroValue=false;	
    }
    return;
  }
  
  if (makeDeriv) {
    setLigColSize(arg->ligSize,arg->colSize);
    zeroInit();
  }
  else {
    setLigColSize(0,0);
    zeroInit();
  }
  
  ligIndex=arg->ligIndex;
  colIndex=arg->colIndex;
  
  // Building of val :
  if (makeValue)
    val=fParam->value(arg->val);
  
  if (makeDeriv) {
    if (arg->isZero()) {
      goto endOfFunctionExec ;
    }
    else {
      factor=fParam->deriv(arg->val);
      
      // Building of col :
      biter1=colBool;
      biter2=arg->colBool;
      diter1=col;
      diter2=arg->col;
      dFinal=diter1+colSize;
      while (diter1<dFinal) {
	*(biter1++)=*(biter2++);
	*(diter1++)=factor*(*(diter2++));
      }
      
      // Building of lig :
      biter1=ligBool;
      biter2=arg->ligBool;
      diter1=lig;
      diter2=arg->lig;
      dFinal=diter1+ligSize;
      while (diter1<dFinal) {
	*(biter1++)=*(biter2++);
	*(diter1++)=factor*(*(diter2++));
      }
      
      // Building of mat :
      if (arg->ligSize!=0) {
	for (i=0;i<colSize;i++) {
	  if (colBool[i]) {
	    diter1=mat[i];
	    diter2=arg->mat[i];
	    dFinal=diter1+ligSize;
	    while (diter1<dFinal)
	      *(diter1++)=factor*(*(diter2++));
	  }
	}
      }
      // ATTENTION : IL MANQUE LA DERIVEE SECONDE DE F.
    } // if (!arg->isZero())

  } // if (makeDeriv)
  
  endOfFunctionExec :
  updateBoolZeroValue();
}



void scal::addDyadicWhite(const scal &left,const scal &right) {
  // One makes the assumption that the mappings of left and right operands
  // are contained in this mapping.
  
  register bool *biter1,*biter2,*bFinal;
  
  // Building of col :
  biter1=colBool+(left.colIndex-colIndex);
  biter2=left.colBool;
  bFinal=biter2+left.colSize;
  while(biter2<bFinal)
    *(biter1++)|=*(biter2++);
  biter1=colBool+(right.colIndex-colIndex);
  biter2=right.colBool;
  bFinal=biter2+right.colSize;
  while(biter2<bFinal)
    *(biter1++)|=*(biter2++);
  
  // Building of lig :
  biter1=ligBool+(left.ligIndex-ligIndex);
  biter2=left.ligBool;
  bFinal=biter2+left.ligSize;
  while(biter2<bFinal)
    *(biter1++)|=*(biter2++);
  biter1=ligBool+(right.ligIndex-ligIndex);
  biter2=right.ligBool;
  bFinal=biter2+right.ligSize;
  while(biter2<bFinal)
    *(biter1++)|=*(biter2++);
  
  if (val==0.0&&ligSize==0&&colSize==0)
    boolZeroValue=true;      
  else	
    boolZeroValue=false;   
}



void scal::operator+=(const SSMult &tmp) {
  static bool ok;
  static scal tmpScal;
  static Int i;
  register Double *diter1,*diter2;
  register bool *biter1,*biter2;
  register Double *dFinal;
  register Double factor;
  
  const scal &left(tmp.s1);
  const scal &right(tmp.s2);
  
  if (left.isZero()||right.isZero())
    return;
  
  tmpScal.mapMerge(*this,left,right);
  
  ok=true;
  if (tmpScal.colSize) {
    ok=(colIndex<=tmpScal.colIndex
	&&(colIndex+colSize)>=(tmpScal.colIndex+tmpScal.colSize));
  }
  if (ok&&tmpScal.ligSize) {
    ok=(ligIndex<=tmpScal.ligIndex
	&&(ligIndex+ligSize)>=(tmpScal.ligIndex+tmpScal.ligSize));
  }
  
  if (!ok) {
    exchange(*this,tmpScal);
    assignValues(tmpScal);
  }
  
  if (white) {
    addDyadicWhite(left,right);
    val+=left.val*right.val;
    
    if (val==0.0&&ligSize==0&&colSize==0)
      boolZeroValue=true;
    else
      boolZeroValue=false;
    
    return;
  }
  // Building of val :
  val+=left.val*right.val;
  
  // Building of col :
  diter1=col+(left.colIndex-colIndex);
  diter2=left.col;
  dFinal=diter2+left.colSize;
  biter1=colBool+(left.colIndex-colIndex);
  biter2=left.colBool;
  factor=right.val;
  while(diter2<dFinal) {
    *(diter1++)+=factor*(*(diter2++));
    *(biter1++)|=*(biter2++);
  }
  diter1=col+(right.colIndex-colIndex);
  diter2=right.col;
  dFinal=diter2+right.colSize;
  biter1=colBool+(right.colIndex-colIndex);
  biter2=right.colBool;
  factor=left.val;
  while(diter2<dFinal) {
    *(diter1++)+=factor*(*(diter2++));
    *(biter1++)|=*(biter2++);
  }
  
  // Building of lig :
  diter1=lig+(left.ligIndex-ligIndex);
  diter2=left.lig;
  dFinal=diter2+left.ligSize;
  biter1=ligBool+(left.ligIndex-ligIndex);
  biter2=left.ligBool;
  factor=right.val;
  while(diter2<dFinal) {
    *(diter1++)+=factor*(*(diter2++));
    *(biter1++)|=*(biter2++);
  }
  diter1=lig+(right.ligIndex-ligIndex);
  diter2=right.lig;
  dFinal=diter2+right.ligSize;
  biter1=ligBool+(right.ligIndex-ligIndex);
  biter2=right.ligBool;
  factor=left.val;
  while(diter2<dFinal) {
    *(diter1++)+=factor*(*(diter2++));
    *(biter1++)|=*(biter2++);
  }
  
  // Building of mat :
  if (left.ligSize!=0) {
    factor=right.val;
    for (i=0;i<left.colSize;i++) {
      if (left.colBool[i]) {
	diter1=mat[i+left.colIndex-colIndex]+(left.ligIndex-ligIndex);
	diter2=left.mat[i];
	dFinal=diter2+left.ligSize;
	while (diter2<dFinal)
	  *(diter1++)+=factor*(*(diter2++));
      }
    }
  }
  if (right.ligSize!=0) {
    factor=left.val;
    for (i=0;i<right.colSize;i++) {
      if (right.colBool[i]) {
	diter1=mat[i+right.colIndex-colIndex]+(right.ligIndex-ligIndex);
	diter2=right.mat[i];
	dFinal=diter2+right.ligSize;
	while (diter2<dFinal)
	  *(diter1++)+=factor*(*(diter2++));
      }
    }
  }
  if (right.ligSize!=0) {
    for (i=0;i<left.colSize;i++) {
      if (left.colBool[i]) {
	factor=left.col[i];
	diter1=mat[i+left.colIndex-colIndex]+(right.ligIndex-ligIndex);
	diter2=right.lig;
	dFinal=diter2+right.ligSize;
	while (diter2<dFinal)
	  *(diter1++)+=factor*(*(diter2++));
      }
    }
  }
  if (left.ligSize!=0) {
    for (i=0;i<right.colSize;i++) {
      if (right.colBool[i]) {
	factor=right.col[i];
	diter1=mat[i+right.colIndex-colIndex]+(left.ligIndex-ligIndex);
	diter2=left.lig;
	dFinal=diter2+left.ligSize;
	while (diter2<dFinal)
	  *(diter1++)+=factor*(*(diter2++));
      }
    }
  }
  
  updateBoolZeroValue();
}



void scal::operator-=(const SSMult &tmp) {
  static bool ok;
  static scal tmpScal;
  static Int i;
  register Double *diter1,*diter2;
  register bool *biter1,*biter2;
  register Double *dFinal;
  register Double factor;
  
  const scal &left(tmp.s1);
  const scal &right(tmp.s2);
  
  if (left.isZero()||right.isZero())
    return;
  
  tmpScal.mapMerge(*this,left,right);
  
  ok=true;
  if (tmpScal.colSize) {
    ok=(colIndex<=tmpScal.colIndex
	&&(colIndex+colSize)>=(tmpScal.colIndex+tmpScal.colSize));
  }
  if (ok&&tmpScal.ligSize) {
    ok=(ligIndex<=tmpScal.ligIndex
	&&(ligIndex+ligSize)>=(tmpScal.ligIndex+tmpScal.ligSize));
  }
  
  if (!ok) {
    exchange(*this,tmpScal);
    assignValues(tmpScal);
  }
  
  if (white) {
    addDyadicWhite(left,right);
    val-=left.val*right.val;
    
    if (val==0.0&&ligSize==0&&colSize==0)
      boolZeroValue=true;
    else
      boolZeroValue=false;
    
    return;
  }
  
  // Building of val :
  val-=left.val*right.val;
  
  // Building of col :
  diter1=col+(left.colIndex-colIndex);
  diter2=left.col;
  dFinal=diter2+left.colSize;
  biter1=colBool+(left.colIndex-colIndex);
  biter2=left.colBool;
  factor=right.val;
  while(diter2<dFinal) {
    *(diter1++)-=factor*(*(diter2++));
    *(biter1++)|=*(biter2++);
  }
  diter1=col+(right.colIndex-colIndex);
  diter2=right.col;
  dFinal=diter2+right.colSize;
  biter1=colBool+(right.colIndex-colIndex);
  biter2=right.colBool;
  factor=left.val;
  while(diter2<dFinal) {
    *(diter1++)-=factor*(*(diter2++));
    *(biter1++)|=*(biter2++);
  }
  
  // Building of lig :
  diter1=lig+(left.ligIndex-ligIndex);
  diter2=left.lig;
  dFinal=diter2+left.ligSize;
  biter1=ligBool+(left.ligIndex-ligIndex);
  biter2=left.ligBool;
  factor=right.val;
  while(diter2<dFinal) {
    *(diter1++)-=factor*(*(diter2++));
    *(biter1++)|=*(biter2++);
  }
  diter1=lig+(right.ligIndex-ligIndex);
  diter2=right.lig;
  dFinal=diter2+right.ligSize;
  biter1=ligBool+(right.ligIndex-ligIndex);
  biter2=right.ligBool;
  factor=left.val;
  while(diter2<dFinal) {
    *(diter1++)-=factor*(*(diter2++));
    *(biter1++)|=*(biter2++);
  }
  
  // Building of mat :
  if (left.ligSize!=0) {
    factor=right.val;
    for (i=0;i<left.colSize;i++) {
      if (left.colBool[i]) {
	diter1=mat[i+left.colIndex-colIndex]+(left.ligIndex-ligIndex);
	diter2=left.mat[i];
	dFinal=diter2+left.ligSize;
	while (diter2<dFinal)
	  *(diter1++)-=factor*(*(diter2++));
      }
    }
  }
  if (right.ligSize!=0) {
    factor=left.val;
    for (i=0;i<right.colSize;i++) {
      if (right.colBool[i]) {
	diter1=mat[i+right.colIndex-colIndex]+(right.ligIndex-ligIndex);
	diter2=right.mat[i];
	dFinal=diter2+right.ligSize;
	while (diter2<dFinal)
	  *(diter1++)-=factor*(*(diter2++));
      }
    }
  }
  if (right.ligSize!=0) {
    for (i=0;i<left.colSize;i++) {
      if (left.colBool[i]) {
	factor=left.col[i];
	diter1=mat[i+left.colIndex-colIndex]+(right.ligIndex-ligIndex);
	diter2=right.lig;
	dFinal=diter2+right.ligSize;
	while (diter2<dFinal)
	  *(diter1++)-=factor*(*(diter2++));
      }
    }
  }
  if (left.ligSize!=0) {
    for (i=0;i<right.colSize;i++) {
      if (right.colBool[i]) {
	factor=right.col[i];
	diter1=mat[i+right.colIndex-colIndex]+(left.ligIndex-ligIndex);
	diter2=left.lig;
	dFinal=diter2+left.ligSize;
	while (diter2<dFinal)
	  *(diter1++)-=factor*(*(diter2++));
      }
    }
  }
  
  updateBoolZeroValue();
}



void scal::stackCalculate(void) {
  static Int lIndexMin,lIndexMax,cIndexMin,cIndexMax;
  static Int i,lSize,cSize;
  static scal *tmpScal;
  static Double factor;
  
  if (stackSize==0) {
    operator=(0.0);
    return;
  }
  
  lIndexMin=mapCapacity;
  cIndexMin=mapCapacity;
  lIndexMax=0;
  cIndexMax=0;
  for (i=0;i<stackSize;i++) {
    tmpScal=sStack[i];
    if (tmpScal->ligSize) {
      lIndexMin=min(lIndexMin,tmpScal->ligIndex);
      lIndexMax=max(lIndexMax,tmpScal->ligIndex+tmpScal->ligSize);
    }
    if (tmpScal->colSize) {
      cIndexMin=min(cIndexMin,tmpScal->colIndex);
      cIndexMax=max(cIndexMax,tmpScal->colIndex+tmpScal->colSize);
    }
  }
  if (lIndexMax)
    lSize=lIndexMax-lIndexMin;
  else
    lSize=0;
  if (cIndexMax)
    cSize=cIndexMax-cIndexMin;
  else
    cSize=0;
  setLigColSize(lSize,cSize);
  ligIndex=lIndexMin;
  colIndex=cIndexMin;
  
  // First operand :
  tmpScal=sStack[0];
  factor=dStack[0];
  if (factor==1.0)
    assignValues(*tmpScal);
  else if (factor==-1.0)
    uminusAssignValues(*tmpScal);
  else
    multAssignValues(factor,*tmpScal);
  
  // Other operands :
  for (i=1;i<stackSize;i++) {
    tmpScal=sStack[i];
    factor=dStack[i];
    if (factor==1.0)
      operator+=(*tmpScal);
    else if (factor==-1.0)
      operator-=(*tmpScal);
    else
      multAddValues(factor,*tmpScal);
  }
}



