
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <fstream>
#include <iomanip>

#include "test.h"
#include "array/Array.h"
#include "array/array_base.h"
#include "domain/Domain.h"
#include "field/Field.h"
#include "mesh/Mesh.h"
#include "param/param.h"
#include "param/paramList.h"
#include "scal/scal.h"
#include "step/step.h"

using namespace std ;
using namespace keyFE2 ;



namespace keyFE2 {
  
  void arrayComp_test1(void) {
    int k,l;
    Mesh MESH;
    Domain dom;
    
    MESH.read("parallelogramme.msh","rsMesh");
    dom=MESH.getDefDomain();
    dom.setName("the Domain");
    
    Field coord((DOM=dom,INTERPOLATION="linear",NBRCOMPONENTS=2,
		 COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		 NAME="coordinates",INTERPMESH=MESH));
    
    Array ACoord((TYPE="arrayField",ORDER=1,DIM(0)=2,
		  NAME="Champ de coordonnes",DERIV=false,
		  COMPONENT(0)=coord.component(0),
		  COMPONENT(1)=coord.component(1)));
    Array Metrix((TYPE="arrayComp",ORDER=2,DIM(0)=3,DIM(1)=3,
		  COMPONENT(0,0)=Double(1.0),COMPONENT(1,1)=ACoord.component(0),
		  COMPONENT(2,2)=Double(1.0)));
    Metrix.write();
    
    Object unic=dom.getObject(0);
    
    Double coordpoint[3]={0.0,0.0,0.0};
    
    scal *s1,s2;
    Double r=1/sqrt(3.0);
    array *tmpArray;
    
    step tmpStep(Metrix.getData(),unic.getData(),unic.getDirect());
    tmpStep.buildActiveFieldList();
    
    for (l=0;l<0;l++) {
      s2=0.0;
      for (k=0;k<9;k++) { 
	switch(k/3) {
	case 0 : coordpoint[0]=-r;
	  break;
	case 1 : coordpoint[0]=0;
	  break;
	case 2 : coordpoint[0]=r;
	  break;
	}
	switch(k%3) {
	case 0 : coordpoint[1]=-r;
	  break;
	case 1 : coordpoint[1]=0;
	  break;
	case 2 : coordpoint[1]=r;
	  break;
	}
	
	tmpStep.exec(coordpoint);
	
	tmpArray=Metrix.getData();
	s1=tmpArray->getComponentValues();
	s2+=(*s1);
      }
    }
    s2.writeMat(cerr);
    cerr << endl << endl ;
  }
  
  
  
  void arrayComp_test2(void) {
    array *tmpArray;
    
    int k,l;
    Mesh MESH;
    Domain dom;
    
    MESH.read("parallelogramme.msh","rsMesh");
    dom=MESH.getDefDomain();
    dom.setName("the Domain");
    
    Field coord((DOM=dom,INTERPOLATION="linear",NBRCOMPONENTS=2,
		 COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		 NAME="coordinates",INTERPMESH=MESH));
    
    Array ACoord((TYPE="arrayField",ORDER=1,DIM(0)=2,
		  NAME="Champ de coordonnes",DERIV=false,
		  COMPONENT(0)=coord.component(0),
		  COMPONENT(1)=coord.component(1)));
    Array X((TYPE="arrayComp",COMPONENT=ACoord.component(0)));
    Array Metrix((TYPE="arrayComp",ORDER=2,DIM(0)=3,DIM(1)=3,
		  COMPONENT(0,0)=Double(1.0),COMPONENT(1,1)=X.component(),
		  COMPONENT(2,2)=Double(1.0)));
    
    Object unic=dom.getObject(0);
    
    Double coordpoint[3]={0.0,0.0,0.0};
    
    scal *s1,s2;
    Double r=1/sqrt(3.0);
    
    step tmpStep(Metrix.getData(),unic.getData(),unic.getDirect());
    tmpStep.buildActiveFieldList();
    
    for (l=0;l<1;l++) {
      s2=0.0;
      for (k=0;k<9;k++) { 
	switch(k/3) {
	case 0 : coordpoint[0]=-r;
	  break;
	case 1 : coordpoint[0]=0;
	  break;
	case 2 : coordpoint[0]=r;
	  break;
	}
	switch(k%3) {
	case 0 : coordpoint[1]=-r;
	  break;
	case 1 : coordpoint[1]=0;
	  break;
	case 2 : coordpoint[1]=r;
	  break;
	}
	
	tmpStep.exec(coordpoint);
	
	tmpArray=Metrix.getData();
	s1=tmpArray->getComponentValues();
	s2+=(*s1);
      }
    }
    //  s2.writeMat(cerr);
    //  cerr << endl << endl ;
  }
  
}  
  
