
//
//  Copyright 2004 Renaud Sizaire
//
//  This file is part of keyFE2.
//
//  keyFE2 is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  keyFE2 is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with keyFE2; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//

#include <fstream>
#include <iomanip>

#include "test.h"
#include "array/Array.h"
#include "array/array_base.h"
#include "array/ArrayOperators.h"
#include "domain/Domain.h"
#include "field/Field.h"
#include "mesh/Mesh.h"
#include "param/paramList.h"
#include "scal/scal.h"
#include "step/step.h"

using namespace std ;
using namespace keyFE2 ;



namespace keyFE2 {
  
  void corlocDeriv_test1(void) {
    Mesh MESH;
    Domain dom;
    
    MESH.read("oneQuad.msh","rsMesh");
    dom=MESH.getDefDomain();
    dom.setName("the domain");
    
    Field coord((DOM=dom,INTERPOLATION="linear",NBRCOMPONENTS=2,
		 COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		 NAME="coordinates",INTERPMESH=MESH));
    Field velo((DOM=dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		NAME="velocities"));
    velo.interpolate(coord,dom,coord);
    
    Array A1((TYPE="arrayField",ORDER=1,DIM(0)=2,NAME="Champ de coordonnes",
	      DERIV=false,COMPONENT(0)=coord.component(0),
	      COMPONENT(1)=coord.component(1)));
    Array A2((TYPE="arrayField",ORDER=1,DIM(0)=2,NAME="Champ de vitesses",
	      COMPONENT(0)=velo.component(0),COMPONENT(1)=velo.component(1)));
    Array A3(A1),A4;
    
    A4=exp(A2);
    A4=exp(3.0*A2/2.0);
    A4.write();  
    A2=makeCorlocDeriv(A4,3);
    A2.write();
    
    Object unic=dom.getObject(0);
    
    Double coordpoint[3]={0.7,0.2,0};
    
    step tmpStep(A2.getData(),unic.getData(),unic.getDirect());
    tmpStep.buildActiveFieldList();
    int step=7;
    for (int i=-10;i<=10;i+=step) {
      for (int j=-10;j<=10;j+=step) {
	coordpoint[0]=0.1*i;
	coordpoint[1]=0.1*j;
	tmpStep.exec(coordpoint);
      }
    }
  }
  
  
  
  void corlocDeriv_test2(void) {
    Mesh MESH;
    Domain dom;
    
    MESH.read("oneQuad.msh","rsMesh");
    dom=MESH.getDefDomain();
    dom.setName("the domain");
    
    Field coord((DOM=dom,INTERPOLATION="linear",NBRCOMPONENTS=2,
		 COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		 NAME="coordinates",INTERPMESH=MESH));
    Field velo((DOM=dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		NAME="velocities"));
    velo.interpolate(coord,dom,coord);
    
    Array A1((TYPE="arrayField",ORDER=1,DIM(0)=2,NAME="Champ de vitesses",
	      DERIV=false,COMPONENT(0)=velo.component(0),
	      COMPONENT(1)=velo.component(1)));
    Array A2=makeCorlocDeriv(A1,2);
    
    A2.write();
    
    Object unic=dom.getObject(0);
    
    Double coordpoint[3]={0.7,0.2,0};
    
    //  ofstream outfile("corlocDeriv2.lst");
    //  outfile.setf(ios::scientific);
    //  outfile << setprecision(7);
    scal *s1,*s2;
    array *tmpArray;
    int istep=7;
    step tmpStep(A2.getData(),unic.getData(),unic.getDirect());
    tmpStep.buildActiveFieldList();
    for (int i=-10;i<=10;i+=istep) {
      for (int j=-10;j<=10;j+=istep) {
	coordpoint[0]=0.1*i;
	coordpoint[1]=0.1*j;
	tmpStep.exec(coordpoint);
	tmpArray=A2.getData();
	s1=tmpArray->getComponentValues();
	s2=s1+1;
	//      outfile << setw(16) << coordpoint[0] << setw(16) << coordpoint[1] ;
	//      outfile << setw(16) << s1->getVal() << setw(16) << s2->getVal() ;     
	//      s2->writeLig(outfile);
      }
      //    outfile << endl ;
    }
    //  outfile.close();
  }
  
  
  
  void corlocDeriv_test3(void) {
    Mesh MESH;
    Domain dom;
    
    MESH.read("oneQuad.msh","rsMesh");
    dom=MESH.getDefDomain();
    dom.setName("the domain");
    
    Field coord((DOM=dom,INTERPOLATION="linear",NBRCOMPONENTS=2,
		 COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		 NAME="coordinates",INTERPMESH=MESH));
    Field velo((DOM=dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		NAME="velocities"));
    velo.interpolate(coord,dom,coord);
    
    Array A1((TYPE="arrayField",ORDER=1,DIM(0)=2,NAME="Champ de vitesses",
	      DERIV=false,COMPONENT(0)=velo.component(0),
	      COMPONENT(1)=velo.component(1)));
    Array A3=7.0*exp(A1/2.0);
    Array A2=makeCorlocDeriv(A3,2);
    
    Object unic=dom.getObject(0);
    
    Double coordpoint[3]={0.7,0.2,0};
    
    //  ofstream outfile("corlocDeriv3.lst");
    //  outfile.setf(ios::scientific);
    //  outfile << setprecision(7);
    scal *s1,*s2;
    array *tmpArray;
    int istep=7;
    step tmpStep(A2.getData(),unic.getData(),unic.getDirect());
    tmpStep.buildActiveFieldList();
    for (int i=-10;i<=10;i+=istep) {
      for (int j=-10;j<=10;j+=istep) {
	coordpoint[0]=0.1*i;
	coordpoint[1]=0.1*j;
	tmpStep.exec(coordpoint);
	tmpArray=A2.getData();
	s1=tmpArray->getComponentValues();
	s2=s1+1;
	//      outfile << setw(16) << coordpoint[0] << setw(16) << coordpoint[1] ;
	//      outfile << setw(16) << s1->getVal() << setw(16) << s2->getVal() ;
	//      s1->writeLig(outfile);
      }
      //    outfile << endl ;
    }
    //  outfile.close();
  }
  
  
  
  void corlocDeriv_test4(void) {
    Mesh MESH;
    Domain dom;
    
    MESH.read("oneQuad.msh","rsMesh");
    dom=MESH.getDefDomain();
    dom.setName("the domain");
    
    Field coord((DOM=dom,INTERPOLATION="linear",NBRCOMPONENTS=2,
		 COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		 NAME="coordinates",INTERPMESH=MESH));
    Field velo((DOM=dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		NAME="velocities"));
    
    velo.interpolate(coord,dom,coord);
    
    Array A1((TYPE="arrayField",ORDER=1,DIM(0)=2,NAME="Champ de vitesses",
	      DERIV=false,COMPONENT(0)=velo.component(0),
	      COMPONENT(1)=velo.component(1)));
    Array A2=7.0*exp(A1/2.0);
    
    A2.write();
    
    Object unic=dom.getObject(0);
    
    Double coordpoint[3]={0.7,0.2,0};
    
    //  ofstream outfile("corlocDeriv4.lst");
    //  outfile.setf(ios::scientific);
    //  outfile << setprecision(7);
    scal *s1,*s2;
    array *tmpArray;
    int istep=7;
    step tmpStep(A2.getData(),unic.getData(),unic.getDirect());
    tmpStep.buildActiveFieldList();
    for (int i=-10;i<=10;i+=istep) {
      for (int j=-10;j<=10;j+=istep) {
	coordpoint[0]=0.1*i;
	coordpoint[1]=0.1*j;
	tmpStep.exec(coordpoint);
	tmpArray=A2.getData();
	s1=tmpArray->getComponentValues();
	s2=s1+1;
	//      outfile << setw(16) << coordpoint[0] << setw(16) << coordpoint[1] ;
	//      outfile << setw(16) << s1->getVal() << setw(16) << s2->getVal() ;
	//      s1->writeLig(outfile);
      }
      //    outfile << endl ;
    }
    //  outfile.close();
  }
  
  
  
  void corlocDeriv_test5(void) {
    Mesh MESH;
    Domain dom;
    
    MESH.read("oneQuad.msh","rsMesh");
    dom=MESH.getDefDomain();
    dom.setName("the domain");
    
    Field coord((DOM=dom,INTERPOLATION="linear",NBRCOMPONENTS=2,
		 COMPONENTNAME(0)="X",COMPONENTNAME(1)="Y",
		 NAME="coordinates",INTERPMESH=MESH));
    Field velo((DOM=dom,INTERPOLATION="quadratic",NBRCOMPONENTS=2,
		COMPONENTNAME(0)="u",COMPONENTNAME(1)="v",
		NAME="velocities"));
    
    velo.interpolate(coord,dom,coord);
    
    Array A1((TYPE="arrayField",ORDER=1,DIM(0)=2,NAME="Champ de vitesses",
	      DERIV=false,COMPONENT(0)=velo.component(0),
	      COMPONENT(1)=velo.component(1)));
    Array A3=7.0*exp(A1/2.0);
    Array A2=makeCorlocDeriv(A3,2);
    
    A2.write();
    
    Object unic=dom.getObject(0);
    Double coordpoint[3]={0.7,0.2,0};
    int istep=7;
    step tmpStep(A2.getData(),unic.getData(),unic.getDirect());
    tmpStep.buildActiveFieldList();
    for (int i=-10;i<=10;i+=istep) {
      for (int j=-10;j<=10;j+=istep) {
	coordpoint[0]=0.1*i;
	coordpoint[1]=0.1*j;
	tmpStep.exec(coordpoint);
      }
    }
  }
  
}




